# -*- coding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
# #############################################################################

from openerp.osv import osv
from SigUtils import sigerp, cnpj
from openerp.tools.translate import _
import logging
from openerp import SUPERUSER_ID
from unicodedata import normalize
from .....web_sig.control.utils import websig_get_my_ip
import xmlrpclib, socket
from re import sub
from openerp.tools import config as openerp_config
import requests
import json


class valida_pendencias(osv.Model):
    _inherit = 'valida.pendencias'
    _logger = logging.getLogger(_inherit)

    def valpen_validar_erp(self, cr, uid, ids, context=None):
        """
        Método executado pelo Cron para comunicar com a Sig e pegar uma nova licenca
        @param cr:
        @param uid:
        @param ids:
        @param context:
        @return: True ou False dependendo se conseguiu ou não
        """
        context = context and context.copy() or {}
        super(valida_pendencias, self).valpen_validar_erp(cr, uid, ids, context)
        if self.valpen_verifica_rede_sig(cr, uid, context):
            return True

        self._logger.info(_('Validando o SigERP'))
        _company_dados = self.pool.get('res.company').browse(cr, uid, 1, context=context)
        if _company_dados.rescon_pessoa_id.pessoa_tipo == 'F':
            cnpj_cadastro = sub('[^0-9]', '', _company_dados.rescon_pessoa_id.cpf_cnpj)
        else:
            res = cnpj.Cnpj(_company_dados.rescon_pessoa_id.cpf_cnpj).remove_edicao
            cnpj_cadastro = res.get('cnpj', '')
        if not cnpj_cadastro:
            self._logger.info(_('Empresa(pessoa nro %s) sem cnpj cadastrado!') %
                              _company_dados.rescon_pessoa_id.codigo)
            return False
        servidor = self.pool.get('ir.config_parameter').get_sig_server(cr, uid).split(',')
        self._logger.info(_('Validando para o CNPJ: %s Servidor: %s') % (cnpj_cadastro, servidor))
        objOcorrencias = self.pool.get('adm.monitor.ocorrencias')
        objAlertas = self.pool.get('adm.monitor.alertas')
        _alerta_id = objAlertas.search(cr, uid, [('name', '=', 'Validar SigERP')], context=context)
        if _alerta_id:
            _ocorrencias_ids = objOcorrencias.search(cr, uid,
                                                     [('admmoc_alerta_id', '=', _alerta_id[0])], context=context)
            if _ocorrencias_ids:
                objOcorrencias.unlink(cr, uid, _ocorrencias_ids)
                cr.commit()
        try:
            host = servidor[0]
            db = servidor[1]
            porta = servidor[2]
            user = servidor[3]
            senha = servidor[4]
            ip_cliente = websig_get_my_ip()
            self._logger.info(_('Vou pedir geracao de licensa cnpj: %s IP: %s') % (cnpj_cadastro, ip_cliente))
            res = requests.post('https://%s/valida/controller_gerar_licensa' % host,
                                auth=(user, senha),
                                data={'db': db,
                                      'pwd': senha,
                                      'user': user,
                                      'url': host,
                                      'port': porta,
                                      'dados': str({'cnpj': cnpj_cadastro, 'ip': ip_cliente})
                                      },
                                timeout=10000)
            res = json.loads(res.content)
            if res.get('codigo') != '00':
                self._logger.info('SigERP nao validado! Codigo do erro: %s Erro:%s' % (res.get('codigo'),
                                                                                       res.get('mensagem')))
            licenca = res.get('retorno')[0]

            if not licenca:
                raise Exception('Licenca nao gerada no ERP da SIG')

            self.pool.get('ir.config_parameter').set_param(cr, uid, 'licensa_sig_erp', licenca)
            self._logger.info(
                _('SigERP validado! Licenca:%s') % str(sigerp._crypto_sig('d', licenca)))
        except Exception as e:
            if isinstance(e, object) and not isinstance(e, ValueError) and 'faultCode' in e.__dict__:
                e = e.faultCode or ''
            else:
                e = str(e)
            try:
                string = normalize('NFKD', e.decode('utf-8')).encode('ASCII', 'ignore')
            except:
                string = normalize('NFKD', e).encode('ASCII', 'ignore')
            self._logger.info(('SigERP nao validado! Erro:%s') % string)
            vals = {
                'admmoc_alerta_id': _alerta_id[0],
                'company_id': 1,
                'name': _('Nao consegui validar o SigERP.')
            }
            objOcorrencias.create(cr, uid, vals, context=context)
            # Não conseguindo a conecção, remover o servidor da configuração para que possar ser gerado novamente,
            # talvez ele esteja errado
            server_id = self.pool.get('ir.config_parameter').search(cr, SUPERUSER_ID,
                                                                    [('key', '=', 'server_sig')], context=context)
            self.pool.get('ir.config_parameter').unlink(cr, SUPERUSER_ID, server_id)

        return True
