# -*- coding: utf-8 -*-
#
# OBS: para funcionar eh preciso que tenha a pasta 'static' dentro do modulo
import traceback

import openerp.addons.web.http as http
import json
import psycopg2
import xmlrpclib
from datetime import datetime
from base64 import b64decode
from ..valida.controller import Config, MySock as Ms
from openerp.modules.module import load_information_from_description_file as info
from SigUtils.cpf import Cpf
from SigUtils.cnpj import Cnpj
from openerp.osv.fields import float_round

import time
import subprocess
from tempfile import NamedTemporaryFile
from logging import getLogger
_logger = getLogger("ColetorAndroid")

SEM_ERRO = '00'
ERRO_CONEXAO = '01'
USUARIO_SENHA = '02'
ERRO_CNPJ = '03'
ERRO_LICENCA = '04'
ITEM_NAO_CADASTRADO = '05'
ITEM_NAO_CADASTRADO_LOCAL = '06'
LEITURA_DE_PRODUTO = '07'
SEM_PARAMETRO_ESTOQUE = '08'
CB_NAO_CADASTRADO = '09'
LOCAIS_NAO_CADASTRADOS = '10'
MODELOS_ETIQUETA_NAO_CADASTRADOS = '15'
IMPRESSORAS_NAO_CADASTRADAS = '20'
ERRO_IMPRESSAO = '21'
SEM_INVENTARIO = '25'
ITEM_NAO_CADASTRADO_INVENTARIO = '26'
MODULO_RESSUPRIMENTO_NAO_INSTALADO = '30'
SEM_REQUISICOES = '40'
REQUISICAO_NAO_CRIADA = '41'
ITEM_REQUISICAO_NAO_CRIADO = '42'
ID_REQUISICAO_INCORRETO = '43'
ERRO_ALTERAR_REQUISICAO = '44'
ERRO_AO_SEPARAR_ITEM = '45'
ITEM_NAO_CADASTRADO_REQUISICAO = '46'
ITEM_NAO_REQUERIDO = '47'
ID_PEDIDO_INCORRETO = '50'
ERRO_ALTERAR_PEDIDO = '51'
ERRO_CPF_INVALIDO = '60'
CLIENTE_NAO_CADASTRADO = '61'
PRODUTO_SEM_PRECO = '70'
SEM_LOCAL_PADRAO = '71'

ERRO_INTERNO = '99'
TITULOS = ['AVENIDA', 'RUA', 'PRAÇA', 'PRACA', 'TRAVESSA', 'RODOVIA',
           'SERRA', 'VIA', 'VIADUTO', 'SETOR', 'VILA']


def dictfetchall(_cr):
    """ fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
     para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
     conversao."""

    # testa resultados armazenados
    results = _cr.fetchall()
    if results < 1:
        return results

    # substitui resultados das listas por dicionarios
    rownum = 0
    for row in results:
        dictrow = {}
        dictnum = 0
        for col in _cr.description:
            dictrow[col[0]] = row[dictnum]
            dictnum += 1

        results[rownum] = dictrow
        rownum += 1

    return results


class MySock(Ms):
    # faz heranca do MySock do modulo 'valida'
    def __init__(self, params, modulo=''):
        super(MySock, self).__init__(params, modulo or 'coletor_android')


class sc_coletor(http.Controller):
    _cp_path = '/coletor'

    @http.httprequest
    def teste(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path,
                                                                     req.params,
                                                                     req.httprequest.data))
        ip = req.httprequest.environ.get('REMOTE_ADDR')
        print(ip)
        return json.dumps({'codigo': SEM_ERRO,
                           'mensagem': 'ok',
                           'retorno': [ip]})

    def _get_cursor(self, params):
        try:
            cnpj = params.get('cnpj')
            config = Config('coletor_android')
            try:
                user = config.dados.get(cnpj, 'username')
            except Exception as e:
                if 'No section:' in e.message:
                    raise Exception('O CNPJ "%s" não está cadastrado no arquivo de configuração'
                                    ' "app.config"' % str(cnpj))
                else:
                    raise e
            pwd = config.dados.get(cnpj, 'password')
            if pwd == "''":
                pwd = ''
            host = config.dados.get(cnpj, 'hostname')
            port = config.dados.get(cnpj, 'port')
            db = config.dados.get(cnpj, 'database')
            conn = psycopg2.connect(database=db, password=pwd, user=user, host=host,
                                    port=port, connect_timeout=4)
            cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            return cr, False
        except Exception as e:
            return False, json.dumps({'codigo': ERRO_CONEXAO,
                                      'mensagem': 'Erro: %s' % e,
                                      'retorno': [],
                                      })

    @http.httprequest
    def set_log(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        log = '%s: %s\n' % (datetime.now(), str(req.params.get('log') or ''))
        open('/tmp/log.txt', 'a').write(log)
        return json.dumps({'codigo': SEM_ERRO,
                           'mensagem': 'ok',
                           'retorno': []})

    @http.httprequest
    def check_licenca(self, req, s_action=None, **kw):
        try:
            _logger.info(
                '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
            sock = MySock(req.params)
            vals = {'serial': req.params.get('serial'),
                    'model': req.params.get('model'),
                    'brand': req.params.get('brand'),
                    'cnpj': req.params.get('cnpj'),
                    }
            if not req.params.get('cnpj'):
                return json.dumps({'codigo': ERRO_CNPJ,
                                   'mensagem': 'CNPJ não informado!',
                                   'retorno': []})

            dispositivo_ids = sock.execute('coletor.dispositivo',
                                           'coldis_cadastrar_dispositivo', vals)
            res, msg = sock.execute('coletor.dispositivo',
                                    'coldis_check_licenca', vals, dispositivo_ids)
            if not res:
                return json.dumps({'codigo': ERRO_LICENCA,
                                   'mensagem': msg,
                                   'retorno': []})

            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': 'ok',
                               'retorno': []})

        except Exception as e:
            if type(e) == xmlrpclib.Fault:
                msg_erro = e.faultString
                if e.faultCode == 'AccessDenied':
                    msg_erro = 'Erro: %s' % "Acesso ao ERP negado para este Usuário e Senha!"
                elif '9999\n' in e.faultCode:
                    msg_erro = e.faultCode.split('9999\n')[1]
                return json.dumps({'codigo': ERRO_INTERNO,
                                   'mensagem': msg_erro,
                                   'retorno': []})

            msg_erro = ''
            if 'message' in dir(e):
                msg_erro = e.message
            else:
                msg_erro = str(e)
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % msg_erro,
                               'retorno': []})

    @http.httprequest
    def logar(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        versao_requerida = '1.0.7'
        if not req.params.get('version') or req.params.get('version') < versao_requerida:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Versão do Aplicativo menor que "%s"'
                                           '\nAtualize o Aplicativo!' % versao_requerida,
                               'retorno': []})

        versao_erp = req.params.get('version_erp')
        versao_atual_erp = info('adm').get('version')
        if versao_erp and versao_atual_erp < versao_erp:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Atualize o ERP para versão: "%s" ou maior!\n'
                                           'Versão atual: "%s"' % (str(versao_erp), versao_atual_erp),
                               'retorno': []})

        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            login = req.params.get('username')
            senha = req.params.get('password')
            cr.execute("""SELECT uc.usuario_id,
                                 ru.password AS pwd,
                                 mc.name     AS nome,
                                 mc.fonte,
                                 mc.sequence AS sequencia,
                                 mc.caminho
                          FROM usuario_coletor     AS uc
                          LEFT JOIN usuario_id     AS us ON us.usuario_id = uc.id  -- tabela many2many 
                          LEFT JOIN modulo_coletor AS mc ON mc.id =  us.modulo_id
                          LEFT JOIN res_users      AS ru ON ru.id = uc.usuario_id
                          WHERE login_usuario = '{login}'
                            AND senha_coletor = '{senha}'
                          ORDER BY mc.sequence""".format(login=login, senha=senha))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'codigo': USUARIO_SENHA,
                                   'mensagem': 'Login ou senha incorretos!',
                                   'retorno': []})

            vals = {}
            for i in dados:
                modulo = {'nome': i.get('nome'),
                          'sequencia': i.get('sequencia'),
                          'arquivo': i.get('fonte')}
                if not vals.get('id'):
                    vals = {'id': i.get('usuario_id'),
                            'company_id': 0,
                            'modulos': [modulo],
                            'pwd': i.get('pwd'),
                            }
                else:
                    vals['modulos'].append(modulo)

            cr.execute("""SELECT rc.id,
                                 rc.name AS nome
                          FROM res_company_users_rel AS rel
                          INNER JOIN res_company     AS rc ON rc.id = rel.cid
                          WHERE user_id = {id}     
                          ORDER BY rc.id             
                       """.format(id=vals.get('id')))
            vals['empresas'] = dictfetchall(cr)
            cr.close()
            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_locais(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        cr, erro = self._get_cursor(req.params)
        if not cr:
            return erro

        try:
            company_id = int(req.params.get('loc_company_id') or 0) or int(req.params.get('company_id') or 0)
            sock = MySock(req.params)
            company_id = sock.execute('adm.empresa.objeto', 'getEmpresa', 'estoque.local', False, False, company_id)
            local_ids = sock.execute('estoque.local', 'search', [('company_id', '=', company_id)], 0, 0, '',
                                     {'sig_force_company': True})
            cr.execute("""SELECT id, 
                                 numero,
                                 name AS nome
                          FROM estoque_local
                          WHERE id IN ({ids})
                          ORDER BY numero
                       """.format(ids=','.join(map(lambda x: '%s' % x, local_ids))))
            dados = dictfetchall(cr)
            if not dados:
                cr.close()
                return json.dumps({'codigo': LOCAIS_NAO_CADASTRADOS,
                                   'mensagem': 'Nenhum local cadastrado para esta empresa!',
                                   'retorno': []})

            cr.close()
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            cr.close()
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    def _get_item(self, req):
        try:
            local_id = int(req.params.get('local_id') or 0)
            locais_ids = eval(req.params.get('locais_ids') or '[]')
            codigo = req.params.get('codigo_barra')
            codigo_barras = codigo.zfill(14)
            sock = MySock(req.params)
            par_id = sock.execute('estoque.parametro', 'search', [])
            if not par_id:
                return json.dumps({'codigo': SEM_PARAMETRO_ESTOQUE,
                                   'mensagem': 'Sem parâmetro de Estoque cadastrado!',
                                   'retorno': []})

            vals_balanca = {}
            if codigo.startswith('2') and len(codigo) == 13:
                # consulta produto de balanca
                res = sock.execute('coletor.dispositivo', 'coldis_busca_produto_balanca', codigo)
                if res.get('erro'):
                    return json.dumps({'codigo': ERRO_INTERNO,
                                       'mensagem': res.get('erro'),
                                       'retorno': []})

                automacao_codigo_id = [res.get('ret').get('cb_id')]
                vals_balanca = {'quantidade': res.get('ret').get('qtd'),
                                'vr_total': res.get('ret').get('vr_total'),
                                'vr_unitario': res.get('ret').get('vr_unitario')}
            else:
                args = [('codigo_barra', '=', codigo_barras)]
                automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)

            produto_id = False
            tam_codigo = sock.execute('estoque.parametro', 'read', par_id[0],
                                      ['estpar_tamanho_codigo']).get('estpar_tamanho_codigo')
            consulta_por_cb = True
            if not automacao_codigo_id:
                consulta_por_cb = False
                args = [('codigo', '=', codigo.zfill(tam_codigo or 0))]
                produto_id = sock.execute('estoque.produto', 'search', args)
                if not produto_id:
                    return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                       'mensagem': 'Produto não cadastrado!',
                                       'retorno': []})

                produto_id = produto_id[0]
                args = [('produto_id', '=', produto_id)]
                automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
                if not automacao_codigo_id:
                    return json.dumps({'codigo': CB_NAO_CADASTRADO,
                                       'mensagem': 'Produto sem Código de Barras cadastrado!',
                                       'retorno': []})

            campos = ['produto_id', 'grade_id', 'produto_embalagem_id', 'codigo_barra']
            dados_automacao = sock.execute('automacao.codigo.barra', 'read',
                                           int(automacao_codigo_id[0]), campos)
            if not local_id:
                local_id = sock.execute('estoque.parametro', 'estpar_get_fields',
                                        'local_venda_padrao_id', {})[0].get('local_venda_padrao_id')

            codigo_barras = dados_automacao.get('codigo_barra')
            produto_id = produto_id or dados_automacao.get('produto_id')[0]
            codigo = sock.execute('estoque.produto', 'read', produto_id, ['codigo']).get('codigo')
            grade_id = int(dados_automacao.get('grade_id')[0])
            args = [('local_id', '=', local_id), ('grade_id', '=', grade_id),
                    ('produto_id', '=', produto_id)]
            estoque_local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)
            datas_E_S = sock.execute('wizard.listagem.geral.item', 'get_datas_movimento', [grade_id],
                                     {'retorna_string': True})
            data_entrada = ''
            if datas_E_S.get(str(grade_id)):
                data_entrada = datas_E_S.get(str(grade_id)).get('dt_entrada') or ''
            if data_entrada:
                data_entrada = data_entrada.split('-')
                data_entrada = '%s/%s/%s' % (data_entrada[2], data_entrada[1], data_entrada[0])
            else:
                data_entrada = '-'
            if not estoque_local_locacao_id:
                # cr.close()
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_LOCAL,
                                   'mensagem': 'Produto não cadastrado neste local!',
                                   'retorno': []})

            dados_locacao = sock.execute('estoque.local.locacao', 'read',
                                   estoque_local_locacao_id[0], ['locacao', 'estllc_lastro'])
            lastro = dados_locacao.get('estllc_lastro') or 0
            locacao = dados_locacao.get('locacao') or ''
            campos = ['name', 'grade']
            dados_grade = sock.execute('estoque.produto.grade', 'read', grade_id, campos)
            prod_embalagem_id = dados_automacao.get('produto_embalagem_id')[0]
            if isinstance(prod_embalagem_id, list):
                prod_embalagem_id = prod_embalagem_id[0]
            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read',
                                                int(prod_embalagem_id),
                                                ['embalagem_id'])
            embalagem_id = int(dados_prod_embalagem.get('embalagem_id')[0])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                               embalagem_id,
                                               ['sigla', 'quantidade_na_embalagem', 'unidade_id'],
                                               {}, '_classic_write')
            fracao = sock.execute('estoque.embalagem.unidade', 'read', dados_emb_embalagem.get('unidade_id'),
                         ['fracao']).get('fracao') or False
            descricao = dados_grade.get('name')
            eps_grade_id = int(dados_automacao.get('grade_id')[0])
            eps_data = datetime.now().strftime('%Y-%m-%d')
            eps_local_id = local_id
            saldos = {}
            if not locais_ids:
                locais_ids.append(eps_local_id)

            for local_id in sorted(locais_ids):
                saldo = sock.execute('estoque.produto.saldo', 'get_saldo_do_dia',
                                     eps_grade_id, eps_data, local_id, False, {'coletor_estoque_saldo': True})
                saldo = saldo.get(str(eps_grade_id)).get('disponivel')
                saldo = float(saldo) / float(dados_emb_embalagem.get('quantidade_na_embalagem'))
                saldos.update({local_id: str(saldo).replace('.', ',')})

            grade_sigla = str(dados_grade.get('grade'))
            produto = sock.execute('estoque.produto', 'read', produto_id, ['preco_venda', 'estpro_foto'])
            preco = '0.0'
            foto = ''
            if produto and isinstance(produto, list):
                produto = produto[0]
            if produto and isinstance(produto, dict):
                preco = produto.get('preco_venda')
                foto = produto.get('estpro_foto') or ''

            preco = str(preco)
            preco = preco.replace('.', ',')
            if preco[len(preco) - 2:len(preco) - 1] == ',':
                preco += '0'

            vals = {'id': produto_id,
                    'foto': foto,
                    'nome': descricao,
                    'cb': codigo_barras,
                    'codigo': codigo,
                    'embalagem_id': prod_embalagem_id,
                    'qtd_embalagem': dados_emb_embalagem.get('quantidade_na_embalagem') or 1.0,
                    'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                    'saldo': saldos.get(eps_local_id),
                    'lastro': lastro,
                    'saldos': saldos,
                    'grade_id': grade_id,
                    'sigla_grade': grade_sigla,
                    'fracao': fracao,
                    'preco': preco,
                    'data_entrada': data_entrada,
                    'locacao': locacao}
            if vals_balanca:
                vals['preco'] = vals_balanca.pop('vr_unitario')
                vals.update(vals_balanca)

            if consulta_por_cb:
                res = sock.execute('estoque.produto.grade', 'read', grade_id, ['descricao', 'name'])
                vals['grades'] = [res]
            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            msg_erro = e
            try:
                if '9999\n' in e.faultCode:
                    msg_erro = e.faultCode.split('9999\n')[1]
            except:
                pass

            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % msg_erro,
                               'retorno': []})

    @http.httprequest
    def get_item(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            return self._get_item(req)
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Funcoes de etiqueta de gondola  ---------------------------------------
    @http.httprequest
    def get_modelos(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            modelo_ids = sock.execute('automacao.modelo.etiqueta', 'search', [])
            if not modelo_ids:
                return json.dumps({'codigo': MODELOS_ETIQUETA_NAO_CADASTRADOS,
                                   'mensagem': 'Nenhum modelo de etiqueta cadastrado!',
                                   'retorno': []})

            modelos = sock.execute('automacao.modelo.etiqueta', 'read', modelo_ids)
            dados = []
            cont = 1
            for i in modelos:
                dados.append({'nome': i.get('name'),
                              'numero': cont,
                              'arquivo': b64decode(i.get('autmet_arquivo') or '')
                              })
                cont += 1

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_impressoras(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            args = [('admimp_user_id', 'in', [sock.uid, False])]
            impressora_ids = sock.execute('adm.impressora', 'search', args)
            if not impressora_ids:
                return json.dumps({'codigo': IMPRESSORAS_NAO_CADASTRADAS,
                                   'mensagem': 'Nenhuma impressora cadastrada no ADM!',
                                   'retorno': []})

            impressoras = []
            for i in sock.execute('adm.impressora', 'name_get', impressora_ids):
                impressoras.append(i[1])

            return json.dumps({'retorno': impressoras,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def imprimir_etiqueta(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            codigo = req.params.get('codigo')
            modelo = req.params.get('modelo')
            impressora = req.params.get('impressora')
            quantidade = int(req.params.get('quantidade') or '1')
            company_id = int(req.params.get('company_id'))
            cb_ids = sock.execute('automacao.codigo.barra', 'search', [('codigo_barra', '=', codigo)])
            if not cb_ids:
                return json.dumps({'codigo': ERRO_INTERNO,
                                   'mensagem': 'Produto não encontrado para o códido:\n %s' % codigo,
                                   'retorno': []})

            cb_id = cb_ids and cb_ids[0]
            item_dados = cb_id and sock.execute(
                'automacao.codigo.barra', 'read', cb_id, ['produto_id', 'grade_id',
                                                          'produto_embalagem_id'])
            produto_id = item_dados and item_dados.get('produto_id')[0]
            produto_dados = produto_id and sock.execute(
                'estoque.produto', 'read', produto_id, ['name', 'preco_venda', 'codigo',
                                                        'digito'], {'coletor': True}) or dict()
            if produto_dados.get('erro'):
                return json.dumps({'codigo': LEITURA_DE_PRODUTO,
                                   'mensagem': produto_dados.get('msg'),
                                   'retorno': []})

            grade_id = item_dados and item_dados.get('grade_id')[0]
            grade_dados = grade_id and sock.execute(
                'estoque.produto.grade', 'read', grade_id, [
                    'grade', 'name', 'descricao', 'codigo_no_fabricante', 'descricao_resumida',
                    'produto_codigo_fornecedor_ids']) or dict()
            fornecedor_ids = grade_dados and grade_dados.get('produto_codigo_fornecedor_ids')
            fornecedor_id = fornecedor_ids and fornecedor_ids[0]
            fornecedor_dados = fornecedor_id and sock.execute(
                'estoque.produto.codigo.fornecedor', 'read', fornecedor_id, ['codigo_fornecedor']) or \
                               dict()
            embalagem_id = item_dados and item_dados.get('produto_embalagem_id')[0]
            embalagem_dados = embalagem_id and sock.execute(
                'estoque.produto.embalagem', 'read', embalagem_id, ['embalagem_id']) or dict()
            embalagem_id = embalagem_dados and embalagem_dados.get('embalagem_id')[0]
            embalagem_dados = embalagem_id and sock.execute(
                'estoque.embalagem', 'read', embalagem_id, ['name', 'unidade_id']) or dict()
            unidade_id = embalagem_dados and embalagem_dados.get('unidade_id')[0]
            unidade_dados = unidade_id and sock.execute(
                'estoque.embalagem.unidade', 'read', unidade_id, ['name', 'unidade']) or dict()
            preco_dados = sock.execute('estoque.preco.venda', 'etiqueta_get_preco_venda', produto_id,
                                         company_id, False, grade_id, True)

            # Valores
            DESCRICAO_PRODUTO = grade_dados and (grade_dados.get('descricao_resumida') or
                                                 grade_dados.get('name'))
            EMBALAGEM = embalagem_dados and embalagem_dados.get('name')
            CODIGO_DE_BARRA = codigo[-13:-1]
            PRECO_CADASTRO = '%.2f' % produto_dados.setdefault('preco_venda', 0)
            preco = preco_dados.setdefault(str(grade_id), {}).setdefault('preco_venda', float(PRECO_CADASTRO))
            if not preco:
                preco = preco_dados.setdefault(str(produto_id), {}).setdefault('preco_venda',
                                                                               float(PRECO_CADASTRO))
            PRECO_VENDA = '%.2f' % preco
            VALOR_DESCONTO = '%.2f' % (float(PRECO_CADASTRO) - float(PRECO_VENDA))
            UNI = unidade_dados.get('unidade') or str()
            PERC_DESC = '%.2f' % ((float(PRECO_CADASTRO) and
                                   float(VALOR_DESCONTO) * 100 / float(PRECO_CADASTRO)) or 0)
            name = produto_dados.get('name')
            DESCRICAO_PRODUTO1 = name[:20]
            DESCRICAO_PRODUTO2 = name[20:40]
            CODIGO_INTERNO = produto_dados.get('codigo') or str()
            GRADE = grade_dados.get('grade') or str()
            GRADE_DESCRICAO = grade_dados.get('descricao') or str()
            CODIGO_FABRICANTE = grade_dados.get('codigo_no_fabricante') or str()
            CODIGO_FORNECEDOR = fornecedor_dados.get('codigo_fornecedor') or str()
            DT_EMISSAO = time.strftime('%d/%m/%Y')

            # Substituicao dos valores no arquivo
            modelo = modelo.format(**locals())

            # Substituicao para o digito (#)
            modelo = modelo.format(**{'#': produto_dados.get('digito') or str()})

            # Impressao
            arquivo = NamedTemporaryFile()
            arquivo.write(modelo)
            arquivo.flush()
            try:
                if ' -> ' in impressora:
                    impressora = impressora.split(' -> ')[-1]
                subprocess.call(["lp", "-n%s" % quantidade, "-d" + impressora, arquivo.name])
            except OSError as e:
                return json.dumps({'codigo': ERRO_IMPRESSAO,
                                   'mensagem': 'Erro ao enviar impressao: %s' % e.value,
                                   'retorno': []})
            finally:
                arquivo.close()

            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': 'Impressão enviada com sucesso!',
                               'retorno': []})

        except Exception as e:
            return json.dumps({'codigo': ERRO_IMPRESSAO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Funcoes do Inventario  ---------------------------------------
    @http.httprequest
    def get_inventarios(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            args = [('atualizado', '=', False)]
            ids = sock.execute('estoque.inventario.capa', 'search', args)
            if not ids:
                return json.dumps({'codigo': SEM_INVENTARIO,
                                   'mensagem': 'Não há inventário disponível!',
                                   'retorno': []})

            campos = ['id', 'name']
            dados = []
            for i in sock.execute('estoque.inventario.capa', 'read', ids, campos):
                dados.append({'id': i.get('id'),
                              'nome': i.get('name')})

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_produto_inventario(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            codigo_barra = req.params.get('codigo') or ''
            inventario_id = int(req.params.get('inventario_id') or 0)
            local_id = int(req.params.get('local_id') or 0)
            incluir_item = int(req.params.get('incluir_item') or 0)  # 0 ou 1
            sock = MySock(req.params)
            args = [('codigo_barra', '=', codigo_barra)]
            automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
            if not automacao_codigo_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                   'mensagem': 'Produto não cadastrado!',
                                   'retorno': []})

            campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
            if isinstance(automacao_codigo_id,list):
                    automacao_codigo_id=automacao_codigo_id[0]
            dados_automacao = sock.execute('automacao.codigo.barra', 'read', int(automacao_codigo_id), campos)
            args = [('inventario_id', '=', inventario_id), ('produto_id', '=', dados_automacao.get('produto_id')[0]),
                    ('grade_id', '=', dados_automacao.get('grade_id')[0]), ('local_id', '=', local_id)]
            contagem_invent_id = sock.execute('estoque.inventario.contagem', 'search', args)
            if not contagem_invent_id and not incluir_item:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_INVENTARIO,
                                   'mensagem': 'Este item não está no inventário!',
                                   'retorno': []})

            campos = ['name']
            dados_grade = sock.execute('estoque.produto.grade', 'read', int(dados_automacao.get('grade_id')[0]), campos)
            prod_embalagem_id = int(dados_automacao.get('produto_embalagem_id')[0])
            if isinstance(prod_embalagem_id, list):
                prod_embalagem_id =prod_embalagem_id[0]

            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read', int(prod_embalagem_id), ['embalagem_id'])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read', int(dados_prod_embalagem.get('embalagem_id')[0]), ['sigla', 'quantidade_na_embalagem', 'unidade_id'])
            emb_unidade_id = dados_emb_embalagem.get('unidade_id')
            qtde_embalagem = dados_emb_embalagem.get('quantidade_na_embalagem', 1)
            if emb_unidade_id and isinstance(emb_unidade_id, list):
                emb_unidade_id=emb_unidade_id[0]

            fracionado = sock.execute('estoque.embalagem.unidade', 'read', int(emb_unidade_id), ['fracao'])
            if isinstance(fracionado, dict):
                fracionado=fracionado.get('fracao')

            produto_id = dados_automacao.get('produto_id')[0]
            grade_id = dados_automacao.get('grade_id')[0]
            descricao = dados_grade.get('name')
            args = [('inventario_id', '=', inventario_id),('produto_id', '=', produto_id),
                    ('grade_id', '=', grade_id),('local_id', '=', local_id)]
            contagem_invent_id = sock.execute('estoque.inventario.contagem', 'search', args)
            contagem_invent_id = contagem_invent_id and contagem_invent_id[0]
            usuario_ids = sock.execute('usuario.coletor', 'search', [('usuario_id', '=', sock.uid)])
            usuario_col_dados = sock.execute('usuario.coletor', 'read', usuario_ids[0], ['saldo_produto'])
            if usuario_col_dados and isinstance(usuario_col_dados, list):
                usuario_col_dados=usuario_col_dados[0]

            mostra_saldo_produto = False
            if usuario_col_dados and isinstance(usuario_col_dados, dict):
                mostra_saldo_produto = usuario_col_dados.get('saldo_produto')

            vals = {'nome': str(descricao),
                    'qtd_embalagem': str(int(qtde_embalagem)).replace('.', ','),
                    'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                    'saldo': 0,
                    'mostra_saldo': False,
                    'fracionado': fracionado,
                    'produto_id': produto_id,
                    'grade_id': grade_id,
                    'produto_embalagem_id': prod_embalagem_id}
            if mostra_saldo_produto:
                if contagem_invent_id:
                    saldo_contagem = sock.execute(
                        'estoque.inventario.contagem', 'read', contagem_invent_id,
                        ['saldo_cadastro']) or {}
                    saldo = float(saldo_contagem.get('saldo_cadastro')) / qtde_embalagem
                    vals.update({'mostra_saldo': True,
                                 'saldo': str(saldo).replace('.', ',')})

            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_locacao_inventario(self, req, s_action=None, **kw):
        # verifica se tem locacao, e cria uma locacao se for impormado o parametro para criar
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            produto_id = int(req.params.get('produto_id'))
            company_id = int(req.params.get('company_id'))
            grade_id = int(req.params.get('grade_id'))
            produto_embalagem_id = int(req.params.get('produto_embalagem_id'))
            local_id = int(req.params.get('local_id') or 0)
            incluir_locacao = int(req.params.get('incluir_locacao') or 0)  # 0 ou 1
            if incluir_locacao:
                dic_local_locacao = {'company_id': company_id,
                                     'local_id': local_id,
                                     'produto_id': produto_id,
                                     'grade_id': grade_id,
                                     'estllc_produto_embalagem_id': produto_embalagem_id}
                sock.execute('estoque.local.locacao', 'create', dic_local_locacao)

            args = [('local_id', '=', local_id), ('grade_id', '=', grade_id), ('produto_id', '=', produto_id)]
            local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)
            if not local_locacao_id and not incluir_locacao:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_LOCAL,
                                   'mensagem': 'Este item não está no Local informado!',
                                   'retorno': []})
            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_contagem_inventario(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            inventario_id = int(req.params.get('inventario_id') or 0)
            produto_id = int(req.params.get('produto_id'))
            grade_id = int(req.params.get('grade_id'))
            local_id = int(req.params.get('local_id') or 0)
            produto_embalagem_id = int(req.params.get('produto_embalagem_id') or 0)
            contagem = float(req.params.get('contagem').replace(',', '.') or 0 )
            if contagem > 999999.99:
                contagem = 999999.99

            args = [('local_id','=',local_id),('grade_id','=',grade_id),('produto_id','=', produto_id)]
            local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)
            pessoa_id  = sock.execute('res.users', 'read', sock.uid, ['pessoa_id','company_id'])
            company_id = ''
            if pessoa_id and isinstance(pessoa_id, list):
                pessoa_id = pessoa_id[0]
            if pessoa_id and isinstance(pessoa_id, dict):
                company_id = pessoa_id.get('company_id')
                pessoa_id = pessoa_id.get('pessoa_id')
                if pessoa_id and isinstance(pessoa_id, list):
                    pessoa_id = pessoa_id[0]
            if company_id and isinstance(company_id, list):
                company_id=company_id[0]
            if isinstance(local_locacao_id, list):
                local_locacao_id=local_locacao_id[0]

            custos = sock.execute('estoque.preco.custo', 'get_custos_do_dia', produto_id, False, False,
                                    {'chamada_pelo_coletor': True})
            args = [('pessoa','=',pessoa_id)]
            funcionario_id = sock.execute('rh.funcionario', 'search', args)
            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read', produto_embalagem_id,
                                                ['embalagem_id'])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                               int(dados_prod_embalagem.get('embalagem_id')[0]),
                                               ['quantidade_na_embalagem'])
            contagem = contagem*int(dados_emb_embalagem.get('quantidade_na_embalagem'))
            args = [('inventario_id', '=', inventario_id), ('produto_id', '=', produto_id),
                    ('grade_id', '=', grade_id), ('local_id', '=', local_id)]
            contagem_invent_id = sock.execute('estoque.inventario.contagem', 'search', args)
            contagem_invent_id = contagem_invent_id and contagem_invent_id[0]
            if contagem_invent_id:
                parametro_id = sock.execute('estoque.parametro', 'search',
                                              [('company_id', '=', company_id)])
                tipo_contagem = 'S' # S = substituir, I = incrementar

                if parametro_id:
                    if isinstance(parametro_id, list):
                        parametro_id = parametro_id[0]

                    tipo_contagem = sock.execute('estoque.parametro', 'read',
                                          parametro_id, ['estpar_inventario_tipo_contagem']).get('estpar_inventario_tipo_contagem')


                if tipo_contagem == 'I':
                    saldo_contado = sock.execute('estoque.inventario.contagem', 'read', contagem_invent_id,
                                                 ['saldo_contado']) or {}
                    contagem += saldo_contado.get('saldo_contado') or 0

                sock.execute('estoque.inventario.contagem', 'write', contagem_invent_id,
                               {'saldo_contado':contagem})
            else:
                dic_invent_contagem = {'inventario_id':inventario_id,
                                       'produto_id': produto_id,
                                       'grade_id': grade_id,
                                       'estict_custo_medio_contabil': custos.get('estprc_custo_medio_contabil'),
                                       'estict_ultimo_custo': custos.get('estprc_ultimo_custo'),
                                       'conferido': False,
                                       'saldo_cadastro': 0,
                                       'saldo_contado': contagem,
                                       'requisitado': 0,
                                       'locacao_id': local_locacao_id,
                                       'local_id': local_id,
                                       'estict_item_atualizado': False,}
                if funcionario_id and isinstance(funcionario_id, list):
                    funcionario_id=funcionario_id[0]
                dic_invent_contagem['funcionario_id'] = funcionario_id
                sock.execute('estoque.inventario.contagem', 'create', dic_invent_contagem)

            return json.dumps({'retorno': ['ok'],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Funcoes Digitar Requisicao  -----------------------------------
    @http.httprequest
    def checa_ressuprimento(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            checa_modulo = sock.execute('adm.utils', 'admutl_check_modulo_instalado', 'estoqueressuprimento', {})
            if not checa_modulo:
                return json.dumps({'codigo': MODULO_RESSUPRIMENTO_NAO_INSTALADO,
                                   'mensagem': 'Modulo:"estoqueressuprimento" não está instalado!',
                                   'retorno': []})
            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_state_requisicao(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            state = req.params.get('state')  # 'R'=Esboco, 'P'=Aprovada, 'E'=Em Separacao
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            if not requisicao_id or requisicao_id <= 0:
                return json.dumps({'codigo': ID_REQUISICAO_INCORRETO,
                                   'mensagem': 'Id da Requisição incorreto!',
                                   'retorno': []})

            context = {'ressuprimento_alterar': True}
            res = sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id,
                                          {'state': state}, context)
            if not res:
                return json.dumps({'codigo': ERRO_ALTERAR_REQUISICAO,
                                   'mensagem': 'Erro ao alterar a Requisição!',
                                   'retorno': []})
            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_requisicoes(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            state = req.params.get('state')  # 'R'=Esboco, 'P'=Aprovada, 'E'=Em Separacao
            if len(state) > 1:
                state = eval(state)
            else:
                state = [state]
            args = [('state', 'in', state)]
            requisicao_ids = sock.execute('estoqueressuprimento.requisicao.capa', 'search', args, 0, [], 'data')
            if not requisicao_ids:
                return json.dumps({'codigo': SEM_REQUISICOES,
                                   'mensagem': 'Nenhuma requisição encontrada!',
                                   'retorno': []})
            dados = []
            for requisicao_id in requisicao_ids:
                # Utiliza a requisicao informada
                campos = ['id', 'numero','local_saida_id', 'company_id', 'resrca_empresa_entrada_id',
                          'local_entrada_id', 'state', 'requisicao_itens_ids', 'data']
                requisicao_dados = sock.execute('estoqueressuprimento.requisicao.capa',
                                               'read', int(requisicao_id), campos, {}, '_classic_write')
                if requisicao_dados and isinstance(requisicao_dados, list):
                    requisicao_dados=requisicao_dados[0]

                sai_local_id = int(requisicao_dados.get('local_saida_id'))
                ent_local_id = int(requisicao_dados.get('local_entrada_id'))
                local_saida = sock.execute('estoque.local', 'read', sai_local_id, ['id', 'name','numero'],
                                                  {}, '_classic_write')
                local_saida['nome'] = local_saida.pop('name')
                local_entrada = sock.execute('estoque.local', 'read', ent_local_id, ['id', 'name','numero'],
                                                  {}, '_classic_write')
                local_entrada['nome'] = local_entrada.pop('name')
                empresa_saida_id = int(requisicao_dados.get('company_id'))
                empresa_entrada_id = int(requisicao_dados.get('resrca_empresa_entrada_id'))
                empresa_saida = sock.execute('res.company', 'read', empresa_saida_id, ['id', 'name'], {},
                                             '_classic_write')
                empresa_saida['nome'] = empresa_saida.pop('name')
                empresa_entrada = sock.execute('res.company', 'read', empresa_entrada_id, ['id', 'name'], {},
                                               '_classic_write')
                empresa_entrada['nome'] = empresa_entrada.pop('name')
                data = str(requisicao_dados.get('data')).split('-')
                data.reverse()
                data = '%s/%s/%s' % tuple(data)
                vals = {'id': requisicao_dados.get('id'),
                        'numero': requisicao_dados.get('numero'),
                        'empresa_saida': empresa_saida,
                        'empresa_entrada': empresa_entrada,
                        'local_saida': local_saida,
                        'local_entrada': local_entrada,
                        'state': requisicao_dados.get('state'),
                        'item_ids': requisicao_dados.get('requisicao_itens_ids'),
                        'data': data,
                        }

                dados.append(vals)
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def criar_requisicao(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            entrada_company_id = int(req.params.get('entrada_company_id') or 0)
            entrada_local_id = int(req.params.get('entrada_local_id') or 0)
            saida_local_id = int(req.params.get('saida_local_id') or 0)
            dic_est_requisicao = {'resrca_empresa_entrada_id': entrada_company_id,
                                  'local_entrada_id': entrada_local_id,
                                  'local_saida_id': saida_local_id, }
            requisicao_id = sock.execute('estoqueressuprimento.requisicao.capa', 'create', dic_est_requisicao)
            if not requisicao_id:
                return json.dumps({'codigo': REQUISICAO_NAO_CRIADA,
                                   'mensagem': 'Não foi possível criar a requisição!',
                                   'retorno': []})
            dados_req = sock.execute('estoqueressuprimento.requisicao.capa', 'read', requisicao_id, ['numero'])
            dados = [requisicao_id]
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'Requisição %s criada com sucesso!' % dados_req.get('numero')})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_item_requisicao(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            produto_id = int(req.params.get('produto_id') or 0)
            embalagem_id = int(req.params.get('embalagem_id') or 0)
            grade_id = int(req.params.get('grade_id') or 0)
            saldo = float(req.params.get('saldo').replace(',', '.') or 0)
            quantidade_requisitada = float(req.params.get('quantidade_requisitada') or 0)
            requisicao_item = {'requisicao_id': int(requisicao_id),
                               'produto_id': produto_id,
                               'grade_id': grade_id,
                               'quantidade_requisitada': quantidade_requisitada,
                               'embalagem_id': embalagem_id,
                               'saldo': saldo}

            if not sock.execute('estoqueressuprimento.requisicao.itens', 'create', requisicao_item):
                return json.dumps({'codigo': ITEM_REQUISICAO_NAO_CRIADO,
                                   'mensagem': 'Não foi possível inclior o item na requisição!',
                                   'retorno': []})

            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item incluído na Requisição com sucesso!'})
        
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Separar Requisicao  -----------------------------------
    @http.httprequest
    def set_separar_item_requisicao(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            req_item_id = int(req.params.get('req_item_id') or 0)
            separar_conferir = int(req.params.get('separar_conferir') or 0)
            dict_req_item = {'ja_confirmada': True}
            requisicao_id = False
            if separar_conferir:
                campos = ['quantidade_requisitada', 'requisicao_id']
                dados_item = sock.execute('estoqueressuprimento.requisicao.itens', 'read',
                                          req_item_id, campos, {}, '_classic_write')
                qtd_req = dados_item.get('quantidade_requisitada')
                requisicao_id = dados_item.get('requisicao_id')
                dict_req_item['quantidade_confirmada'] = qtd_req

            if not sock.execute('estoqueressuprimento.requisicao.itens', 'write', req_item_id,
                                dict_req_item):
                return json.dumps({'codigo': ERRO_AO_SEPARAR_ITEM,
                                   'mensagem': 'Não foi possível separar o item na requisição!',
                                   'retorno': []})
            msg = 'Item separado com sucesso!'
            if separar_conferir and requisicao_id:
                msg = 'Item separado e conferido!'
                context = {'ressuprimento_alterar': True}
                sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id, {},
                             context)

            dados = []
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': msg})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_itens_requisicao(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            context = {'ressuprimento_alterar': True}
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            local_id = int(req.params.get('local_id') or 0)
            company_id = int(req.params.get('company_id') or 0)
            item_ids = eval(req.params.get('item_ids') or '[]')
            args = [('id', 'in', item_ids), ('ja_confirmada', '=', False)]
            item_ids = sock.execute('estoqueressuprimento.requisicao.itens', 'search', args)
            if not item_ids:
                state = {'state': 'S'}
                if sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id, state, context):
                    return json.dumps({'retorno': [],
                                       'codigo': SEM_ERRO,
                                       'mensagem': 'Separação da requisição concluída!'})
                return json.dumps({'codigo': ERRO_ALTERAR_REQUISICAO,
                                   'mensagem': 'Erro ao alterar a Requisição!!',
                                   'retorno': []})

            campos = ['id', 'produto_id', 'grade_id', 'quantidade_requisitada', 'embalagem_id']
            itens_dados = sock.execute('estoqueressuprimento.requisicao.itens', 'read',
                                                     item_ids, campos)

            dados = []
            for item in itens_dados:
                produto_id = int(item.get('produto_id')[0])
                grade_id = int(item.get('grade_id')[0])
                args = [('company_id', '=', company_id),
                        ('produto_id', '=', produto_id),
                        ('grade_id', '=', grade_id),
                        ('local_id', '=', local_id)]
                local_locacao_id = sock.execute('estoque.local.locacao', 'search', args,  0, 0, '',
                                                {'sig_force_company': True})[0]
                campos = ['locacao', 'estllc_produto_embalagem_id']
                locacao_dados = sock.execute('estoque.local.locacao', 'read', local_locacao_id,
                                             campos, {}, '_classic_write')
                locacao = str(locacao_dados.get('locacao') or 'Sem locação')
                prod_embalagem_id = locacao_dados.get('estllc_produto_embalagem_id')
                qtd_requisitada = item.get('quantidade_requisitada')
                campos = ['name']
                grade_dados = sock.execute('estoque.produto.grade', 'read', grade_id, campos)
                dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read',
                                                      prod_embalagem_id, ['embalagem_id'],
                                                    {}, '_classic_write')
                dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                                     dados_prod_embalagem.get('embalagem_id'),
                                                     ['sigla', 'quantidade_na_embalagem'])
                qtd_embalagem = dados_emb_embalagem.get('quantidade_na_embalagem')
                args = [('grade_id', '=', grade_id),
                        ('produto_id', '=', produto_id),
                        ('produto_embalagem_id', '=', prod_embalagem_id)]
                automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)[0]
                campos = ['codigo_barra']
                dados_automacao = sock.execute('automacao.codigo.barra', 'read',
                                               automacao_codigo_id, campos)
                dados.append({'locacao': locacao,
                              'req_item_id': item.get('id'),
                              'codigo': dados_automacao.get('codigo_barra'),
                              'descricao': grade_dados.get('name'),
                              'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                              'qtd_embalagem': str(qtd_embalagem).replace('.',','),
                              'qtd_requisitada': str(qtd_requisitada/qtd_embalagem).replace('.',',')})

            dados = sorted(dados, key=lambda x: x.get('locacao'))
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Conferir Requisicao  -----------------------------------
    @http.httprequest
    def get_item_requisicao_conferencia(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            requisicao_id = int(req.params.get('requisicao_id') or 0)
            grade_id = int(req.params.get('grade_id') or 0)
            codigo_barras = req.params.get('codigo_barra').zfill(14)
            args = [('codigo_barra', '=', codigo_barras)]
            automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
            if not automacao_codigo_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                   'mensagem': 'Produto não cadastrado!',
                                   'retorno': []})

            if isinstance(automacao_codigo_id, list):
                automacao_codigo_id = automacao_codigo_id[0]
            campos = ['produto_id', 'grade_id', 'codigo_barra']
            dados_automacao = sock.execute('automacao.codigo.barra', 'read',
                                           int(automacao_codigo_id), campos)
            if dados_automacao and isinstance(dados_automacao, list):
                dados_automacao = dados_automacao[0]

            args = [('requisicao_id', '=', requisicao_id),
                    ('produto_id', '=', dados_automacao.get('produto_id')[0]),
                    ('grade_id', '=', dados_automacao.get('grade_id')[0])]
            item_requisicao_id = sock.execute('estoqueressuprimento.requisicao.itens', 'search',
                                                args)
            if not item_requisicao_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO_REQUISICAO,
                                   'mensagem': 'Este produto não está na requisição!',
                                   'retorno': []})
            # verifica se a grade do item digitado eh a mesma do item da requisicao vindo de
            # Separar e Conferir
            if grade_id and grade_id != dados_automacao.get('grade_id')[0]:
                return json.dumps({'codigo': ITEM_NAO_REQUERIDO,
                                   'mensagem': 'Produto diferente do requerido!',
                                   'retorno': []})

            if isinstance(item_requisicao_id, list):
                item_requisicao_id = item_requisicao_id[0]
            grade_id = dados_automacao.get('grade_id')
            if grade_id and isinstance(grade_id, list):
                grade_id = grade_id[0]
            campos = ['name']
            grade_descricao = sock.execute('estoque.produto.grade', 'read', int(grade_id), campos)
            if grade_descricao and isinstance(grade_descricao, dict):
                grade_descricao = grade_descricao.get('name')

            campos = ['embalagem_id', 'quantidade_confirmada', 'quantidade_requisitada']
            dados_req_itens = sock.execute('estoqueressuprimento.requisicao.itens', 'read',
                                             int(item_requisicao_id), campos)
            if isinstance(dados_req_itens, list):
                dados_req_itens = dados_req_itens[0]
            prod_embalagem_id = dados_req_itens.get('embalagem_id')[0]
            if isinstance(prod_embalagem_id, list):
                prod_embalagem_id = prod_embalagem_id[0]
            dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read',
                                                  int(prod_embalagem_id), ['embalagem_id'])
            dados_emb_embalagem = sock.execute('estoque.embalagem', 'read',
                                                 int(dados_prod_embalagem.get('embalagem_id')[0]),
                                                 ['sigla', 'quantidade_na_embalagem'])

            qtd_embalagem = dados_emb_embalagem.get('quantidade_na_embalagem')
            qtd_requisitada = dados_req_itens.get('quantidade_requisitada')
            qts_item = 0  # quantidade do item que sera conferida
            qtd_confirmada = dados_req_itens.get('quantidade_confirmada') / qtd_embalagem + qts_item
            vals = {'req_item_id': item_requisicao_id,
                    'codigo': dados_automacao.get('codigo_barra'),
                    'descricao': grade_descricao,
                    'sigla_embalagem': dados_emb_embalagem.get('sigla'),
                    'qtd_embalagem': str(qtd_embalagem).replace('.',','),
                    'qtd_requisitada': str(qtd_requisitada/qtd_embalagem).replace('.',','),
                    'qtd_confirmada': str(qtd_confirmada).replace('.',',')}

            return json.dumps({'retorno': [vals],
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item separado com sucesso!'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_qtd_conferida(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            requisicao_id = int(req.params.get('requisicao_id'))
            req_item_id = int(req.params.get('req_item_id'))
            qtd_confirmada = float(req.params.get('qtd_confirmada') or 0)
            ja_confirmada = eval(req.params.get('ja_confirmada'))
            dict_req_item = {'quantidade_confirmada': qtd_confirmada,
                             'ja_confirmada': ja_confirmada and True or False}
            context = {'ressuprimento_alterar': True}
            sock.execute('estoqueressuprimento.requisicao.itens', 'write', req_item_id,
                         dict_req_item, context)
            sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id, {},context)
            campos = ['resrca_com_pendencia']
            dados_req = sock.execute('estoqueressuprimento.requisicao.capa', 'read',
                                      requisicao_id, campos)
            if not dados_req.get('resrca_com_pendencia'):
                sock.execute('estoqueressuprimento.requisicao.capa', 'write', requisicao_id,
                                   {'state': 'F'}, context)
                return json.dumps({'retorno': [{'requisicao_conferida': True}],
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'Requisicao conferida!'})

            if ja_confirmada:
                return json.dumps({'retorno': [],
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'Item conferido com sucesso!'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # -----------------------------  Requisicao de compra  -----------------------------------------

    @http.httprequest
    def get_requisicao_compra(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            company_id = int(req.params.get('company_id'))
            context = {'sig_force_company': company_id}
            requisicao_ids = sock.execute('compras.requisicao.capa', 'search',
                                          [('state', '=', 'B'),
                                           ('company_id', '=', company_id)],  0, 0, '', context)
            res = []
            if requisicao_ids:
                campos = ['id', 'name']
                dados = sock.execute('compras.requisicao.capa', 'read', requisicao_ids, campos)
                for i in dados:
                    res.append({'id': i.get('id'),
                                'nome': i.get('name')})

            return json.dumps({'retorno': res,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_item_requisicao_compra(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            codigo = (req.params.get('codigo_barra') or '').zfill(14)
            args = [('codigo_barra', '=', codigo)]
            automacao_codigo_id = sock.execute('automacao.codigo.barra', 'search', args)
            if not automacao_codigo_id:
                return json.dumps({'codigo': ITEM_NAO_CADASTRADO,
                                   'mensagem': 'Produto não cadastrado!',
                                   'retorno': []})
            automacao_codigo_id = automacao_codigo_id[0]
            grade_id = sock.execute('automacao.codigo.barra', 'read', automacao_codigo_id,
                                           ['grade_id'], {}, '_classic_write').get('grade_id')
            dados_grade = sock.execute('estoque.produto.grade', 'read', grade_id, ['name'])
            return json.dumps({'retorno': [{'descricao': dados_grade.get('name'),
                                            'codigo_id':  automacao_codigo_id }],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_item_requisicao_compra(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            company_id = int(req.params.get('company_id'))
            context = {'sig_force_company': company_id}
            requisicao_id = int(req.params.get('requisicao_id'))
            automacao_codigo_id = int(req.params.get('codigo_id'))
            qtd = float((req.params.get('quantidade') or '').replace(',', '.'))
            campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
            dados_automacao = sock.execute('automacao.codigo.barra', 'read', automacao_codigo_id,
                                           campos, {}, '_classic_write')

            prod_embalagem_id = dados_automacao.get('produto_embalagem_id')
            numero = ''
            nome = ''
            if not requisicao_id:
                requisicao_id = sock.execute('compras.requisicao.capa', 'create',
                                             {'company_id': company_id}, context)

                dados = sock.execute('compras.requisicao.capa', 'read', requisicao_id,
                                      ['name', 'numero'])
                numero = dados.get('numero')
                nome = dados.get('name')

            sock.execute('compras.requisicao.itens', 'create',
                           {'requisicao_id': requisicao_id,
                            'produto_id': dados_automacao.get('produto_id'),
                            'grade_id': dados_automacao.get('grade_id'),
                            'comrit_produto_embalagem_id': prod_embalagem_id,
                            'quantidade': qtd})

            return json.dumps({'retorno': [{'requisicao_id': requisicao_id,
                                            'numero': numero,
                                            'nome': nome}],
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item Adicionado!'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # --------------------------------  Separar Pedido  --------------------------------------------
    @http.httprequest
    def get_pedidos(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            args = [('state', 'in', ('V','S')), ('tipo', '=', 'V')]
            offset = 0
            limit = 1
            order = 'data_emissao'
            pedido_capa_id = sock.execute('estoquepedidos.pedido.capa', 'search', args, offset,
                                            limit, order)
            if not pedido_capa_id:
                return json.dumps({'retorno': [],
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'Nenhum pedido para separar!'})

            pedido_capa_id = pedido_capa_id[0]
            campos = ['numero_pedido', 'pedido_item_ids', 'state']
            pedido_capa_dados = sock.execute('estoquepedidos.pedido.capa', 'read',
                                               int(pedido_capa_id), campos)
            dados = []
            for pedido in pedido_capa_dados:
                pedido_itens_ids = pedido.get('pedido_item_ids') or []
                if not pedido_itens_ids:  # pedidos sem itens nao sao enviados
                    continue

                numero_pedido = pedido.get('numero_pedido')
                vals = {'id': pedido.get('id'),
                        'numero': str(numero_pedido),
                        'item_ids': pedido_itens_ids,
                        'state': pedido.get('state')}
                dados.append(vals)

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_state_pedido(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            pedido_id = int(req.params.get('id') or 0)
            state = req.params.get('state')
            if not pedido_id or pedido_id <= 0:
                return json.dumps({'codigo': ID_PEDIDO_INCORRETO,
                                   'mensagem': 'Id do Pedido incorreto!',
                                   'retorno': []})

            context = {'pedcap_write_state': True}
            if not sock.execute('estoquepedidos.pedido.capa', 'write', pedido_id, {'state': state},
                                context):
                return json.dumps({'codigo': ERRO_ALTERAR_PEDIDO,
                                   'mensagem': 'Erro ao alterar a Pedido!',
                                   'retorno': []})

            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_itens_pedido(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            company_id = int(req.params.get('company_id') or 0)
            item_ids = eval(req.params.get('item_ids') or '[]')
            campos = ['produto_id', 'grade_id', 'local_id', 'qtde_atendida', 'qtde_pedida']
            itens_dados = sock.execute('estoquepedidos.pedido.item', 'read', item_ids, campos)
            dados = []
            for item in itens_dados:
                produto_id = item.get('produto_id')[0]
                grade_id = item.get('grade_id')[0]
                local_id = item.get('local_id')[0]
                args = [('company_id', '=', company_id), ('produto_id', '=', produto_id),
                        ('grade_id', '=', grade_id), ('local_id', '=', local_id)]
                local_locacao_id = sock.execute('estoque.local.locacao', 'search', args)[0]
                args = [('grade_id', '=', grade_id),
                        ('produto_id', '=', produto_id)]
                codigo_id = sock.execute('automacao.codigo.barra', 'search', args)[0]
                codigo_barra = sock.execute('automacao.codigo.barra', 'read', codigo_id,
                                        ['codigo_barra']).get('codigo_barra')
                campos = ['locacao']
                local_locacao = sock.execute('estoque.local.locacao', 'read', local_locacao_id,
                                             campos)
                local_locacao = local_locacao.get('locacao') or 'Sem locação'

                codigo_item = \
                sock.execute('estoque.produto', 'read', int(produto_id), ['codigo']).get(
                    'codigo')
                descricao = sock.execute('estoque.produto.grade', 'read', int(grade_id),
                                           ['name']).get('name')
                qtde_atendida = item.get('qtde_atendida')
                qtd_pedida = item.get('qtde_pedida')
                dict_produto = {'id': item.get('id'),
                                'locacao': local_locacao,
                                'codigo': codigo_barra,
                                'codigo_interno': codigo_item,
                                'descricao': descricao,
                                'produto_id': produto_id,
                                'grade_id': grade_id,
                                'local_id': local_id,
                                'qtd_atendida': str(qtde_atendida or '0').replace('.', ','),
                                'qtd_pedida': str(qtd_pedida or '0').replace('.', ','),
                                }
                dados.append(dict_produto)

            # ordena dicionario pelas ordem alfabetica das chaves
            dados = sorted(dados, key=lambda x: x.get('locacao'))
            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})


    @http.httprequest
    def set_qtd_item_pedido(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            item_id = int(req.params.get('item_id') or 0)
            qtde_atendida = float((req.params.get('qtd_atendida') or '0').replace(',', '.') or 0)
            sock.execute('estoquepedidos.pedido.item', 'write', item_id,
                         {'qtde_atendida': qtde_atendida})

            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': 'Item separado'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    # --------------------------------  Criar Pedido de venda  --------------------------------------------

    @http.httprequest
    def get_verifica_modulo_vendas_externas(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            cr, erro = self._get_cursor(req.params)
            if not cr:
                return erro

            cr.execute("""  SELECT state 
                            FROM ir_module_module 
                            WHERE name = 'vendas_externas';""")
            dados = dictfetchall(cr)
            if not dados or dados[0].get('state') == 'uninstalled':
                cr.close()
                return json.dumps({'retorno': [],
                                   'codigo': ERRO_INTERNO,
                                   'mensagem': 'Para acessar o Pedido, precisa do módulo'
                                               ' "vendas_externas" instalado no ERP!'})
            cr.close()
            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': ''})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    def _get_cliente(self, req):
        sock = MySock(req.params)
        cpf = req.params.get('cpf').replace('.', '').replace('/', '').replace('-', '')
        res = Cpf().validar(cpf)
        res = res or Cnpj(cpf).validar().get('status')
        if not res:
            return json.dumps({'retorno': [],
                               'codigo': ERRO_CPF_INVALIDO,
                               'mensagem': 'CPF/CNPJ inválido!'})

        company_id = int(req.params.get('company_id'))
        company_id = sock.execute('adm.empresa.objeto', 'getEmpresa', 'financeiro.cliente',
                                  False, False, company_id)
        res = sock.execute('financeiro.cliente', 'search', [('cpf_cnpj', '=', cpf),
                                                            ('company_id', '=', company_id)],
                           0, 0, '', {'sig_force_company': company_id})
        if not res:
            res = sock.execute('adm.pessoa', 'search', [('cpf_cnpj', '=', cpf)])
            if not res:
                return json.dumps({'retorno': [],
                                   'codigo': CLIENTE_NAO_CADASTRADO,
                                   'mensagem': 'Cliente não está cadastrado!'})
            vals_cliente = {'pessoa_id': res[0],
                            'company_id': company_id}
            res = [sock.execute('financeiro.cliente', 'create', vals_cliente,
                                      {'sig_force_company': company_id})]

        campos = ['cpf_cnpj', 'name', 'endereco_ids', 'endereco_nf_id', 'email_ids', 'telefones_ids']
        cliente_id = res[0]
        res = sock.execute('financeiro.cliente', 'read', cliente_id, campos, {}, '_classic_write')
        email_id = res.get('email_ids')
        dados_email = {}
        if email_id:
            dados_email = sock.execute('adm.pessoa.email', 'read', email_id[0], ['name'])

        telefone_id = res.get('telefones_ids')
        dados_telefone = {}
        if telefone_id:
            dados_telefone = sock.execute('adm.pessoa.telefone', 'read', telefone_id[0], ['numero', 'ddd'])[0]

        endereco_id = res.get('endereco_nf_id') or res.get('endereco_ids') or False
        dados_end = {}
        if endereco_id:
            if isinstance(endereco_id, list):
                endereco_id = endereco_id[0]

            campos = ['cep', 'uf_id', 'cidade_id', 'bairro_id', 'logradouro_id', 'numero', 'complemento']
            dados_end = sock.execute('adm.pessoa.endereco', 'read', endereco_id, campos)
            if dados_end.get('uf_id'):
                dados_end['uf'] = dados_end.get('uf_id')[1]
            if dados_end.get('cidade_id'):
                dados_end['cidade'] = dados_end.get('cidade_id')[1]
            if dados_end.get('bairro_id'):
                dados_end['bairro'] = dados_end.get('bairro_id')[1]
            if dados_end.get('logradouro_id'):
                dados_end['logradouro'] = dados_end.get('logradouro_id')[1]

        telefone = dados_telefone.get('numero')
        if dados_telefone.get('ddd') and telefone:
            telefone = '(' + dados_telefone.get('ddd').replace('(', '').replace(')', '') + ')' + telefone
        dados = {'id': cliente_id,
                 'cpf': res.get('cpf_cnpj'),
                 'nome': res.get('name'),
                 'email': dados_email.get('name') or '',
                 'email_id': dados_email.get('id') or 0,
                 'telefone': telefone or '',
                 'telefone_id': dados_telefone.get('id') or 0,
                 'endereco': {'id': dados_end.get('id'),
                              'cep': (dados_end.get('cep') or '').replace('.', ''),
                              'uf': dados_end.get('uf') or '',
                              'cidade': dados_end.get('cidade') or '',
                              'bairro': dados_end.get('bairro') or '',
                              'logradouro': dados_end.get('logradouro') or '',
                              'numero': dados_end.get('numero'),
                              'complemento': dados_end.get('complemento')},
                 }
        return json.dumps({'retorno': [dados],
                           'codigo': SEM_ERRO,
                           'mensagem': 'ok'})

    @http.httprequest
    def get_cliente(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            return self._get_cliente(req)

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    def cadastrar_endereco(self, sock, endereco, pessoa_id):
        endereco_id = endereco.get('id') or False
        cep = endereco.get('cep')
        numero = endereco.get('numero')
        complemento = endereco.get('complemento')

        pais_id = sock.execute('adm.pais', 'search', [('name', '=', 'Brasil')])[0]
        uf_id = sock.execute('adm.uf', 'search', [('sigla', '=', endereco.get('uf').upper())])
        if uf_id:
            uf_id = uf_id[0]

        cidade = endereco.get('cidade')
        cidade_id = False
        if cidade:
            cidade_id = sock.execute('adm.cidade', 'search', [('name', 'ilike', cidade),
                                                              ('uf_id', '=', uf_id)])
            if cidade_id:
                cidade_id = cidade_id[0]
            elif cidade:
                cidade_id = sock.execute('adm.cidade', 'create', {'name': cidade,
                                                                  'uf_id': uf_id,
                                                                  'pais_id': pais_id})

        bairro = endereco.get('bairro')
        bairro_id = False
        if bairro:
            bairro_id = sock.execute('adm.bairro', 'search', [('name', 'ilike', bairro)])
            if bairro_id:
                bairro_id = bairro_id[0]
            else:
                bairro_id = sock.execute('adm.bairro', 'create', {'name': bairro})

        logradouro = endereco.get('logradouro')
        logradouro_id = False
        if logradouro:
            logradouro = logradouro.split()
            titulo = logradouro[0]
            logradouro = ' '.join(logradouro[1:])
            logradouro_id = sock.execute('adm.logradouro', 'search',
                                         [('titulo', 'ilike', titulo), ('descricao', 'ilike', logradouro)])
            if logradouro_id:
                logradouro_id = logradouro_id[0]
            else:
                logradouro_id = sock.execute('adm.logradouro', 'create',
                                             {'titulo': titulo, 'descricao': logradouro})

        if endereco_id:
            sock.execute('adm.pessoa.endereco', 'write', endereco_id,
                         {'uf_id': uf_id,
                          'cidade_id': cidade_id,
                          'bairro_id': bairro_id,
                          'logradouro_id': logradouro_id,
                          'cep': cep,
                          'numero': numero,
                          'complemento': complemento})

        elif uf_id or cidade_id or bairro_id or logradouro_id or cep or numero or complemento:
            tipo_id = sock.execute('adm.endereco.tipo', 'search', [('name', '=', 'Normal')])
            endereco_id = sock.execute('adm.pessoa.endereco', 'create',
                                         {'pessoa_id': pessoa_id,
                                          'endereco_tipo_id': tipo_id[0],
                                          'pais_id': pais_id,
                                          'uf_id': uf_id,
                                          'cidade_id': cidade_id,
                                          'bairro_id': bairro_id,
                                          'logradouro_id': logradouro_id,
                                          'cep': cep,
                                          'numero': numero,
                                          'complemento': complemento})
        return endereco_id

    def testar_logradouro(self, logradouro):
        if not logradouro:
            return True

        titulo = logradouro.split()[0].upper()
        if titulo not in TITULOS:
            raise Exception('O logradouro informado não contém algum dos títulos no início do nome:\n%s'
                            % ' '.join(map(lambda x: x.title(), TITULOS)))
        return True

    def get_vals_email(self, sock, par):
        email = par.get('email')
        email_id = int(par.get('email_id').replace('null', '') or '0')
        vals = {}
        if email:
            tipo_id = sock.execute('adm.email.tipo', 'search', [], 0, 0, 'id')
            if not tipo_id:
                tipo_id = sock.execute('adm.email.tipo', 'create', {'name': 'Comum'})
            else:
                tipo_id = tipo_id[0]

            op = email_id and 1 or 0
            vals['email_ids'] = [(op, email_id, {'name': email,
                                                 'email_tipo_id': tipo_id})]
        return vals

    def get_vals_telefone(self, sock, par):
        telefone = par.get('telefone')
        telefone_id = int(par.get('telefone_id').replace('null', '') or '0')
        vals = {}
        if telefone:
            tipo_id = sock.execute('adm.telefone.tipo', 'search', [])
            if not tipo_id:
                tipo_id = sock.execute('adm.telefone.tipo', 'create', {'name': 'Celular'})
            else:
                tipo_id = tipo_id[0]

            op = telefone_id and 1 or 0
            ddd = ''
            if ')' in telefone:
                ddd, telefone = telefone.split(')')
                ddd = ddd.replace('(', '')
            vals_telefone = {'numero': telefone, 'telefone_tipo_id': tipo_id}
            if ddd:
                vals_telefone['ddd'] = ddd

            vals['telefones_ids'] = [(op, telefone_id, vals_telefone)]
        return vals

    @http.httprequest
    def set_criar_atualizar_cliente(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            endereco = eval(req.params.get('endereco') or '{}')
            self.testar_logradouro(endereco.get('logradouro'))
            company_id = int(req.params.get('company_id'))
            company_id = sock.execute('adm.empresa.objeto', 'getEmpresa', 'financeiro.cliente',
                                      False, False, company_id)
            cpf_cnpj = req.params.get('cpf_cnpj')
            nome = req.params.get('nome')
            if not nome:
                return json.dumps({'codigo': ERRO_INTERNO,
                                   'mensagem': 'Infome o nome do cliente!',
                                   'retorno': []})

            vals_email = self.get_vals_email(sock, req.params)
            vals_telefone = self.get_vals_telefone(sock, req.params)
            tipo = req.params.get('tipo')
            vals_cliente = {}
            vals_cliente.update(vals_email)
            vals_cliente.update(vals_telefone)
            cliente_id = sock.execute('financeiro.cliente', 'search', [('cpf_cnpj', '=', cpf_cnpj),
                                                                ('company_id', '=', company_id)],
                                      0, 0, '', {'sig_force_company': company_id})
            if tipo == '0' and not cliente_id:  # 0 = Criar, 1 = Atualizar
                tipo = len(cpf_cnpj) == 11 and 'F' or 'J'
                vals_cliente.update({'company_id': company_id,
                                     'pessoa_tipo': tipo,
                                     'name': nome,
                                     'cpf_cnpj': cpf_cnpj})

                cliente_id = sock.execute('financeiro.cliente', 'create', vals_cliente,
                                          {'sig_force_company': company_id})
            else:
                cliente_id = cliente_id[0]
                vals_cliente.update({'name': nome})
                sock.execute('financeiro.cliente', 'write', cliente_id, vals_cliente)

            dados_cliente = sock.execute('financeiro.cliente', 'read', cliente_id,
                                     ['pessoa_id', 'email_ids', 'telefones_ids', 'cpf_cnpj'],
                                     {}, '_classic_write')
            pessoa_id = dados_cliente.get('pessoa_id')
            endereco_id = self.cadastrar_endereco(sock, endereco, pessoa_id)
            email_id = dados_cliente.get('email_ids')
            telefone_id = dados_cliente.get('telefones_ids')
            return json.dumps({'retorno': [{'id': cliente_id,
                                            'cpf': dados_cliente.get('cpf_cnpj'),
                                            'endereco_id': endereco_id,
                                            'email_id': email_id and email_id[0] or 0,
                                            'telefone_id': telefone_id and telefone_id[0] or 0}],
                               'codigo': SEM_ERRO,
                               'mensagem': 'Cliente Atualizado!'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_produto_pedido(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            res = self._get_item(req)
            dados = json.loads(res)
            if dados.get('codigo') != SEM_ERRO:
                return res

            dados = dados.get('retorno')[0]
            if not dados.get('preco') or dados.get('preco') in ['0', '0,0', '0,00']:
                return json.dumps({'retorno': [],
                                   'codigo': PRODUTO_SEM_PRECO,
                                   'mensagem': 'Produto sem preço!'})

            for i in ['foto', 'saldo', 'saldos', 'data_entrada', 'locacao', 'qtd_embalagem', 'lastro']:
                try:
                    del (dados[i])
                except:
                    pass

            if not dados.get('fracao'):
                qtd = str(req.params.get('qtd') or '1').replace('.', ',').split(',')
                if len(qtd) > 1 and int(qtd[-1]) > 0:
                    return json.dumps({'retorno': [],
                                       'codigo': ERRO_INTERNO,
                                       'mensagem': 'Este produto não aceita quantidade fracionada!'})

            qtd = float(req.params.get('qtd').replace(',', '.'))
            if qtd <= 0:
                return json.dumps({'retorno': [],
                                   'codigo': ERRO_INTERNO,
                                   'mensagem': 'Quantidade Inválida!'})
            # se o produto veio com vr_total, ele eh de balanca, nao precisa calcular
            if not dados.get('vr_total'):
                preco = float(dados.get('preco').replace(',', '.'))
                vr_total = float_round(qtd * preco, 2)
                vr_total = str(vr_total).replace('.', ',')
                if vr_total[len(vr_total) - 2:len(vr_total) - 1] == ',':
                    vr_total += '0'
                dados['vr_total'] = vr_total

            if not dados.get('grades'):
                produto_id = dados.get('id')
                res = sock.execute('estoque.produto.grade', 'search', [('produto_id', '=', produto_id)])
                res = sock.execute('estoque.produto.grade', 'read', res, ['descricao', 'name'])
                dados['grades'] = res

            if len(dados.get('grades')) > 1:
                dados['grade_id'] = 0
            return json.dumps({'retorno': [dados],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})
        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_forma_condicao_pedido(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            forma_ids = sock.execute('adm.forma.pgto', 'search', [])
            if not forma_ids:
                return json.dumps({'retorno': [],
                                   'codigo': ERRO_INTERNO,
                                   'mensagem': 'Nenhuma forma de pagamento cadastrada!'})
            dados_forma = sock.execute('adm.forma.pgto', 'read', forma_ids, ['name', 'condicao_forma_ids'],
                                       {}, '_classic_write')
            dict_forma = {'sequencia': forma_ids}
            for forma in dados_forma:
                forma_id = forma.get('id')
                dict_forma.update({forma_id: {'nome': forma.get('name')}})
                for condicao in sock.execute('adm.condicao.forma', 'read', forma.get('condicao_forma_ids'),
                                             ['adm_condicao_id'], {}, '_classic_write'):
                    nome_condicao = sock.execute('adm.condicao', 'read', condicao.get('adm_condicao_id'),
                                                 ['name']).get('name')
                    if not dict_forma.get(forma_id).get('condicoes'):
                        dict_forma[forma_id]['condicoes'] = {condicao.get('id'): nome_condicao,
                                                             'sequencia': forma.get('condicao_forma_ids')}
                    else:
                        dict_forma[forma_id]['condicoes'].update({condicao.get('id'): nome_condicao})

            return json.dumps({'retorno': [dict_forma],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_pedido_venda(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        pedido_id = 0
        numero = ''
        try:
            sock = MySock(req.params)
            company_id = int(req.params.get('company_id'))
            pedido_id = int(req.params.get('pedido_id') or 0)
            cliente_id = int(req.params.get('cliente_id') or 0)
            local_id = sock.execute('estoque.parametro', 'estpar_get_fields',
                                    'local_venda_padrao_id', {})[0].get('local_venda_padrao_id')
            if not local_id:
                return json.dumps({'codigo': SEM_LOCAL_PADRAO,
                                   'mensagem': 'Sem local padrão de venda cadastrado'
                                               ' no parâmetro do estoque!',
                                   'retorno': [{'pedido_id': pedido_id}]})

            itens = []
            for item in eval(req.params.get('itens') or []):
                produto_id = item.get('produto_id')
                grade_id = item.get('grade_id')
                args = [('local_id', '=', local_id), ('grade_id', '=', grade_id),
                        ('produto_id', '=', produto_id), ('company_id', '=', company_id)]
                locacao_id = sock.execute('estoque.local.locacao', 'search', args, 0, 0, '',
                                          {'sig_force_company': company_id})
                prod_emb_id = sock.execute('estoque.local.locacao', 'read', locacao_id[0],
                                           ['estllc_produto_embalagem_id'], {},
                                           '_classic_write').get('estllc_produto_embalagem_id')
                vals_itens = {'produto_id': produto_id,
                              'grade_id': grade_id,
                              'local_id': local_id,
                              'produto_embalagem_id': prod_emb_id,
                              'qtde_pedida': item.get('qtde_pedida'),
                              'vr_unitario': item.get('vr_unitario')}
                itens.append([0, 0, vals_itens])

            condicoes = []
            for condicao in eval(req.params.get('condicoes') or []):
                vals_condicao = {'forma_pagto_id': condicao.get('forma_pagto_id'),
                                 'condicao_forma_id': condicao.get('condicao_forma_id'),
                                 'valor': condicao.get('valor')}
                condicoes.append([0, 0, vals_condicao])

            vals_pedido = {'company_id': company_id,
                           'tipo': 'V',
                           'pedido_item_ids': itens,
                           'pedido_condicao_ids': condicoes,
                           'pessoa_id': False,
                           'pedcap_endereco_id': False}
            vendedor_id = self.get_vendedor_id(sock)
            if vendedor_id:
                vals_pedido['vendedor_id'] = vendedor_id

            if cliente_id:
                pessoa_id = sock.execute('financeiro.cliente', 'read', cliente_id, ['pessoa_id'],
                                         {}, '_classic_write').get('pessoa_id')
                vals_pedido['pessoa_id'] = pessoa_id
                dados_pessoa = sock.execute('adm.pessoa', 'read', pessoa_id, ['endereco_nf_id',
                                  'endereco_ids'], {}, '_classic_write')
                end_id = dados_pessoa.get('endereco_nf_id')
                if end_id:
                    vals_pedido['pedcap_endereco_id'] = end_id
                elif dados_pessoa.get('endereco_ids'):
                    vals_pedido['pedcap_endereco_id'] = dados_pessoa.get('endereco_ids')[0]

            if not pedido_id:
                pedido_id = sock.execute('estoquepedidos.pedido.capa', 'create', vals_pedido,
                                          {'sig_force_company': company_id})
            else:
                item_ids = sock.execute('estoquepedidos.pedido.item', 'search',
                                        [('pedido_id', '=', pedido_id)])
                if item_ids:
                    sock.execute('estoquepedidos.pedido.item', 'unlink', item_ids)

                condicao_ids = sock.execute('estoquepedidos.pedido.condicao', 'search',
                                        [('capa_id', '=', pedido_id)])
                if condicao_ids:
                    sock.execute('estoquepedidos.pedido.condicao', 'unlink', condicao_ids)

                sock.execute('estoquepedidos.pedido.capa', 'write', pedido_id, vals_pedido)

            numero = sock.execute('estoquepedidos.pedido.capa', 'read', pedido_id,
                                  ['numero_pedido'])[0].get('numero_pedido')
            sock.execute('estoquepedidos.pedido.capa', 'pedcap_button_salvar', [pedido_id])
            sock.execute('estoquepedidos.pedido.capa', 'calcular_parcelas_pedido', [pedido_id])
            return json.dumps({'retorno': [{'pedido_id': pedido_id,
                                            'numero': numero}],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': [{'pedido_id': pedido_id,
                                            'numero': numero}]})

    @http.httprequest
    def set_faturar_pedido(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        state = ''
        try:
            sock = MySock(req.params)
            pedido_id = int(req.params.get('pedido_id') or 0)
            tipo_nota = int(req.params.get('tipo_nota') or 0)  # 1 = NF-e;  2 = NFC-e
            campos = ['state', 'pessoa_id', 'pedcap_endereco_id']
            dados_pedido = sock.execute('estoquepedidos.pedido.capa', 'read', pedido_id, campos,
                                        {}, '_classic_write')[0]
            pessoa_id = dados_pedido.get('pessoa_id')
            enderedo_id = dados_pedido.get('pedcap_endereco_id')
            if pessoa_id and not enderedo_id:
                raise Exception('Cliente sem endereço!')

            state = dados_pedido.get('state')
            if state == 'F':
                nf_ids = sock.execute('estoquepedidos.pedido.capa', 'read', pedido_id, ['pedcap_nf_ids'], {},
                                      '_classic_write')[0].get('pedcap_nf_ids')
                dados_nf = sock.execute('fiscal.nf.capa', 'read', nf_ids[0], ['numero_nf', 'serie_nf', 'state'], {},
                                        '_classic_write')
                return json.dumps({'codigo': SEM_ERRO,
                                   'mensagem': 'O pedido já se encontra faturado!',
                                   'retorno': [{'state': state,
                                                'nf': dados_nf}]})
            if state != 'V':
                sock.execute('estoquepedidos.pedido.capa', 'button_validar', [pedido_id])

            state = 'V'
            erro = False
            try:
                if tipo_nota == 1:
                    sock.execute('estoquepedidos.pedido.capa', 'button_faturar_pedido', [pedido_id], {})
                else:
                    sock.execute('estoquepedidos.pedido.capa', 'pedcap_button_faturar_pedido_nfce', [pedido_id])
            except Exception as e:
                erro = e

            nf_ids = sock.execute('estoquepedidos.pedido.capa', 'read', pedido_id, ['pedcap_nf_ids'], {},
                                  '_classic_write')[0].get('pedcap_nf_ids')
            if not nf_ids:
                raise erro

            state = 'F'
            dados_nf = sock.execute('fiscal.nf.capa', 'read', nf_ids[0], ['numero_nf', 'serie_nf', 'state'], {},
                                    '_classic_write')
            codigo = SEM_ERRO
            msg = 'ok'
            if erro:
                codigo = ERRO_INTERNO
                msg = 'Erro ao faturar nota: %s' % erro

            return json.dumps({'retorno': [{'state': state,
                                            'nf': dados_nf}],
                               'codigo': codigo,
                               'mensagem': msg})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': [{'state': state,
                                            'nf': {}}]})

    def get_vendedor_id(self, sock):
        pessoa_id = sock.execute('res.users', 'read', sock.uid, ['pessoa_id'], {},
                                 '_classic_write').get('pessoa_id')
        vendedor_id = sock.execute('vendas.vendedor', 'search', [('pessoa_id', '=', pessoa_id)])
        return vendedor_id and vendedor_id[0] or False


    @http.httprequest
    def get_pedido_venda(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            company_id = int(req.params.get('company_id'))
            codigo = req.params.get('codigo') or ''
            state = eval(req.params.get('state') or '[]')
            vendedor = eval(req.params.get('vendedor') or 'False')
            args = []
            if codigo:
                args.append(('numero_pedido', '=', 'PVE-%s' % codigo.zfill(10)))

            if vendedor:
                vendedor_id = self.get_vendedor_id(sock)
                if vendedor_id:
                    args.extend(['|', ('vendedor_id', '=', vendedor_id), ('vendedor_id', '=', False)])

            args.extend([('state', 'in', state),
                         ('tipo', '=', 'V'),
                         ('company_id', '=', company_id)])

            pedido_ids = sock.execute('estoquepedidos.pedido.capa', 'search', args, 0, 0, 'numero_pedido',
                                      {'sig_force_company': company_id})
            if codigo and not pedido_ids:
                args_pessoa = [('cpf_cnpj', '=', codigo)]
                pessoa_id = sock.execute('adm.pessoa', 'search', args_pessoa)
                if pessoa_id:
                    args = [('pessoa_id', '=', pessoa_id[0])] + args[1:]
                    pedido_ids = sock.execute('estoquepedidos.pedido.capa', 'search', args, 0, 0,
                                              'numero_pedido', {'sig_force_company': company_id})

            if not pedido_ids:
                return json.dumps({'retorno': [],
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'Nenhum pedido encontrado para esses filtros!'})

            campos = ['numero_pedido', 'data_emissao', 'valor_total', 'pessoa_id', 'state',
                      'pedido_item_ids', 'pedido_condicao_ids']
            dados = sock.execute('estoquepedidos.pedido.capa', 'read', pedido_ids, campos, {},
                                 '_classic_write')
            pessoa_ids = []
            for i in dados:
                i['valor_total'] = str(i.get('valor_total') or '0,0').replace('.', ',')
                if i['data_emissao']:
                    i['data_emissao'] = datetime.strftime(
                        datetime.strptime(i.get('data_emissao'), '%Y-%m-%d'), '%d/%m/%Y')
                else:
                    i['data_emissao'] = ''

                if i.get('pessoa_id'):
                    pessoa_ids.append(i.get('pessoa_id'))
                else:
                    i['pessoa_id'] = {}

            if pessoa_ids:
                dict_pessoa = {}
                for i in sock.execute('adm.pessoa', 'read', pessoa_ids, ['cpf_cnpj', 'name']):
                    dict_pessoa[i.get('id')] = i
                for i in dados:
                    if not i.get('pessoa_id'):
                        continue
                    i['pessoa_id'] = dict_pessoa.get(i.get('pessoa_id'))

            return json.dumps({'retorno': dados,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_extra_pedido(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            res = json.loads(self._get_cliente(req))
            cliente = {}
            if res.get('codigo') == '00' and res.get('retorno'):
                cliente = res.get('retorno')[0]

            item_ids = eval(req.params.get('item_ids') or '[]')
            campos = ['produto_id', 'grade_id', 'produto_embalagem_id', 'valor_total', 'vr_unitario',
                      'qtde_pedida']
            dados_itens = []
            cont = 0
            for i in sock.execute('estoquepedidos.pedido.item', 'read', item_ids, campos, {},
                                  '_classic_write'):
                cont += 1
                prod_emb_id = i.get('produto_embalagem_id')
                produto_id = i.get('produto_id')
                codigo = sock.execute('estoque.produto', 'read', produto_id, ['codigo']).get('codigo')
                grade_id = i.get('grade_id')
                dados_grade = sock.execute('estoque.produto.grade', 'read', grade_id, ['name', 'grade'])
                args = [('grade_id', '=', grade_id),
                        ('produto_id', '=', produto_id)]
                cb_id = sock.execute('automacao.codigo.barra', 'search', args)
                cb = 'Sem GTIM'
                if cb_id:
                    cb = sock.execute('automacao.codigo.barra', 'read', cb_id[0], ['codigo_barra']
                                      ).get('codigo_barra')
                dados_prod_embalagem = sock.execute('estoque.produto.embalagem', 'read', prod_emb_id,
                                                    ['embalagem_id'], {}, '_classic_write')
                sigla = sock.execute('estoque.embalagem', 'read',
                                     int(dados_prod_embalagem.get('embalagem_id')),
                                     ['sigla']).get('sigla')
                res = sock.execute('estoque.produto.grade', 'search', [('produto_id', '=', produto_id)])
                grades = sock.execute('estoque.produto.grade', 'read', res, ['descricao'])

                preco = str(i.get('vr_unitario') or 0)
                preco = preco.replace('.', ',')
                if preco[len(preco) - 2:len(preco) - 1] == ',':
                    preco += '0'

                vr_total = str(i.get('valor_total') or 0)
                vr_total = vr_total.replace('.', ',')
                if vr_total[len(vr_total) - 2:len(vr_total) - 1] == ',':
                    vr_total += '0'

                vals_itens = {'id': produto_id,
                              'sequencia': cont,
                              'nome': dados_grade.get('name'),
                              'cb': cb,
                              'codigo': codigo,
                              'embalagem_id': prod_emb_id,
                              'sigla_embalagem': sigla,
                              'grade_id': grade_id,
                              'grades': grades,
                              'sigla_grade': str(dados_grade.get('grade') or ''),
                              'preco': preco,
                              'quantidade': i.get('qtde_pedida'),
                              'vr_total': vr_total,
                              }
                dados_itens.append(vals_itens)

            condicao_ids = eval(req.params.get('condicao_ids') or '[]')
            dados_condicoes = []
            campos = ['valor', 'condicao_forma_id', 'forma_pagto_id']
            for i in sock.execute('estoquepedidos.pedido.condicao', 'read', condicao_ids, campos):
                forma_id = i.get('forma_pagto_id')
                condicao_id = i.get('condicao_forma_id')
                nome_condicao = sock.execute('adm.condicao.forma', 'read', condicao_id[0],
                                             ['adm_condicao_id']).get('adm_condicao_id')[1]
                dados_condicoes.append({'forma': {'id': forma_id[0],
                                                  'nome': forma_id[1]},
                                        'condicao': {'id': condicao_id[0],
                                                     'nome': nome_condicao},
                                        'valor': str(i.get('valor')).replace('.', ','),
                                        })
            return json.dumps({'retorno': [{'cliente': cliente,
                                            'itens': dados_itens,
                                            'condicoes': dados_condicoes}],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def get_departamento_item_pedido(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            departamento_id = int(req.params.get('departamento_id') or '0')
            dados_sub_deps = []
            if not departamento_id:
                departamento_ids = sock.execute('vendas.externas.departamento', 'search',
                                                [('vexdep_departamento_pai_id', '=', False)])
                if not departamento_ids:
                    return json.dumps({'codigo': ERRO_INTERNO,
                                       'mensagem': 'Nenhum departamento cadastrado em:\n'
                                                   'Estoque->Cadastro->Departamento',
                                       'retorno': []})
                dados_departamentos = []
                for d in sock.execute('vendas.externas.departamento', 'read',
                                      departamento_ids, ['name']):
                    dados_departamentos.append({'id': d.get('id'),
                                                'nome': d.get('name'),
                                                'itens': [],
                                                'sub_departamentos': []})

                return json.dumps({'retorno': dados_departamentos,
                                   'codigo': SEM_ERRO,
                                   'mensagem': 'ok'})
            else:
                sub_dep_ids = sock.execute('vendas.externas.departamento', 'search',
                                                [('vexdep_departamento_pai_id', '=', departamento_id)])
                for d in sock.execute('vendas.externas.departamento', 'read', sub_dep_ids,
                                    ['name']):
                    dados_sub_deps.append({'id': d.get('id'),
                                           'nome': d.get('name'),
                                           'itens': [],
                                           'sub_departamentos': []})

            local_id = sock.execute('estoque.parametro', 'estpar_get_fields',
                                    'local_venda_padrao_id', {})[0].get('local_venda_padrao_id')
            campos = ['name', 'vexdep_produtos_ids']
            dados_departamentos = []
            for dep in sock.execute('vendas.externas.departamento', 'read', [departamento_id],
                                    campos, {}, '_classic_write'):
                vals_deps = {'id': dep.get('id'),
                             'nome': dep.get('name'),
                             'itens': [],
                             'sub_departamentos': dados_sub_deps}
                dados_departamentos.append(vals_deps)
                # caso nao seja passado departamento nao precisa pesquisar os itens
                if not dep.get('vexdep_produtos_ids') and not dados_sub_deps:
                    if not dados_sub_deps and departamento_id:
                        return json.dumps({'codigo': ERRO_INTERNO,
                                           'mensagem': 'Departamento sem Itens!',
                                           'retorno': []})
                    continue

                produto_ids = dep.get('vexdep_produtos_ids')
                dict_produtos = {}
                for i in sock.execute('estoque.produto', 'read', produto_ids,
                                      ['codigo', 'preco_venda'], {}, '_classic_write'):
                    dict_produtos.update({i.get('id'): i})
                grade_ids = sock.execute('estoque.produto.grade', 'search',
                                      [('produto_id', 'in', produto_ids)], 0, 0, 'name')
                controle_emb = {}
                dados_produtos = []
                campos_grade = ['name', 'produto_id', 'grade', 'descricao']
                dict_grades = {}
                for i in sock.execute('estoque.produto.grade', 'read', grade_ids,  campos_grade,
                                          {}, '_classic_write'):
                    dict_grades.update({i.get('id'): i})

                dados_grades = []
                for i in grade_ids:
                    grade = dict_grades.get(i)
                    dados_grades.append(grade)

                for grade in dados_grades:
                    produto = dict_produtos.get(grade.get('produto_id'))
                    preco = str(produto.get('preco_venda') or 0.0).replace('.', ',')
                    if preco[len(preco) - 2:len(preco) - 1] == ',':
                        preco += '0'

                    args = [('local_id', '=', local_id), ('grade_id', '=', grade.get('id')),
                            ('produto_id', '=', produto.get('id'))]
                    locacao_id = sock.execute('estoque.local.locacao', 'search', args)
                    prod_emb_id = sock.execute('estoque.local.locacao', 'read', locacao_id[0],
                                               ['estllc_produto_embalagem_id'], {},
                                               '_classic_write').get('estllc_produto_embalagem_id')
                    if not controle_emb.get(prod_emb_id):
                        emb_id = sock.execute('estoque.produto.embalagem', 'read',
                                                            prod_emb_id, ['embalagem_id'],
                                                            {}, '_classic_write').get('embalagem_id')
                        if not controle_emb.get(emb_id):
                            sigla = sock.execute('estoque.embalagem', 'read', emb_id, ['sigla']).get('sigla')
                            controle_emb[emb_id] = sigla
                            controle_emb[prod_emb_id] = sigla
                        else:  # alternativa para nao ficar lendo o registro que ja foi lido uma vez
                            sigla = controle_emb.get(emb_id)
                            controle_emb[prod_emb_id] = sigla

                    else:  # alternativa para nao ficar lendo o registro que ja foi lido uma vez
                        sigla = controle_emb.get(prod_emb_id)

                    vals_itens = {'id': grade.get('produto_id'),
                                  'nome': grade.get('name'),
                                  'codigo': produto.get('codigo'),
                                  'embalagem_id': prod_emb_id,
                                  'sigla_embalagem': sigla,
                                  'grade_id': grade.get('id'),
                                  'grades': [grade],
                                  'sigla_grade': str(grade.get('grade') or ''),
                                  'preco': preco,
                                  }
                    dados_produtos.append(vals_itens)

                vals_deps['itens'].extend(dados_produtos)

            return json.dumps({'retorno': dados_departamentos,
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})

    @http.httprequest
    def set_pedido_esboco(self, req, s_action=None, **kw):
        _logger.info(
            '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            pedido_id = int(req.params.get('pedido_id') or 0)
            if not pedido_id:
                return json.dumps({'retorno': [],
                                   'codigo': ERRO_INTERNO,
                                   'mensagem': 'Informe um pedido!'})
            sock.execute('estoquepedidos.pedido.capa', 'write', pedido_id, {'state': 'R'},)
            return json.dumps({'retorno': [],
                               'codigo': SEM_ERRO,
                               'mensagem': 'ok'})

        except Exception as e:
            return json.dumps({'codigo': ERRO_INTERNO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': []})
