# -*- coding: utf-8 -*-
# $Id$

from openerp.tools.translate import _
from openerp.osv import fields, osv
from dateutil.parser import parse



# noinspection PyPep8Naming,PyAbstractClass
class wizard_nf_ajuste(osv.TransientModel):
    """
    Cria notas fiscais de ajustes
    """

    _name = 'wizard.nf.ajuste'  # wiznfa
    _description = 'Gera nota fiscal Ajuste'
    _rec_name = 'nf_referenciada_id'
    _columns = {
        'nf_referenciada_id': fields.many2one('fiscal.nf.capa', 'Nota fiscal referenciada',
                                              required=True),
        'movimento_item_ids': fields.one2many('wizard.nf.ajustes.movimento.itens',
                                              'ajuste_id', 'Item'),
        'wiznfa_tipo': fields.selection([('E', 'Entrada'), ('S', 'Saida')], 'Tipo da nota fiscal',
                                        help='Tipo da nota fiscal de ajuste: entrada ou '
                                               'saida.'),
        'wiznfa_destiatario_id': fields.many2one('adm.pessoa', 'Destinatario'),
        'wiznfa_destinatario_endereco_id': fields.many2one('adm.pessoa.endereco', 'Endereco'),
        'wiznfa_data_emissao': fields.datetime('Data emissao:'),
        'wiznfa_valor': fields.float('Valor'),
        'wiznfa_natureza_id': fields.many2one('fiscal.cfop.natureza', 'Natureza', required=True, domain="[('tipo', '=', wiznfa_tipo)]"),
    }
    _defaults = {
        'wiznfa_tipo': lambda self, cr, user, context: context.get('tipo', 'S')  # Saida eh o padrao
    }


    def _wiznfa_get_tributacao_st(self, cr, user, ids, context=None):
        """
        Item com substituicao tributaria.

        :param cr: a linha atual do cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com ID da tributacao
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type context: dict
        :return:
        :rtype:
        """
        # noinspection PyProtectedMember
        return bool(filter(lambda f: f.get('tipo_substituicao'),
                           self.pool.get('fiscal.nf.item.tributacao')._fisnit_get_dados_fiscais(
                               cr, user, ids, None, None, context).values()))

    # noinspection PyMethodMayBeStatic
    def wiznfa_onchange_tipo(self, cr, user, ids, tipo, context=None):
        """
        Remove a nota fiscal selecionada sempre que houver troca do tipo de nota fiscal.

        :param cr: o cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: o ID do formulario de nota fiscal de ajuste
        :param tipo: o tipo da nota fiscal ("E"= entrada ou "S"= saida)
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type tipo: str
        :type context: dict
        :return: nenhuma nota fiscal selecionada.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        return dict(value=dict(nf_referenciada_id=False))

    def onchange_nf_referenciada(self, cr, uid, ids, nf_id, context=None):
        """
        Retorna os itens da nota fiscal referenciada e seus respectivos valores.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da nota fiscal de referencia
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: os itens da nota fiscal referenciada.
        :rtype: list[tuple]
        """
        context = context and context.copy() or {}
        brw_nf = self.pool.get('fiscal.nf.capa').browse(cr, uid, nf_id, context)
        vals = {
            'wiznfa_data_emissao': brw_nf.data_emissao,
            'wiznfa_valor': brw_nf.vr_total_nf,
            'wiznfa_destiatario_id': brw_nf.destinatario_id.id,
            'wiznfa_destinatario_endereco_id': brw_nf.destinatario_endereco_id.id,
        }

        return {'value': vals}

    def button_gerar_nf_ajuste(self, cr, uid, ids, context=None):
        """
        Cria uma nota fiscal referenciada.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da nota fiscal de referencia
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list[long]
        :type context: dict
        :return: formulario com os dados da nota ajuste.
        :rtype: dict
        """
        context = context and context.copy() or {}
        ajuste_nf_id = self.gerar_nf_ajuste(cr, uid, ids, context)
        nf_ajuste_dados = self.pool.get('fiscal.nf.capa').read(
            cr, uid, ajuste_nf_id, ['nf_propria'], context, '_classic_write')
        # Retorna a ´view´ correta
        view = nf_ajuste_dados.get('nf_propria') and 'view_fiscal_nf_capa_form'
        view = view or 'view_fiscal_nf_capa_terceiro_form'
        result = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'fiscal', view)
        view_id = result and result[1] or False

        return dict(name='Nota fiscal referenciada', view_type='form', view_mode='form',
                    view_id=[view_id], res_model='fiscal.nf.capa', res_id=ajuste_nf_id,
                    type='ir.actions.act_window', context=context)

    def gerar_nf_ajuste(self, cr, uid, ids, context=None):
        """
        Cria a nota fiscal ajuste.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da wizard
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: os formulario com os dados da nota fiscal ajuste.
        :rtype: object
        """
        context = context and context.copy() or {}
        obj_fiscal_nf = self.pool.get('fiscal.nf.capa')
        context.update({'nota_referenciada': True, 'proprio': True})
        brw = self.browse(cr, uid, ids[0], context=context)
        nf_id = brw.nf_referenciada_id.id
        if brw.wiznfa_tipo == 'S':
            context['nota_fiscal_saida'] = True
        else:
            context['nota_fiscal_entrada'] = True
        fields2read = ['quantidade_volume', 'numero_conhecimento', 'prazo', 'marca', 'especie',
                       'serie_frete', 'numero_volume', 'fisnfc_infcomp_contribuinte', 'tipo',
                       'fisnfc_infcomp_fisco', 'fisnfc_cod_consumo_id', 'modelo_55', 'state',
                       'gerar_xml', 'nf_propria', 'fisnfc_tipo_servico', 'data_emissao', 'frete',
                       'hora_emissao', 'daeta_emissao_frete', 'data_vencimento_frete', 'prazo_tipo',
                       'tipo_frete', 'tipo_venc_frete', 'fisnfc_energia_tipo_ligacao', 'company_id',
                       'vendedor_id', 'natureza_id', 'fisnfc_energia_grupo_tensao', 'modelo_nf_id',
                       'modelo_nf_series_id', 'tributacao_id', 'modelo_nf_frete', 'destinatario_id',
                       'transportador_id', 'endereco_coleta_id', 'endereco_entrega_id',
                       'vendedor_externo_id', 'endereco_cobranca_id', 'destinatario_endereco_id',
                       'fisnfc_nf_frete_id', 'nfe_processo_emissao', 'nfe_forma_impressao_danfe',
                       'nfe_versao_processo_emissao', 'nfe_forma_emissao', 'serie_nf', 'numero_nf',
                       'fisnfc_consumidor']
        dados_nf_capa = obj_fiscal_nf.read(cr, uid, nf_id, fields2read, context, '_classic_write')
        propria = brw.wiznfa_tipo == 'E' or dados_nf_capa.get('nf_propria')
        vals_itens = list()
        tot_base_icms_substituicao = tot_icms_substituicao = 0
        cfop_id = brw.wiznfa_natureza_id.cfop_id.id
        # Recupera as informacoes dos itens da nota referenciada
        tributacoes = []
        for brw_item in brw.movimento_item_ids:
            item_comp = {'produto_id': brw_item.produto_id.id,
                         'quantidade': brw_item.quantidade,
                         'vr_total_item': brw_item.vr_total_item,
                         'local_id': brw_item.produto_id.local_ids[0].id,
                         'vr_total_item_rateado': brw_item.vr_total_item,
                         'nf_id': nf_id,
                         'cfop_id': cfop_id,
                         'grade_id': brw_item.produto_id.produto_grade_ids[0].id,
                         'produto_embalagem_id': brw_item.produto_id.produto_embalagem_ids[0].id}
            # Ajuste dos valores da tributacao do ICMS
            tributacao = {}
            is_icms = brw_item.wiznami_encargo_id.id
            tributacao['encargo_id'] = brw_item.wiznami_encargo_id.id
            if is_icms:
                tributacao['cst_id'] = brw_item.wiznami_cst_id.id
                tributacao['tributacao_id'] = brw_item.wiznami_tributacao_id.id
                tributacao['cfop_id'] = cfop_id
            tributacao['aliquota'] = brw_item.aliquota
            tributacao['vr_base_calculo'] = brw_item.vr_base_calculo
            tributacao['vr_encargo'] = brw_item.vr_icms
            tributacao['vr_icms_substituicao'] = brw_item.vr_icms_substituicao
            tributacao['vr_base_icms_substituicao'] = brw_item.vr_base_icms_substituicao
            tributacoes.append([0, False, tributacao])

            tributacao = {}
            _encargos = self.pool.get('fiscal.encargo').search(cr, uid, [('sigla', '=', 'PIS')])
            if not _encargos:
                raise osv.except_osv(_('Atencao!'),
                                     _('Encargo PIS nao cadastrado!'))
            tributacao['encargo_id'] = _encargos[0]
            args = [('encargo_id', '=', tributacao['encargo_id']),('codigo', '=', '99')]
            _CSTs = self.pool.get('fiscal.cst').search(cr, uid, args, context=context)
            if not _CSTs:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao ha CST cadastrada para o PIS com codigo 99!'))
            tributacao['cst_id'] = _CSTs[0]
            args = [('cst_id', '=', tributacao['cst_id']), ('encargo_id', '=', tributacao['encargo_id'])]
            _tributacoes = self.pool.get('fiscal.tributacao').search(cr, uid, args, context=context)
            if not _tributacoes:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao ha tributacao cadastrada para o PIS com CST 99!'))
            tributacao['tributacao_id'] = _tributacoes[0]
            tributacao['cfop_id'] = cfop_id
            tributacao['aliquota'] = 0
            tributacao['vr_base_calculo'] = 0
            tributacao['vr_encargo'] = 0
            tributacoes.append([0, False, tributacao])

            tributacao = {}
            _encargos = self.pool.get('fiscal.encargo').search(cr, uid, [('sigla', '=', 'COFINS')])
            if not _encargos:
                raise osv.except_osv(_('Atencao!'),
                                     _('Encargo PIS nao cadastrado!'))
            tributacao['encargo_id'] = _encargos[0]
            args = [('encargo_id', '=', tributacao['encargo_id']),('codigo', '=', '99')]
            _CSTs = self.pool.get('fiscal.cst').search(cr, uid, args, context=context)
            if not _CSTs:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao ha CST cadastrada para o COFINS com codigo 99!'))
            tributacao['cst_id'] = _CSTs[0]
            args = [('cst_id', '=', tributacao['cst_id']), ('encargo_id', '=', tributacao['encargo_id'])]
            _tributacoes = self.pool.get('fiscal.tributacao').search(cr, uid, args, context=context)
            if not _tributacoes:
                raise osv.except_osv(_('Atencao!'),
                                     _('Nao ha tributacao cadastrada para o COFINS com CST 99!'))
            tributacao['tributacao_id'] = _tributacoes[0]
            tributacao['cfop_id'] = cfop_id
            tributacao['aliquota'] = 0
            tributacao['vr_base_calculo'] = 0
            tributacao['vr_encargo'] = 0
            tributacoes.append([0, False, tributacao])

            tot_icms_substituicao += brw_item.vr_icms_substituicao or 0
            tot_base_icms_substituicao += brw_item.vr_base_icms_substituicao or 0

            # Inclusao do item na nota de ajuste
            vals_itens.append([0, False, dict(item_comp, tributacoes_ids=tributacoes)])

        if not vals_itens:
            raise osv.except_osv(_('Atencao!'),
                                 _('Necessario informar pelo menos uma alteracao para que seja '
                                   'gerada uma nota fiscal de ajuste. As alteracoes podem ser '
                                   'no valor total do item, valor do ICMS, aliquota do ICMS, '
                                   'quantidade do item, valor base do ICMS da substituicao ou '
                                   'valor do ICMS da substituicao.'))

        # O ID da nota de referencia nao sera utilizado na criacao da nova nota
        del dados_nf_capa['id']

        emissao = parse(dados_nf_capa.get('data_emissao')).strftime('%d-%m-%Y')
        dados_nf_capa['fisnfc_infcomp_contribuinte'] = '' if not dados_nf_capa.get(
            'fisnfc_infcomp_contribuinte') else unicode(
            dados_nf_capa.get('fisnfc_infcomp_contribuinte'))
        dados_nf_capa['fisnfc_infcomp_contribuinte'] += '\nNota fiscal referenciada: \nNumero: ' \
                                                        '%s. Serie: %s. Data de emissao: %s.\n' % (
            dados_nf_capa.get('numero_nf'), dados_nf_capa.get('serie_nf'), emissao)
        del dados_nf_capa['serie_nf']
        del dados_nf_capa['numero_nf']

        # Cria a nota fiscal ajuste
        dados_nf_ajuste = {
        }
        modelo_padrao = dados_nf_capa.get('modelo_nf_series_id')
        # Dados de NF-e
        dados_nf_ajuste.update({
            'company_id': dados_nf_capa.get('company_id'),
            'state': '1',
            'estoque_movimento_itens_ids': vals_itens,
            'nf_propria': propria,
            'fisnfc_nf_referencia_ids': [[6, False, [nf_id]]],
            'vr_icms_substituicao': tot_icms_substituicao,
            'vr_base_icms_substituicao': tot_base_icms_substituicao,
            'modelo_nf_series_id': modelo_padrao,
            'nfe_versao_processo_emissao': dados_nf_capa.get('nfe_versao_processo_emissao'),
            'nfe_forma_impressao_danfe': '1',
            'nfe_forma_emissao': '1',
            'fisnfc_consumidor': False,
            'modelo_55': True,
            'tipo': brw.wiznfa_tipo,
            'destinatario_endereco_id': dados_nf_capa.get('destinatario_endereco_id'),
            'destinatario_id': dados_nf_capa.get('destinatario_id'),
            'natureza_id': brw.wiznfa_natureza_id.id,
            'data_emissao': fields.datetime.now(),
        })
        vals_aux = obj_fiscal_nf.dados_onchange_modelo(
            cr, uid, modelo_padrao, propria, context=context)
        dados_nf_ajuste.update(vals_aux)
        dados_nf_ajuste.update({'finalidade_emissao': '3',})
        ajuste_nf_id = obj_fiscal_nf.create(cr, uid, dados_nf_ajuste, context=context)

        # Inclui a referencia da nota fiscal de ajuste
        obj_fiscal_nf.write(
            cr, uid, nf_id, dict(fisnfc_nf_referencia_ids=[[6, False, [ajuste_nf_id]]]), context)

        return ajuste_nf_id


# noinspection PyAbstractClass,PyPep8Naming
class wizard_nf_ajuste_movimento_itens(osv.TransientModel):
    """
    Itens de movimento da nota fiscal ajuste.
    """
    _name = 'wizard.nf.ajustes.movimento.itens'  # prefixo: wiznami

    _description = 'Itens da nota fiscal de ajuste'
    _columns = {
        'produto_id': fields.many2one('estoque.produto', 'Produto', ondelete="restrict",
                                      required=True),
        'produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem',
                                                ondelete='restrict',
                                                domain="[('produto_id', '=', produto_id)]"),
        'quantidade': fields.float('Quantidade', digits=(12, 3)),
        'vr_total_item': fields.float('Valor total', digits=(15, 2)),
        'vr_icms': fields.float('Valor ICMS', digits=(15, 2)),
        'vr_base_calculo': fields.float('Valor Base Calculo', digits=(15, 2)),
        'vr_base_icms_substituicao': fields.float('Valor base do ICMS da substituicao',
                                                  digits=(14, 2)),
        'vr_icms_substituicao': fields.float('Valor do ICMS da substituicao', digits=(14, 2)),
        'aliquota': fields.float('Aliquota ICMS', digits=(15, 2)),
        'movimento_item_id': fields.many2one('estoque.movimento.itens', 'Item'),
        'ajuste_id': fields.many2one('wizard.nf.ajuste', 'ajuste'),
        'local_id': fields.many2one('estoque.local', 'Local', ondelete="restrict"),
        'wiznami_encargo_id': fields.many2one('fiscal.encargo', 'Encargo', required=True),
        'wiznami_cst_id': fields.many2one('fiscal.cst', 'CST', required=True),
        'wiznami_tributacao_id': fields.many2one('fiscal.tributacao', 'Tributacao', required=True),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade',
                                    domain="[('produto_id', '=', produto_id)]"),
        'tipo_substituicao': fields.boolean('Item com ST')
    }
    _defaults = {
        'tipo_substituicao': False,
        'wiznami_encargo_id': lambda s, c, u, x: s.pool.get('fiscal.encargo').search(
            c, u, [('sigla', '=', 'ICMS')], context=x)[0],
    }

    def wiznami_onchange_cst(self, cr, user, ids, cst_id, context=None):
        context = context and context.copy() or {}
        tributacao_ids = self.pool.get('fiscal.tributacao').search(
            cr, user, [('cst_id', '=', cst_id)], context=context)
        tributacao_id = len(tributacao_ids) == 1 and tributacao_ids[0]

        return {'value': {
            'wiznami_tributacao_id': tributacao_id,
            'tipo_substituicao': self.pool.get('fiscal.cst').browse(
                cr, user, cst_id, context=context).tipo in ('U', 'TU')
        }}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
