# -*- encoding: utf-8 -*-

import openerp.addons.web.http as http
from logging import getLogger
import json
from ..web_sig.control.utils import websig_MySock
from SigUtils.sigerp import db_sig
from datetime import datetime

_debug = True

_logger = getLogger("gesbas")



class SeuControler(http.Controller):
    _cp_path = '/gesbas'

    @http.httprequest
    def gesbas_set_acessos(self, req, s_action=None, **kw):
        """
        Inserie acessos ao sigerp
        recebe em req.params(dicionario):
        base: nome do banco de dados
        data: data dos acessos formato YYYY-MM-DD
        qtd: quantidade de acessos
        retorna:  {status:'', mensagem:''}
           status 0 se tiver erro e a
           status 1 se deu certo
           qdo hover erro retorna tambem a mensagem de erro
        layout para dados
         - sem versao: data_hora;db;qtd acessos
         - versao 1: versao;data;hora;db;qtdacessos
        """
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            _base = req.params.get('base') or 'erp'
            _dados = req.params.get('dados') or ''
            if not _dados:
                return json.dumps({'status': '0', 'mensagem': 'Nao informado dados a incluir'})
            if _dados[0] == '1':  # Versao 1
                try:
                    _dados = _dados.replace('%3B', ';')
                    _versao, _data, _hora, _base_acessos, _qtd = _dados.split(';')
                    _data_hora = _data + ' ' + _hora
                    _qtd = int(_qtd)
                except:
                    return json.dumps({'status': '1', 'mensagem': 'Dados informados invalidos: %s' % _dados})
            else:  # Sem versao
                try:
                    _data_hora, _base_acessos, _qtd = _dados.split(';')
                    _data_hora = _data_hora.replace('_', ' ') + ':00'
                    _data_hora = _data_hora[:13]+':' + _data_hora[14:]
                    _qtd = int(_qtd)
                except:
                    return json.dumps({'status': '1', 'mensagem': 'Dados informados invalidos: %s' % _dados})

            _db = db_sig(_base)
            _uid = _db.dbsSelect('res_users', ['id'], [('login', '=', 'gestao_user')], limit=1)
            if _uid:
                _uid = _uid[0].get('id')
                _db.uid = _uid
            _args = [('gesbas_clie_base', '=', _base_acessos)]
            _base_data = _db.dbsSelect('gesbas_cliente',
                                       ['id','gesbas_clie_pessoa_id','gesbas_clie_servidor','gesbas_clie_porta'],
                                       _args,
                                       limit=1)
            if not _base_data:
                return json.dumps({'status': '2', 'mensagem': 'Base nao cadastrada'})
            _pessoa_id = _base_data[0].get('gesbas_clie_pessoa_id')
            _args = [('gesbas_aces_data_hora', '=', _data_hora),
                     ('gesbas_aces_pessoa_id', '=', _pessoa_id)]
            _gesbas_ids = _db.dbsSelect('gesbas_acessos', ['id'], _args, limit=1)
            if _gesbas_ids:
                _ids = tuple([x.get('id') for x in _gesbas_ids])
                _db.dbsDelete('gesbas_acessos', [('id', 'in', _ids)])
            vals = {'gesbas_aces_data_hora': _data_hora,
                    'gesbas_aces_pessoa_id': _pessoa_id,
                    'gesbas_aces_base': _base_acessos,
                    'gesbas_aces_quantidade': _qtd,
                    'gesbas_aces_servidor': _base_data[0].get('gesbas_clie_servidor'),
                    'gesbas_aces_porta': _base_data[0].get('gesbas_clie_porta'),
                    'gesbas_aces_dia': datetime.strptime(_data_hora, '%Y-%m-%d %H:%M:%S').strftime('%Y-%m-%d'),  # ano-mes-dia
                    'gesbas_aces_semana': datetime.strptime(_data_hora, '%Y-%m-%d %H:%M:%S').strftime('%Y-%U'),  # ano-semana
                    'gesbas_aces_mes': datetime.strptime(_data_hora, '%Y-%m-%d %H:%M:%S').strftime('%Y-%m'),  # ano-mes
            }

            res = _db.dbsInsert('gesbas_acessos', vals)
            if not res:
                _db.conexao.roolback()
            else:
                _db.conexao.commit()
            _db.conexao.close()
            if not res:
                _logger.info('Erro em gesbas_set_acessos: Nao consegui incluir a informacao')
                return json.dumps({'status': '3', 'mensagem': 'Nao consegui incluir a informacao'})
            return json.dumps({'status': '0', 'mensagem': ''})
        except Exception as e:
            _db.conexao.close()
            try:
                if e.faultCode:
                    _logger.info('Erro em gesbas_set_acessos: %s' % e.faultCode)
                    return json.dumps({'status': '9', 'mensagem': 'Erro: %s' % e.faultCode})
            except:
                pass
            try:
                if e.message:
                    _logger.info('Erro em gesbas_set_acessos: %s' % e.message)
                    return json.dumps({'status': '9', 'mensagem': 'Erro: %s' % e.message})
            except:
                pass
            _logger.info('Erro em gesbas_set_acessos: %s' % e)
            return json.dumps({'status': '9', 'mensagem': 'Erro: %s' % e})

    @http.httprequest
    def gesbas_set_estatisticas(self, req, s_action=None, **kw):
        """
        Inserie estatisticas de uso de servidores sigerp
        recebe em req.params(dicionario):
        base: nome do banco de dados
        data: data dos acessos formato YYYY-MM-DD
        qtd: quantidade de acessos
        retorna:  {status:'', mensagem:''}
           status 0 se tiver certo e a
           status 1-9 se deu erro
           qdo hover erro retorna tambem a mensagem de erro
        """
        try:
            _logger.info(
                '==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
            _base = req.params.get('base') or 'erp'
            _dados = req.params.get('dados') or ''
            if not _dados:
                if _debug:
                    _logger.info('======== erro: Nao informado dados a incluir ')
                return json.dumps({'status': '1', 'mensagem': 'Nao informado dados a incluir'})
            try:
                _dados = _dados.split(';')
                if _dados[0] == '2':
                    _servidor = _dados[1]
                    _data = _dados[2]
                    _cpu_cores = _dados[3]
                    _cpu_uso_1 = float(_dados[4])
                    _cpu_uso_5 = float(_dados[5])
                    _cpu_uso_15 = float(_dados[6])
                    _cpu_avg_1 = float(_dados[7])
                    _cpu_avg_5 = float(_dados[8])
                    _cpu_avg_15 = float(_dados[9])
                    _dsk_total = _dados[10]
                    _dsk_perc = float(_dados[11])
                    _dsk_read_count = _dados[12]
                    _dsk_write_count = _dados[13]
                    _dsk_read_bytes = _dados[14]
                    _dsk_write_bytes = _dados[15]
                else:
                    if _debug:
                        _logger.info('======== erro: Layout desconhecido ')
                    return json.dumps({'status': '1', 'mensagem': 'Layout desconhecido %s' % _dados[0]})
            except:
                if _debug:
                    _logger.info('======== erro: Dados informados invalidos ')
                return json.dumps({'status': '1', 'mensagem': 'Dados informados invalidos: %s' % _dados})
            _db = db_sig(_base)
            _uid = _db.dbsSelect('res_users', ['id'], [('login', '=', 'gestao_user')], limit=1)
            if _uid:
                _uid = _uid[0].get('id')
                _db.uid = _uid
            _args = [('name', '=', _servidor), ('gesbas_serv_data_hora', '=', _data)]
            _gesbas_ids = _db.dbsSelect('gesbas_servidores', ['id'], _args)
            if _gesbas_ids:
                if _debug:
                    _logger.info('======== info: Ja cadastrado, vou excluir para incluir novamente: %s' % _args)
                _ids = tuple([x.get('id') for x in _gesbas_ids])
                _db.dbsDelete('gesbas_servidores', [('id', 'in', _ids)])
            vals = {
                'name': _servidor,
                'gesbas_serv_data_hora': _data,
                'gesbas_serv_cpu_cores': _cpu_cores,
                'gesbas_serv_cpu_1_min': _cpu_uso_1,
                'gesbas_serv_cpu_5_min': _cpu_uso_5,
                'gesbas_serv_cpu_15_min': _cpu_uso_15,
                'gesbas_serv_avg_1_min': float(_cpu_avg_1) / int(_cpu_cores),
                'gesbas_serv_avg_5_min': float(_cpu_avg_5) / int(_cpu_cores),
                'gesbas_serv_avg_15_min': float(_cpu_avg_15) / int(_cpu_cores),
                'gesbas_serv_dsk_uso': _dsk_total,
                'gesbas_serv_dsk_per': _dsk_perc,
                'gesbas_serv_dsk_read_count': _dsk_read_count,
                'gesbas_serv_dsk_read_bytes': _dsk_read_bytes,
                'gesbas_serv_dsk_write_count': _dsk_write_count,
                'gesbas_serv_dsk_write_bytes': _dsk_write_bytes,
            }
            res = _db.dbsInsert('gesbas_servidores', vals)
            if not res:
                _db.conexao.roolback()
            else:
                _db.conexao.commit()
            _db.conexao.close()
            if not res:
                if _debug:
                    _logger.info('======== erro: Nao consegui incluir a informacao ')
                return json.dumps({'status': '1', 'mensagem': 'Nao consegui incluir a informacao'})
            if _debug:
                _logger.info('======== Ok,registrado')
            return json.dumps({'status': '0', 'mensagem': ''})
        except Exception as e:
            _db.conexao.close()
            if _debug:
                _logger.info('======== erro: Exception ')
            try:
                if e.faultCode:
                    return json.dumps({'status': '9', 'mensagem': 'Erro: %s' % e.faultCode})
            except:
                pass
            try:
                if e.message:
                    return json.dumps({'status': '9', 'mensagem': 'Erro: %s' % e.message})
            except:
                pass
            return json.dumps({'status': '9', 'mensagem': 'Erro: %s' % e})
