# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from datetime import datetime
from openerp.tools.translate import _


class wiz_importa_ponto_fortimeflex(osv.Model):
    # prefixo: wizipf_

    _name = 'wiz.importa.ponto.fortimeflex'
    _columns = {
        'wizipf_arquivo': fields.binary('Arquivo', required=True),
    }

    def wizipf_button_importa(self, cr, uid, ids, context=None):
        """
        O método lê os dados do arquivo selecionado, verifica se o mesmo é válido e persiste com os
        dados na tabela rh_marcacao. Essa tabela é populada com o id do funcionário, data e hora do
        ponto batido.
        :param cr: cursor no banco de dados
        :param uid: identificador do usuário, id da tabela res_uses
        :param ids: lista de ids da transient atual
        :param context: dicionário com valores contextuais
        :return: Retorna False se não importou nada ou a chamada para uma nova view informando do
        sucesso da importação
        """
        context = context and context.copy() or {}
        res = False
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        obj_rh_funcionario = self.pool.get('rh.funcionario')
        obj_rh_marcacao = self.pool.get('rh.marcacao')
        importa_ponto_dados = self.read(cr, uid, ids, [], context=context)
        criou_marcacao = False
        for dado in importa_ponto_dados:
            dados_arquivo = dado.get('wizipf_arquivo')
            dados_arquivo = dados_arquivo.decode('base64')
            dados_arquivo = dados_arquivo.split('\r\n')
            for ponto in dados_arquivo:
                if not ponto:
                    continue
                if len(ponto) != 18:
                    raise osv.except_osv(_('Atencao'), _('Arquivo invalido'))
                try:
                    int(ponto[:6])
                    int(ponto[6:14])
                    int(ponto[14:18])
                except:
                    raise osv.except_osv(_('Atencao'), _('Arquivo invalido'))
                codigo_pessoa = int(ponto[:6])
                data = ponto[6:14]
                hora = ponto[14:18]
                pessoa_ids = obj_adm_pessoa.search(cr, uid, [('codigo', '=', codigo_pessoa)], context=context)
                if not pessoa_ids:
                    continue
                pessoa_id = pessoa_ids[0]
                funcionario_ids = obj_rh_funcionario.search(cr, uid, [('pessoa', '=', pessoa_id)], context=context)
                if not funcionario_ids:
                    continue
                funcionario_id = funcionario_ids[0]
                data_hora = data + hora
                data = datetime.strptime(data, '%Y%m%d')
                data_hora = datetime.strptime(data_hora, '%Y%m%d%H%M')
                hora = data_hora - data
                hora = hora.total_seconds() / 3600
                data = data.strftime('%Y-%m-%d')
                rh_marcacao_ids = obj_rh_marcacao.search(
                    cr, uid, [('rh_funcionario_id', '=', funcionario_id),
                              ('rh_data', '=', data),
                              ('rh_horario', '=', hora)], context=context)
                if rh_marcacao_ids:
                    continue
                vals_marcacao = {
                    'rh_funcionario_id': funcionario_id,
                    'rh_data': data,
                    'rh_horario': hora
                }
                obj_rh_marcacao.create(cr, uid, vals_marcacao, context)
                criou_marcacao = True
        if criou_marcacao:
            obj_model_data = self.pool.get('ir.model.data')
            reference = obj_model_data.get_object_reference(
                cr, uid, 'rhponto', 'view_wiz_importa_ponto_fortimeflex_warning_form')
            view_id = reference and reference[1] or False
            return {
                'name': _('Atencao'),
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view_id],
                'res_model': 'wiz.importa.ponto.fortimeflex',
                'type': 'ir.actions.act_window',
                'target': 'new',
            }
        return res
