# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from openerp.addons.supervisor import print_log
import re
import json
import requests


class supervisor_cfg_configuracao(osv.Model):
    _inherit = 'supervisor.cfg.configuracao'

    @staticmethod
    def supcfg_conecta_erp(config, funcao, modulo='nfe', dados=[], timeout=120):
        host = config.get('supcfc_host_erp')
        if host and host[-1] == '/':
            host = host[:-1]
        user = 'ecf_remoto'
        senha = 'ecf_remoto_77813'
        try:
            res = requests.post('%s/%s/%s' % (host, modulo, funcao),
                                auth=(user, senha),
                                data={'db': config.get('supcfc_base_erp'),
                                      'pwd': senha,
                                      'user': user,
                                      'company_id': config.get('supcfc_nro_empresa'),
                                      'dados': str(dados),
                                      },
                                timeout=timeout)
        except Exception as e:
            raise osv.except_osv(_('Atencao!'), 'Erro ao conectar com o ERP:\n%s' % e)
        try:
            res = json.loads(res.content)
        except Exception as e:
            raise osv.except_osv(_('Atencao!'), 'Erro ao descodificar a resposta do servidor do ERP:\n%s' % e)

        if res.get('codigo') != '00':
            raise osv.except_osv(_('Atencao!'), res.get('mensagem').decode('string_escape'))

        return res.get('retorno')

    def supcfc_testar_conexao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        for i in self.read(cr, uid, ids, ['supcfc_base_erp', 'supcfc_host_erp',
                                          'supcfc_nro_empresa'], context=context):
            try:
                self.supcfg_conecta_erp(i, 'teste', 'valida', timeout=5)
            except Exception as e:
                msg = str(e)
                if isinstance(e, osv.except_osv):
                    msg = e.value

                raise osv.except_osv(_('Erro de conexao!'), msg)

        reference = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'supervisor',
                                                                'view_supervisor_cfg_configuracao_msg_form')
        view_id = reference and reference[1] or False
        return {'name': _('Teste de Conexao'),
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'supervisor.cfg.configuracao',
                'view_id': [view_id],
                'context': context,
                'target': 'new',
                'type': 'ir.actions.act_window'}

    def cron_supervisor_exportar_nfce_erp(self, cr, uid, context=None):
        context = context and context.copy() or {}
        if self.pool.get('supervisor.mov.acao.pendente').search(cr, uid,
                                                                [('supape_tabela', '=', 'Importando do ERP')]):
            print_log('Exportacao Cancelada! Aguardando terminar a importacao de dados do ERP!', 'I')
            return True

        ids = self.search(cr, uid, [], context=context)
        if not ids:
            return True

        for i in self.read(cr, uid, ids, ['supcfc_base_erp', 'supcfc_host_erp',
                                          'supcfc_nro_empresa'], context):
            # nao executa a importacao se faltar um dos parametros
            if not i.get('supcfc_base_erp') or not i.get('supcfc_host_erp'):
                continue
            try:
                self.supcfg_conecta_erp(i, 'desativar_cron_importacao_nfce')
                dados_nfce = self.supcfg_get_dados_capa(cr)
                if not dados_nfce:
                    return True

                print_log('Exportando NFC-e(s) para o ERP: %s nota(s)' % len(dados_nfce), 'I')
                capa_ids = map(lambda x: '%s' % x.get('id'), dados_nfce)
                dict_corpos = self.supcfg_get_dados_corpos(cr, capa_ids, context)
                dict_finalizacao = self.supcfg_get_dados_finalizacao(cr, capa_ids, context)
                dict_tributacao = self.supcfg_get_dados_tributacao(cr)
            except Exception as e:
                print_log(_('Erro ao selecionar notas para exportar!:\n%s') % e, 'E')
                return True

            try:
                res = self.supcfg_exportar_notas(cr, dados_nfce, dict_corpos, dict_finalizacao,
                                           dict_tributacao, i)
            except Exception as e:
                print_log(_('Erro comunicando com servidor do ERP!:\n%s') % e, 'E')
                return True

            try:
                self.supcfg_atualizar_cupom_servidor(cr, dados_nfce, res)
            except Exception as e:
                err = self.pool.get('supervisor.util').suputi_remover_acentuacao(e)
                print_log(_('Erro ao atualizar a tabela "Cupom Servidor"!:\n%s') % err, 'E')
                return True

        return True

    def supcfg_get_dados_capa(self, cr):
        sql = """SELECT smc.id,
                        smc.supcap_nro_cupom,
                        smc.supcap_loja,
                        smc.supcap_caixa,
                        smc.supcap_tipo,
                        smc.supcap_data_cupom,
                        smc.supcap_data_inicio,
                        smc.supcap_cliente,
                        smc.supcap_alteracao,
                        smc.supcap_valor_desc_acres,
                        smc.supcap_total_cupom,
                        smc.supcap_operador_id,
                        smc.supcap_cod_vendedor,
                        scv.name                 AS nome_vendedor,
                        smc.supcap_lote,
                        smc.supcap_status,
                        smc.supcap_xml_proc,
                        smc.supcap_protocolo,
                        smc.supcap_protocolo_cancelamento,
                        smc.supcap_xml_gerado,
                        smc.supcap_xml_cancelamento,
                        smc.supcap_chave_acesso,
                        smc.supcap_chave_cte,
                        smc.supcap_state,
                        smc.supcap_ambiente,
                        smc.supcap_numero_pedido,
                        sco.supope_codigo
                 FROM supervisor_mov_capa                 AS smc
                 INNER JOIN supervisor_mov_cupom_servidor AS smcs ON smcs.supcse_capa_id = smc.id
                 INNER JOIN supervisor_cad_operadores     AS sco  ON sco.id = smc.supcap_operador_id
                 LEFT  JOIN supervisor_cad_vendedor       AS scv  ON scv.supven_codigo = smc.supcap_cod_vendedor
                 WHERE smc.supcap_state IN ('3', '2E', '4', '6', '7', 'C', 'I', 'D')
                   AND smcs.supcse_enviado = FALSE
                   AND (smcs.supcse_tipo_envio = '0' or smcs.supcse_tipo_envio IS NULL)
                 ORDER BY smc.supcap_data_cupom
                 LIMIT {limit}
                """
        cr.execute(sql.format(limit=50))
        dados = cr.dictfetchall()
        return dados

    def supcfg_get_dados_corpos(self, cr, capa_ids, context=None):
        context = context and context.copy() or {}
        sql = """
            SELECT smc.supcor_capa_id,
                   smc.supcor_item,
                   smc.supcor_preco_unitario,
                   smc.supcor_quantidade,
                   smc.supcor_preco_total,
                   smc.supcor_venda_cancelamento,
                   smc.supcor_tributacao_ecf,
                   smc.supcor_tributacao_retag,
                   smc.supcor_sequencia,
                   smc.supcor_desconto_item,
                   smc.supcor_desconto_item_sem_rat,
                   smc.supcor_acrescimo_item,
                   smc.supcor_acrescimo_item_sem_rat,
                   smc.supcor_aliquota,
                   smc.supcor_base,
                   smc.supcor_cod_vendedor,
                   smc.supcor_nome_vendedor,
                   smc.supcor_vr_fcp_st,
                   smc.supcor_vr_icms,
                   smc.supcor_base_icms,
                   smc.supcor_codigo_trib_icms,
                   smc.supcor_vr_pis,
                   smc.supcor_base_pis,
                   smc.supcor_codigo_trib_pis,
                   smc.supcor_vr_cofins,
                   smc.supcor_base_cofins,
                   smc.supcor_codigo_trib_cofins,
                   smc.supcor_codigo_cfop,
                   sci.supite_codigo_anp        AS codigo_anp,
                   sci.supite_producao          AS producao,
                   sci.id                       AS item_id,
                   icms.suptnf_tipo_tributacao  AS tipo_trib,
                   icms.suptnf_base             AS base_icms,
                   icms.id                      AS icms_id,
                   smca.supcap_data_cupom       AS data_nota

            FROM   supervisor_mov_corpos             AS smc
            LEFT JOIN supervisor_cad_itens           AS sci  ON sci.supite_codigo_item = smc.supcor_item
            INNER JOIN supervisor_mov_capa           AS smca ON smca.id = smc.supcor_capa_id 
            LEFT JOIN supervisor_cad_tributacao_nfce AS icms ON icms.suptnf_codigo = sci.supite_tributacao_icms_id 
            WHERE smc.supcor_capa_id IN (%s)
            ORDER BY smc.supcor_sequencia
        """ % ','.join(capa_ids)
        cr.execute(sql)
        dict_corpos = {}
        cbs = []  # lista de codigo de barras
        for i in cr.dictfetchall():
            cbs.append('%r' % str(i.get('supcor_item')))
            capa_id = i.get('supcor_capa_id')
            if not dict_corpos.get(capa_id):
                dict_corpos[capa_id] = [i]
            else:
                dict_corpos[capa_id].append(i)

        if not dict_corpos:
            return {}.fromkeys(capa_ids, [])

        return dict_corpos

    def supcfg_get_dados_finalizacao(self, cr, capa_ids, context=None):
        context = context and context.copy() or {}
        dict_fin = {}
        sql = """
            SELECT smf.id AS fechamento_id,
                   smf.supfec_capa_id,
                   smf.supfec_finalizadora,
                   smf.supfec_nome_finalizadora,
                   smf.supfec_valor,
                   smf.supfec_sequencia, 
                   smf.supfec_tipo_finalizadora, 
                   smf.supfec_troco,
                   smf.supfec_nro_banco, 
                   smf.supfec_agencia, 
                   smf.supfec_nro_cheque,
                   smf.supfec_conta, 
                   smf.supfec_juros, 
                   smf.supfec_cnpj, 
                   smf.supfec_aut_cartao,
                   smv.supfve_valor_parcela,
                   smv.supfve_vencimento,
                   smca.supcap_data_cupom,
                   smca.supcap_loja, 
                   smca.supcap_caixa,
                   smca.supcap_cliente
            FROM supervisor_mov_fechamentos      AS smf
            LEFT JOIN supervisor_mov_vencimentos AS smv  ON smv.supfve_fechamentos_id = smf.id
            INNER JOIN supervisor_mov_capa       AS smca ON smf.supfec_capa_id = smca.id
            WHERE smf.supfec_capa_id IN (%s)
            ORDER BY smf.supfec_capa_id, smf.supfec_sequencia
        """ % ','.join(capa_ids)
        cr.execute(sql)
        for i in cr.dictfetchall():
            capa_id = i.get('supfec_capa_id')
            if not dict_fin.get(capa_id):
                dict_fin[capa_id] = [i]
            else:
                dict_fin[capa_id].append(i)

        return dict_fin

    def supcfg_get_dados_tributacao(self, cr):
        cr.execute('SELECT * FROM supervisor_cad_tributacao_nfce')
        res = {}
        for i in cr.dictfetchall():
            res.update({i.get('suptnf_codigo'): i})

        return res

    def supcfg_exportar_notas(self, cr, dados_nfce, dict_corpos, dict_finalizacao,
                              dict_tributacao, params):
        dados = [{'dados_nfce': dados_nfce,
                  'dict_corpos': dict_corpos,
                  'dict_finalizacao': dict_finalizacao,
                  'dict_tributacao': dict_tributacao}]
        return self.supcfg_conecta_erp(params, 'fiscal_controller_exportar_notas', dados=dados)

    def supcfg_atualizar_cupom_servidor(self, cr, dados_nfce, res):
        for nfce_id in res:
            sql = """UPDATE supervisor_mov_cupom_servidor
                     SET supcse_enviado = True,
                         supcse_tipo_envio = '0'
                     WHERE supcse_capa_id = %s""" % nfce_id
            cr.execute(sql)
        nfce_erro_ids = []
        for i in dados_nfce:
            if i.get('id') not in res:
                nfce_erro_ids.append('%s' % i.get('id'))

        # marca as notas que deram problema para nao serem pegas novamente, serao exportadas apenas quando o
        # erro for corrigido e marcadas manualemte para serem exportadas
        if nfce_erro_ids:
            sql = """UPDATE supervisor_mov_cupom_servidor
                     SET supcse_tipo_envio = '1'
                     WHERE supcse_capa_id in (%s)""" % ', '.join(nfce_erro_ids)
            cr.execute(sql)

        return True

    def supcfc_verifica_cnpj(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        cnpj = vals.get('supcfc_cnpj')
        if cnpj and len(cnpj) != 14:
            raise osv.except_osv(_('Atencao!'),
                                 _('CNPJ invalido! Deve conter 14 digitos'))

        cnpj = re.sub(r'[^a-zA-Z0-9]', '', str(vals.get('supcfc_cnpj') or ''))
        if cnpj and (not cnpj.isdigit() or len(cnpj) != 14):
            raise osv.except_osv(_('Atencao!'),
                                 _('CNPJ invalido! Digite apenas os numeros!'))
        return True

    def create(self, cr, user, vals, context=None):
        context = context and context.copy() or {}
        self.supcfc_verifica_cnpj(cr, user, vals, context)
        if 'supcfc_nro_empresa' in vals and not vals.get('supcfc_nro_empresa'):
            raise osv.except_osv('Atencao!',
                                 'A Empresa deve ser diferente de zero!')

        self.verificar_faturamento_simples_nacional(vals, context)
        return super(supervisor_cfg_configuracao, self).create(cr, user, vals, context)

    def write(self, cr, user, ids, vals, context=None):
        context = context and context.copy() or {}
        self.supcfc_verifica_cnpj(cr, user, vals, context)
        if 'supcfc_nro_empresa' in vals and not vals.get('supcfc_nro_empresa'):
            raise osv.except_osv('Atencao!',
                                 'A Empresa deve ser diferente de zero!')

        self.verificar_faturamento_simples_nacional(vals, context)
        return super(supervisor_cfg_configuracao, self).write(cr, user, ids, vals, context)

    def verificar_faturamento_simples_nacional(self, vals, context=None):
        context = context and context.copy() or {}
        if vals.get('supcfc_regime_tributario') == 'S' \
                and vals.get('supcfc_valor_faturamento') <= 0:
            raise osv.except_osv('Atencao!',
                                 'O valor do Faturamento Anual deve ser maior que zero para '
                                 'Simples Nacional!')

        return True

    def supcfc_abrir_importacao(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        return {
            'name': _('Importacao de Certificado Digital'),
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'supervisor.import.certificado',
            'view_id': False,
            'target': 'new',
            'type': 'ir.actions.act_window',
        }
