# -*- coding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
# #############################################################################

from openerp.osv import osv
from SigUtils import sigerp
from openerp.tools.translate import _
import logging
from openerp import SUPERUSER_ID
from unicodedata import normalize
from .....web_sig.control.utils import websig_get_my_ip
from re import sub
import requests
import json


class valida_pendencias(osv.Model):
    _inherit = 'valida.pendencias'
    _logger = logging.getLogger(_inherit)

    def valpen_validar_erp(self, cr, uid, ids, context=None):
        """
        Método executado pelo Cron para comunicar com a Sig e pegar uma nova licenca
        @param cr:
        @param uid:
        @param ids:
        @param context:
        @return: True ou False dependendo se conseguiu ou não
        """
        # Se o modulo ADM estiver instalado, nao faz nada.
        context = context and context.copy() or {}
        super(valida_pendencias, self).valpen_validar_erp(cr, uid, ids, context)
        if self.valpen_verifica_rede_sig(cr, uid, context):
            self._logger.info(_('Nao validou, esta na rede interna da Sig!'))
            return True

        args = [('name', '=', 'adm'), ('state', 'in', ('installed', 'to upgrade'))]
        if self.pool.get('ir.module.module').search(cr, uid, args, context=context):
            return True

        self._logger.info(_('Validando o SigERP'))
        obj_config = self.pool.get('supervisor.cfg.configuracao')
        config_ids = obj_config.search(cr, uid, [], context=context)
        dados_conf = obj_config.read(cr, uid, config_ids[0], ['supcfc_cnpj'], context)
        cnpj_cadastro = sub(r'[^a-zA-Z0-9]', '', dados_conf.get('supcfc_cnpj') or '')
        if not cnpj_cadastro:
            self._logger.info(_('Empresa sem cnpj cadastrado!'))
            return False

        servidor = self.pool.get('ir.config_parameter').get_sig_server(cr, uid).split(',')
        self._logger.info(_('Validando ERP para o CNPJ: %s Servidor: %s') % (cnpj_cadastro, servidor))
        try:
            host = servidor[0]
            db = servidor[1]
            porta = servidor[2]
            user = servidor[3]
            senha = servidor[4]
            ip_cliente = websig_get_my_ip()
            self._logger.info(_('Vou pedir geracao de licensa cnpj: %s IP: %s') % (cnpj_cadastro, ip_cliente))
            res = requests.post('https://%s/valida/controller_gerar_licensa' % host,
                                auth=(user, senha),
                                data={'db': db,
                                      'pwd': senha,
                                      'user': user,
                                      'url': host,
                                      'port': porta,
                                      },
                                timeout=10000)
            res = json.loads(res.content)
            if res.get('codigo') != '00':
                self._logger.info('SigERP nao validado! Codigo do erro: %s Erro:%s' % (res.get('codigo'),
                                                                                       res.get('mensagem')))
            licenca = res.get('retorno')[0]
            if not licenca:
                raise Exception('Licenca nao gerada no ERP da SIG')

            self.pool.get('ir.config_parameter').set_param(cr, uid, 'licensa_sig_erp', licenca)
            self._logger.info(
                _('SigERP validado! Licenca:%s') % str(sigerp._crypto_sig('d', licenca)))
        except Exception as e:
            if isinstance(e, object) and not isinstance(e, ValueError) and 'faultCode' in e.__dict__:
                e = e.faultCode or ''
            else:
                e = str(e)
            try:
                string = normalize('NFKD', e.decode('utf-8')).encode('ASCII', 'ignore')
            except:
                string = normalize('NFKD', e).encode('ASCII', 'ignore')

            self._logger.info('SigERP nao validado! Erro:%s' % string)
            # Não conseguindo a conecção, remover o servidor da configuração para que possar ser gerado novamente,
            # talvez ele esteja errado
            server_id = self.pool.get('ir.config_parameter').search(cr, SUPERUSER_ID,
                                                                    [('key', '=', 'server_sig')], context=context)
            self.pool.get('ir.config_parameter').unlink(cr, SUPERUSER_ID, server_id)

        return True
