# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _
from openerp.addons.supervisor import print_log
from datetime import datetime


class wizard_supervisor_importar_erp(osv.TransientModel):

    # Prefixo: wizsim

    _name = "wizard.supervisor.importar.erp"
    _columns = {
        'wizsim_data': fields.date('Data', required=False),
        'wizsim_gerar_itens': fields.boolean('Importar itens'),
        'wizsim_gerar_clientes': fields.boolean('Importar clientes'),
        'wizsim_gerar_kit': fields.boolean('Importar kits'),
        'wizsim_gerar_vendedores': fields.boolean('Importar vendedores'),
        'wizsim_importar_saldo': fields.boolean('Importar saldo'),
        'state': fields.selection([('1', 'Esboco'), ('2', 'Concluido'), ('3', 'Erro')], 'Estado'),
        'wizsim_importar_itens': fields.selection([('P', 'Parcial'), ('T', 'Todos')], 'Importar itens', size=1, required=True,
                                           help=_(
                                               "'Parcial' importa apenas os itens que o preco da tabela 'automacao_precos' ja tenha sido importada. 'Todos' importa todos os itens ")),
        'wizsim_informacao': fields.text('Informacao'),
        'wizsim_pdv_ids': fields.many2many('supervisor.cad.pdvs', 'supervisor_pdv_importa_para_ecf', 'id',
                                    'pdv_id', 'PDVs'),
        # Arquivos

    }

    _defaults = {
        'wizsim_gerar_itens': True,
        'wizsim_gerar_clientes': True,
        'wizsim_gerar_kit': False,
        'wizsim_gerar_vendedores': False,
        'state': '1',
        'wizsim_importar_itens': 'P',
        'wizsim_importar_saldo': False,
        'wizsim_pdv_ids': lambda s, cr, uid, c: s.pool.get('supervisor.cad.pdvs').search(cr,
                                                                  uid, [], context=c),
    }

    def wizsim_button_importar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        try:
            # PARA IMPEDIR QUE O CRON DA EXPORTACAO DE NOTA TENTE ENVIAR ALGUMA NOTA
            # SEM TERMINAR A IMPORTACAO DE DADOS
            cr.execute("""INSERT INTO supervisor_mov_acao_pendente 
                                    (supape_tabela, supeape_loja)
                          VALUES ('Importando do ERP', '0');
                          COMMIT;
                       """)
            self.wizsim_importar_dados_erp(cr, uid, ids, context)
        except Exception:
            raise Exception
        finally:
            cr.execute("""DELETE FROM supervisor_mov_acao_pendente 
                          WHERE supape_tabela = 'Importando do ERP';
                          COMMIT;""")
        return True

    def wizsim_importar_dados_erp(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_config = self.pool.get('supervisor.cfg.configuracao')
        config_ids = obj_config.search(cr, uid, [], context=context)
        values_exportacao = {'state': '2'}
        if not config_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhuma configuracao cadastrada!'))
        for dados_config in obj_config.read(cr, uid, config_ids, ['supcfc_base_erp', 'supcfc_host_erp',
                                                                  'supcfc_nro_empresa'], context):
            if not dados_config.get('supcfc_base_erp') or not dados_config.get('supcfc_host_erp'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Sem "Base" ou "Host" do ERP informados nas configuracoes'))

            if not dados_config.get('supcfc_nro_empresa'):
                raise osv.except_osv(_('Atencao!'),
                                     _('Sem "Empresa" do ERP informada nas configuracoes'))

            self._wizsim_itens_exportados = False
            self._wizsim_clientes_exportados = False
            self._wizsim_vendedores_exportados = False
            brw_filtro = self.browse(cr, uid, ids[0], context)
            dados = [{'itens': brw_filtro.wizsim_gerar_itens,
                      'clientes': brw_filtro.wizsim_gerar_clientes,
                      'kits': brw_filtro.wizsim_gerar_kit,
                      'vendedores': brw_filtro.wizsim_gerar_vendedores,
                      'saldo': brw_filtro.wizsim_importar_saldo,
                      'tipo_exportacao': brw_filtro.wizsim_importar_itens}]
            cr.execute("""SELECT id 
                          FROM ir_module_module 
                          WHERE name = 'supervisor_producao'
                            AND state = 'installed';""")
            producao_instalado = False
            if cr.dictfetchall():
                producao_instalado = True
                dados[0]['locais'] = brw_filtro.wizsim_gerar_locais
                n_locais = self._wizsim_get_n_locais(cr)
                dados[0]['n_locais'] = n_locais

            try:
                res = obj_config.supcfg_conecta_erp(dados_config, 'ecf_controller_exportar_dados',
                                                    'ecf', dados=dados, timeout=600)
                if not res:
                    return True

                self._wizsim_bloqueia_acao_pendente(cr, uid)
                res = res[0]
                msg = ''
                if res.get('msg'):
                    msg = res.get('msg')

                msg_itens = self._wizsim_criar_itens(cr, uid, res.get('itens'), res.get('tributacoes'),
                                                     brw_filtro.wizsim_importar_itens, producao_instalado)
                if msg_itens:
                    msg = msg_itens + '\n' + msg

                msg_kits = self._wizsim_criar_kits(cr, uid, res.get('kits') or {})
                if msg_kits:
                    msg = msg_kits + '\n' + msg

                self._wizsim_criar_vendedores(cr, uid, res.get('vendedores'))
                self._wizsim_criar_clientes(cr, uid, res.get('clientes'))
                if res.get('clientes') or res.get('msg_clientes'):
                    self._wizsim_criar_msg_clientes(cr, uid, res.get('msg_clientes'))
                self._wizsim_criar_locais_producao(cr, uid, res.get('locais_producao') or [])
                if msg:
                    values_exportacao['wizsim_informacao'] = msg

                self._wizsim_desbloqueia_acao_pendente(cr, uid)
                self._wizsim_inserir_acao_pendente(cr, uid, ids, context)
            except Exception as e:
                self._wizsim_desbloqueia_acao_pendente(cr, uid)
                values_exportacao['wizsim_informacao'] = '%s' % e

        self.write(cr, uid, ids, values_exportacao, context=context)
        return True

    def _wizsim_criar_itens(self, cr, uid, itens, tributacoes, exportar_itens, producao_instalado):
        if not itens:
            return []

        cr.execute("""SELECT id,
                             suptri_tipo_tributacao   AS tipo_trib,
                             suptri_codigo_trib_retag AS cod_retarguarda
                      FROM   supervisor_cad_tributacao;""")
        dict_tibutacao = {}
        for trib in cr.dictfetchall():
            if trib.get('tipo_trib') in ('I', 'F', 'N'):
                dict_tibutacao[trib.get('tipo_trib')] = trib.get('id')
                dict_tibutacao[trib.get('cod_retarguarda')] = trib.get('id')
            else:
                dict_tibutacao[trib.get('cod_retarguarda')] = trib.get('id')

        codigos_tributacao = []
        if exportar_itens == 'P':
            cr.execute("""SELECT id, 
                                 suptnf_codigo 
                          FROM supervisor_cad_tributacao_nfce;""")
            for i in cr.dictfetchall():
                codigos_tributacao.append(i.get('suptnf_codigo'))

        else:
            cr.execute("TRUNCATE supervisor_cad_tributacao_nfce CASCADE")

        sql = """
                    INSERT INTO supervisor_cad_tributacao_nfce 
                               (create_date, create_uid, write_date, write_uid,
                                suptnf_codigo, suptnf_aliquota, suptnf_base, suptnf_encargo,
                                suptnf_codigo_cst, suptnf_codigo_csosn, suptnf_tipo_tributacao,
                                suptnf_aliquota_fcp, suptnf_motivo_beneficio)
                    VALUES %s        
                      """
        dados_insert = []
        prod_trib = {}
        data = str(datetime.now())[:19]
        for i in tributacoes:
            codigo_trib = i.get('suptnf_codigo')
            prod_trib['%s-%s' % (i.get('produto_id'), i.get('encargo'))] = codigo_trib
            if codigo_trib in codigos_tributacao:
                continue

            codigos_tributacao.append(codigo_trib)

            dados_insert.append("""('{create_date}', {create_uid}, '{write_date}', {write_uid},
                                     {suptnf_codigo}, {suptnf_aliquota}, {suptnf_base}, '{suptnf_encargo}',
                                    '{suptnf_codigo_cst}', {suptnf_codigo_csosn}, '{suptnf_tipo_tributacao}',
                                     {suptnf_aliquota_fcp}, '{suptnf_motivo_beneficio}')"""
                                .format(create_date=data,
                                        create_uid=uid,
                                        write_date=data,
                                        write_uid=uid,
                                        suptnf_codigo=codigo_trib,
                                        suptnf_aliquota=i.get('suptnf_aliquota'),
                                        suptnf_base=i.get('suptnf_base'),
                                        suptnf_aliquota_fcp=i.get('suptnf_aliquota_fcp'),
                                        suptnf_encargo=i.get('suptnf_encargo'),
                                        suptnf_codigo_cst=i.get('suptnf_codigo_cst'),
                                        suptnf_codigo_csosn=i.get('suptnf_codigo_csosn'),
                                        suptnf_tipo_tributacao=i.get('suptnf_tipo_tributacao'),
                                        suptnf_motivo_beneficio=i.get('suptnf_motivo_beneficio'))
                                )

        if dados_insert:
            cr.execute(sql % ', \n'.join(dados_insert[:30000]))
            if dados_insert[30000:60000]:
                cr.execute(sql % ', \n'.join(dados_insert[30000:60000]))
                if dados_insert[60000:]:
                    cr.execute(sql % ', \n'.join(dados_insert[60000:]))

        dict_trib_nfce = {}
        cr.execute("""SELECT id, suptnf_codigo, suptnf_tipo_tributacao 
                      FROM supervisor_cad_tributacao_nfce;""")
        for i in cr.dictfetchall():
            dict_trib_nfce[i.get('suptnf_codigo')] = i

        inserts = []
        # string com os itens que serao deletados do supervisor para depois serem inseridos
        codigo_itens = []
        msg_sem_icms = []
        msg_sem_cest = []
        msg_sem_pis = []
        msg_sem_cofins = []
        mensagem = ''
        tributacao = False
        itens_exportados = False
        camp_local_producao = ''
        if producao_instalado:
            camp_local_producao = ', supite_codigo_local'

        for item in itens:
            produto_id = item.get('produto_id')
            chave1 = '%s-%s' % (produto_id, 'ICMS')
            chave2 = '%s-%s' % (produto_id, 'ISS')
            icms_iss_id = False
            tipo_trib = False
            if prod_trib.get(chave1):
                tipo_trib = dict_trib_nfce.get(prod_trib.get(chave1)).get('suptnf_tipo_tributacao')
                icms_iss_id = dict_trib_nfce.get(prod_trib.get(chave1)).get('suptnf_codigo')
            elif prod_trib.get(chave2):
                icms_iss_id = dict_trib_nfce.get(prod_trib.get(chave2)).get('suptnf_codigo')

            chave1 = '%s-%s' % (produto_id, 'PIS')
            pis_id = False
            if prod_trib.get(chave1):
                pis_id = dict_trib_nfce.get(prod_trib.get(chave1)).get('suptnf_codigo')

            chave1 = '%s-%s' % (produto_id, 'COFINS')
            cofins_id = False
            if prod_trib.get(chave1):
                cofins_id = dict_trib_nfce.get(prod_trib.get(chave1)).get('suptnf_codigo')

            desc_completa = item.get('supite_descricao_completa')
            cod_item = item.get('supite_codigo_item')
            tem_msg = False
            if not icms_iss_id:
                msg_sem_icms.append('\n%s - %s' % (desc_completa, cod_item))
                tem_msg = True

            if not pis_id:
                msg_sem_pis.append('\n%s - %s' % (desc_completa, cod_item))
                tem_msg = True

            if not cofins_id:
                msg_sem_cofins.append('\n%s - %s' % (desc_completa, cod_item))
                tem_msg = True

            codigo_itens.append(cod_item)
            if tipo_trib in ('U', 'TU') and item.get('produto_ativo') == 'null' and item.get('active'):
                msg_sem_cest.append('\n%s - %s' % (desc_completa, cod_item))
                tem_msg = True

            if tem_msg:
                continue

            itens_exportados = True
            tributacao_ecf = item.get('tributacao_ecf') or ''
            tributacao_ecf1 = '0%s' % tributacao_ecf
            tributacao_id = dict_tibutacao.get(tributacao_ecf)
            if not tributacao_id:
                tributacao_id = dict_tibutacao.get(tributacao_ecf1)

            if not tributacao_id:
                mensagem += '\n %s - %s' % (desc_completa, cod_item)

            else:
                item.update(
                    {'create_date': data,
                     'write_date': data,
                     'create_uid': uid,
                     'write_uid': uid,
                     'supite_tributacao_id': tributacao_id,
                     'supite_tributacao_icms_id': icms_iss_id or 'null',
                     'supite_tributacao_pis_id': pis_id or 'null',
                     'supite_tributacao_cofins_id': cofins_id or 'null', })
                sql_cad_itens = """
                        ('{create_date}', {create_uid}, '{write_date}', {write_uid},
                         {supite_loja}, '{supite_codigo_retaguarda}', '{supite_codigo_item}', '{name}', 
                         '{origem_mercadoria}', {supite_codigo_anp}, {supite_adrem}, {supite_pglp}, {supite_pgnn},
                         {supite_pgni}, {supite_vpart}, {supite_getin},
                         '{supite_med_codigo_anvisa}', '{supite_med_motivo_isencao}', {supite_med_preco_maximo},
                         {supite_pesavel}, {supite_fracionado}, '{supite_unidade}',
                         {supite_peso_da_embalagem}, {supite_peso_tara},
                         '{supite_producao}', '{supite_iat}', '{supite_ncm}', '{supite_cest}', {supite_preco},
                         '{active}',
                         {supite_aliquota_ibpt_fed}, {supite_aliquota_ibpt_est}, {supite_aliquota_ibpt_mun},
                         '{supite_ibpt_chave}', {supite_tributacao_id}, '{supite_descricao_completa}',
                         {supite_fator_multiplicador}, {supite_tributacao_pis_id}, {supite_tributacao_cofins_id}, 
                         {supite_tributacao_icms_id} %s)"""

                sql_cad_itens = sql_cad_itens % (camp_local_producao and ', {supite_codigo_local}' or '')
                sql_cad_itens = sql_cad_itens.format(**item).replace("'null'", 'null')
                inserts.append(sql_cad_itens)
                tributacao = True

        str_codigo_itens = "'" + "','".join(codigo_itens) + "'"
        sql_delete = """DELETE FROM supervisor_cad_itens\n"""
        # para carga Parcial deleta apenas os itens que tiveram alteracao
        if exportar_itens == 'P':
            sql_delete += """WHERE supite_codigo_item IN (%s)""" % str_codigo_itens

        # os kits devem ser deletados antes de deletar os itens e a carga deve ser sempre de
        # itens e kits
        cr.execute("""SELECT id 
                      FROM supervisor_cad_kits 
                      WHERE (supkit_codigo_barra_kit IN ({cb}) 
                         OR supkit_codigo_barra_item IN ({cb}))
                         """.format(cb=str_codigo_itens)
                   )
        dados_kit = cr.dictfetchall()
        if dados_kit:
            self.exporta_kit = True
            kit_ids = map(lambda x: '%s' % x.get('id'), dados_kit)
            cr.execute("""DELETE FROM supervisor_cad_kits 
                          WHERE id IN ({ids})
                            """.format(ids=', '.join(kit_ids))
                       )

        cr.execute(sql_delete)
        qtd_itens = 0
        if inserts:
            self._wizsim_itens_exportados = True
            qtd_itens = len(inserts)
            sql = """INSERT INTO supervisor_cad_itens
                            (create_date, create_uid, write_date, write_uid, supite_loja,
                             supite_codigo_retaguarda, supite_codigo_item, name, supite_origem_mercadoria,
                             supite_codigo_anp, supite_adrem, supite_pglp, supite_pgnn, supite_pgni, supite_vpart,
                             supite_getin, supite_med_codigo_anvisa, supite_med_motivo_isencao, supite_med_preco_maximo,
                             supite_pesavel, supite_fracionado, supite_unidade,
                             supite_peso_da_embalagem, supite_peso_tara,
                             supite_producao, supite_iat, supite_ncm, supite_cest, supite_preco, active,
                             supite_aliquota_ibpt_fed, supite_aliquota_ibpt_est, supite_aliquota_ibpt_mun,
                             supite_ibpt_chave, supite_tributacao_id,
                             supite_descricao_completa, supite_fator_multiplicador,
                             supite_tributacao_pis_id, supite_tributacao_cofins_id, supite_tributacao_icms_id 
                             {codigo_local})
                      VALUES
                    """.format(codigo_local=(camp_local_producao))
            tam = len(inserts) / 10000
            for i in range(0, tam+1):
                ind = i*10000
                sql_parcela = ', \n'.join(inserts[ind:ind+10000])
                if sql_parcela:
                    cr.execute(sql + sql_parcela + ';\nCOMMIT;')

        msg = ''
        if not itens or not inserts:
            msg += _('\nNao ha itens para importacao.') + '\n' + ''.ljust(120, '-') + '\n'

        if msg_sem_icms:
            msg += _('\nOs itens abaixo nao possuem ICMS: %s') % ''.join(msg_sem_icms) \
                    + '\n' + ''.ljust(120, '-') + '\n'

        if msg_sem_pis:
            msg += _('\nOs itens abaixo nao possuem PIS: %s') % ''.join(msg_sem_pis)\
                    + '\n' + ''.ljust(120, '-') + '\n'

        if msg_sem_cofins:
            msg += _('\nOs itens abaixo nao possuem COFINS: %s') % ''.join(msg_sem_cofins)\
                    + '\n' + ''.ljust(120, '-') + '\n'

        if msg_sem_cest:
            msg += _('\nOs itens abaixo nao possuem CEST: %s') % ''.join(msg_sem_cest)\
                    + '\n' + ''.ljust(120, '-') + '\n'

        if mensagem:
            msg += _('\nOs itens abaixo nao foram exportados. Verifique a Tributacao na '
                     'ECF para os tributos dos itens. Verifique o cadastro de '
                     'tributacao no supervisor.') + mensagem \
                    + '\n' + ''.ljust(120, '-') + '\n'

        cr.execute("""SELECT id 
                      FROM ir_module_module 
                      WHERE name = 'supervisor_comanda'
                        AND state = 'installed';""")
        if cr.dictfetchall():
            sql = """
                UPDATE supervisor_cda_itens_favoritos AS sif SET supifa_item_id = sci.id
                FROM supervisor_cda_itens_favoritos AS sif1
                INNER JOIN supervisor_cad_itens     AS sci  ON sci.supite_codigo_item = sif1.supifa_cod_item
                WHERE sif1.supifa_item_id IS NULL
                  AND sif1.id = sif.id;
                COMMIT;  
                     """
            cr.execute(sql)

        if itens_exportados and tributacao:
            msg = (_('\n%s Itens Importados.' % qtd_itens) +
                   '\n' + ''.ljust(120, '-') + '\n' + msg)

        return msg

    def _wizsim_criar_kits(self, cr, uid, kits):
        vals = kits.get('vals')
        msg_erp = kits.get('msg') or ''
        if not vals:
            return msg_erp

        data = str(datetime.now())[:19]
        cont = 0
        cr.execute('DELETE FROM supervisor_cad_kits')
        inserts = []
        for kit in vals:
            # deve buscar o codigo do kit e do item no supervisor
            cr.execute('''SELECT id
                          FROM supervisor_cad_itens
                          WHERE supite_codigo_item = '%s' ''' % (kit.pop('cb_kit')))
            dados_kit = cr.dictfetchall()

            cr.execute(''' SELECT id
                           FROM supervisor_cad_itens
                           WHERE supite_codigo_item = '%s' ''' % (kit.pop('cb_item')))
            dados_item = cr.dictfetchall()
            if dados_kit and dados_item:
                cont += 1
                codigo_kit_id = dados_kit[0].get('id')
                codigo_item_id = dados_item[0].get('id')
                supkit_codigo_kit = codigo_kit_id
                supkit_codigo_item = codigo_item_id
                sql = """('{create_date}', {create_uid}, '{write_date}', {write_uid},
                          {supkit_loja}, {supkit_codigo_kit}, {supkit_codigo_item},
                         '{supkit_codigo_barra_kit}', '{supkit_codigo_barra_item}',
                          {supkit_quantidade}, {supkit_preco_venda})
                  """.format(create_date=data,
                             create_uid=uid,
                             write_date=data,
                             write_uid=uid,
                             supkit_codigo_kit=supkit_codigo_kit,
                             supkit_codigo_item=supkit_codigo_item,
                             supkit_codigo_barra_kit=kit.get('supkit_codigo_barra_kit'),
                             supkit_codigo_barra_item=kit.get('supkit_codigo_barra_item'),
                             supkit_loja=kit.get('supkit_loja'),
                             supkit_quantidade=kit.get('supkit_quantidade'),
                             supkit_preco_venda=kit.get('supkit_preco_venda'))

                inserts.append(sql)
        sql_kits = """
            INSERT INTO supervisor_cad_kits
                        (create_date, create_uid, write_date, write_uid,
                         supkit_loja, supkit_codigo_kit, supkit_codigo_item,
                         supkit_codigo_barra_kit, supkit_codigo_barra_item,
                         supkit_quantidade, supkit_preco_venda)
            VALUES """
        for i in range(0, (len(inserts)/10000) + 1):
            idx = i * 10000
            sql = sql_kits + ',\n'.join(inserts[idx: idx+10000]) + ';\nCOMMIT;'
            cr.execute(sql)

        msg = _('\nNenhum Kit importado ou Existem Itens em Kits que nao '
                'foram importados. Realize a importacao de itens e '
                'tente novamente.') + '\n' + ''.ljust(120, '-') + '\n'
        if cont:
            msg = _('\n%s Kits Importados.' % cont) + '\n' + ''.ljust(120, '-') + '\n'

        return msg + msg_erp

    def _wizsim_criar_vendedores(self, cr, uid, dados):
        if not dados:
            return True

        self._wizsim_vendedores_exportados = True
        obj_util = self.pool.get('supervisor.util')
        data = str(datetime.now())[:19]
        for vendedor in dados:
            codigo_vendedor = vendedor.get('id')
            nome_vendedor = obj_util.suputi_remover_acentuacao(vendedor.get('nome'))
            nome_vendedor = nome_vendedor[:40]

            sql_vend_id = """SELECT supven_codigo
                             FROM   supervisor_cad_vendedor
                             WHERE  supven_codigo = '%s' """ % codigo_vendedor
            cr.execute(sql_vend_id)
            vend_id = cr.dictfetchall()
            if vend_id:
                update_vendedor = """UPDATE supervisor_cad_vendedor
                                     SET    write_date = '%s',
                                            write_uid = %s,
                                            supven_codigo = %s,
                                            name = '%s'
                                     WHERE supven_codigo = %s
                """ % (data, uid, codigo_vendedor, nome_vendedor, codigo_vendedor)
                cr.execute(update_vendedor)
            else:
                insert_vendedor = """
                    INSERT INTO supervisor_cad_vendedor
                                (supven_codigo,
                                name,
                                create_date, create_uid,
                                write_date, write_uid)
                    VALUES      (%s, '%s', '%s', %s, '%s', %s)
            """ % (codigo_vendedor, nome_vendedor, data, uid, data,
                   uid)
                cr.execute(insert_vendedor + ';\nCOMMIT;')

        return True

    def _wizsim_criar_clientes(self, cr, uid, vals):
        if not vals:
            return True

        self._wizsim_clientes_exportados = True
        cr.execute("""TRUNCATE supervisor_cad_clientes""")
        data = str(datetime.now())[:19]
        inserts = []
        for cliente in vals:
            cliente.update({'create_date': data,
                            'create_uid': uid,
                            'write_date': data,
                            'write_uid': uid})
            sql = """
                ('{cpf_cnpj}', '{nome}', '{supcli_status}', '{endereco}', {limite_compras},
                 {total_em_aberto}, {codigo_interno},
                 '{ie}', '{bairro}', $${cidade}$$, '{estado}', '{cep}',
                 '{telefone}', {rota}, '{pessoa_tipo}', {cod_municipio},
                 '{endereco_numero}', '{complemento}', '{email}',
                 '{create_date}', {create_uid}, '{write_date}', {write_uid})
                """.format(**cliente)
            sql = sql.replace("'null'", 'null')
            inserts.append(sql)

        insert_cliente = """
                INSERT INTO supervisor_cad_clientes
                    (supcli_cnpj_cpf, name, supcli_status, supcli_endereco,
                    supcli_limite_compras, supcli_total_em_aberto,
                    supcli_codigo_interno, supcli_insc_estadual,
                    supcli_bairro, supcli_cidade, supcli_estado,
                    supcli_cep, supcli_telefone, supcli_rota,
                    supcli_tipo_pessoa, supcli_cod_municipio,
                    supcli_endereco_numero, supcli_complemento, supcli_email,
                    create_date, create_uid, write_date, write_uid)
                VALUES """
        # faz parcelado para o banco nao consumir muita memoria
        for i in range(0, (len(inserts)/10000)+1):
            idx = i * 10000
            sql = insert_cliente + ',\n'.join(inserts[idx:idx+10000]) + ';\nCOMMIT;'
            cr.execute(sql)

        return True

    def _wizsim_criar_msg_clientes(self, cr, uid, vals):
        cr.execute("""TRUNCATE  supervisor_cad_mensagens_clientes""")
        if not vals:
            return True

        data = str(datetime.now())[:19]
        inserts = []
        for msg in vals:
            msg.update({'create_date': data,
                        'create_uid': uid,
                        'write_date': data,
                        'write_uid': uid})
            sql = """('{cnpj_cpf}', {sequencia}, '{mensagem}', '{tipo}',
                      '{create_date}', {create_uid}, '{write_date}', {write_uid})""".format(**msg)
            sql = sql.replace("'null'", 'null')
            inserts.append(sql)

        insert_mensagem = """
                INSERT INTO supervisor_cad_mensagens_clientes
                       (supclm_cnpj_cpf, supclm_sequencia, supclm_mensagem, supclm_tipo,
                        create_date, create_uid, write_date, write_uid)
                VALUES """
        # faz parcelado para o banco nao consumir muita memoria
        for i in range(0, (len(inserts)/10000)+1):
            idx = i * 10000
            sql = insert_mensagem + ',\n'.join(inserts[idx: idx+10000]) + ';\nCOMMIT;'
            cr.execute(sql)

        return True

    def wizsim_cron_importa_saldo(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obj_config = self.pool.get('supervisor.cfg.configuracao')
        config_ids = obj_config.search(cr, uid, [], context=context)
        if not config_ids:
            return True

        res = []
        for dados_config in obj_config.read(cr, uid, config_ids, ['supcfc_base_erp', 'supcfc_host_erp',
                                                                  'supcfc_nro_empresa'], context):
            if not dados_config.get('supcfc_base_erp') or not dados_config.get('supcfc_host_erp'):
                print_log(_('"Base" ou "Host" do ERP nao informados nas configuracoes do Supervisor'), 'E')
                return True

            if not dados_config.get('supcfc_nro_empresa'):
                print_log(_('Sem "Empresa" do ERP informada nas configuracoes do Supervisor'), 'E')
                return True

            try:
                res = obj_config.supcfg_conecta_erp(dados_config,
                                                    'ecf_get_saldos', 'ecf', timeout=120)
            except Exception as e:
                print_log(e, 'E')
                return True

        if not res:
            return True

        date = str(datetime.now())[:19]
        inserts = []
        cbs = []
        for saldo in res:
            cbs.append(saldo.get('item'))
            saldo.update({'create_date': date,
                          'create_uid': uid,
                          'write_date': date,
                          'write_uid': uid,
                          'data': date[:10]})
            vals = """({create_uid}, '{create_date}', '{write_date}', {write_uid},
                       {loja}, '{item}', {saldo}, {saldo_inicial}, 
                       '{data}')""".format(**saldo)
            inserts.append(vals)

        sql_insert = """INSERT INTO supervisor_mov_saldo 
                                    (create_uid, create_date, write_date, write_uid,
                                     supsal_loja, supsal_item, supsal_saldo, supsal_saldo_inicial,
                                     supsal_data)
                            VALUES """
        # deleta apenas o registros que vao ser atualizados
        cr.execute("DELETE FROM supervisor_mov_saldo WHERE supsal_item IN ('%s')" % "','".join(cbs))
        for i in range(0, (len(inserts)/10000)+1):
            idx = i * 10000
            sql = sql_insert + ',\n'.join(inserts[idx: idx+10000]) + ';\nCOMMIT;'
            cr.execute(sql)

        return True

    def _wizsim_get_n_locais(self, cr):
        cr.execute("""SELECT sucloc_codigo FROM supervisor_producao_local""")
        res = []
        for i in cr.dictfetchall():
            res.append(i.get('sucloc_codigo'))

        return res

    def _wizsim_criar_locais_producao(self, cr, uid, dados_locais):
        "Implementado por heranca no modulo 'supervisor_producao' "
        return True

    def _wizsim_inserir_acao_pendente(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        pdvs_dados = self.browse(cr, uid, ids[0]).wizsim_pdv_ids
        if not pdvs_dados:
            return True

        erro_acao = False
        lista_ecf = []
        obj_pdvs = self.pool.get('supervisor.cad.pdvs')
        ecf_ids = obj_pdvs.search(cr, uid, [], context=context)
        if ecf_ids:
            for i in obj_pdvs.read(cr, uid, ecf_ids, ['suppdv_caixa', 'suppdv_loja'],
                                                   context):
                lista_ecf.append('%s-%s' % (i.get('suppdv_loja'), i.get('suppdv_caixa')))

        obj_acao = self.pool.get('supervisor.mov.acao.pendente')
        for i in pdvs_dados:
            vals = {'supeape_loja': i.suppdv_loja, 'supeape_pdv': i.suppdv_caixa,
                    'supape_situacao': 'A'}

            args = [('supeape_loja', '=', i.suppdv_loja),
                    ('supeape_pdv', '=', i.suppdv_caixa)]

            if self._wizsim_itens_exportados:
                vals['supape_tabela'] = 'Itens'
                args_itens = args + [('supape_tabela', '=', 'Itens')]
                context['exporta_sem_retornar_raise'] = True
                res = (self.pool.get('wizard.supervisor.exporta.para.ecf').
                       exporta_verificar_promocoes_tipo4(cr, uid, context))
                acao_pendente_id = obj_acao.search(cr, uid, args_itens, context=context)
                chave = '%s-%s' % (i.suppdv_loja, i.suppdv_caixa)
                if not acao_pendente_id and res:
                    obj_acao.create(cr, uid, vals, context)
                    if chave not in lista_ecf:
                        vals['supape_tabela'] = 'Tributacao NFC-e'
                        obj_acao.create(cr, uid, vals, context)

                if not res:
                    erro_acao = True

            if self._wizsim_clientes_exportados:
                vals['supape_tabela'] = 'Mensagem'
                args_msg = args + [('supape_tabela', '=', 'Mensagem')]
                acao_pendente_id = obj_acao.search(cr, uid, args_msg, context=context)
                if not acao_pendente_id:
                    obj_acao.create(cr, uid, vals, context)

                vals['supape_tabela'] = 'Clientes'
                args_cli = args + [('supape_tabela', '=', 'Clientes')]
                acao_pendente_id = obj_acao.search(cr, uid, args_cli, context=context)
                if not acao_pendente_id:
                    obj_acao.create(cr, uid, vals, context)

            if self._wizsim_vendedores_exportados:
                vals['supape_tabela'] = 'Vendedor'
                args_vend = args + [('supape_tabela', '=', 'Vendedor')]
                acao_pendente_id = obj_acao.search(cr, uid, args_vend, context=context)
                if not acao_pendente_id:
                    obj_acao.create(cr, uid, vals, context)

        if erro_acao:
            raise Exception('Erro ao criar acao pendente de Intens, '
                            'problema no valor dos itens na promocao do tipo 4!')

        return True

    def _wizsim_bloqueia_acao_pendente(self, cr, uid):
        '''
         1 - Verifica se ja tem alguma acao pendente bloqueada pela ecf, se tiver duplica essta acao
         2 - Muda a situacao de todos os itens na tabela supervisor_mov_acao_pendente para 'B'
             (bloqueado) nao permitindo que o frente faca alguma atualizacao
        :param _cr: este cursor eh do banco do supervisor, que pode estar em outra maquina
        '''
        create_date = write_date = str(datetime.now())[:19]
        cr.execute("""SELECT *
                      FROM supervisor_mov_acao_pendente
                      WHERE supape_situacao = 'B'
                      """)
        for i in cr.dictfetchall():
            supeape_loja = i.get('supeape_loja')
            supeape_pdv = i.get('supeape_pdv')
            supape_tabela = i.get('supape_tabela')
            supape_total_em_aberto_cliente = i.get('supape_total_em_aberto_cliente') or 0.0
            cr.execute('''
                SELECT id
                FROM supervisor_mov_acao_pendente
                WHERE supeape_loja = {supeape_loja}
                  AND supeape_pdv = {supeape_pdv}
                  AND supape_tabela = '{supape_tabela}'
                  AND supape_situacao = 'B'
                       '''.format(supeape_loja=supeape_loja,
                                  supeape_pdv=supeape_pdv,
                                  supape_tabela=supape_tabela))
            # nao deixa que existam mais de 2 registro para uma mesma loja, pdv e tabela com situacao = 'B'
            if len(cr.dictfetchall()) > 1:
                continue

            sql = '''
                INSERT INTO supervisor_mov_acao_pendente
                    (create_date, create_uid, write_date, write_uid,
                    supeape_loja, supeape_pdv, supape_tabela, supape_situacao, supape_total_em_aberto_cliente)
                VALUES ('{create_date}', {create_uid}, '{write_date}', {write_uid},
                         {supeape_loja},  {supeape_pdv}, '{supape_tabela}', 'B', {supape_total_em_aberto_cliente});
                   '''.format(create_date=create_date,
                              create_uid=uid,
                              write_date=write_date,
                              write_uid=uid,
                              supeape_loja=supeape_loja,
                              supeape_pdv=supeape_pdv,
                              supape_tabela=supape_tabela,
                              supape_total_em_aberto_cliente=supape_total_em_aberto_cliente)
            cr.execute(sql)

        sql = '''
            UPDATE supervisor_mov_acao_pendente
            SET write_date = '%s',
                write_uid = %s,
                supape_situacao = 'B';
            COMMIT;
              ''' % (str(datetime.now())[:19], uid)
        cr.execute(sql)
        return

    def _wizsim_desbloqueia_acao_pendente(self, cr, uid):
        '''
        muda a situacao de todos os itens na tabela supervisor_mov_acao_pendente para 'A' permitindo
        o frente fazer atualizacao
        :param cr: este cursor eh do bando do supervisor, que pode estar em outra maquina
        '''

        sql = """
            UPDATE supervisor_mov_acao_pendente
            SET write_date = '%s',
                write_uid = %s,
                supape_situacao = 'A';
            COMMIT;
              """ % (str(datetime.now())[:19], uid)
        cr.execute(sql)
        return
