#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _
from datetime import datetime
import os
from verifica_cb import verifica_cb
from openerp.addons.supervisor import print_log
from .verifica_cb import PROMOCOES


class supervisor_cad_promo_capa(osv.Model):

    _name = 'supervisor.cad.promo_capa'
    _inherit = 'supervisor.cad.promo_capa'

    _campos_filhas = ['supproc_promo1_ids','supproc_promo2_ids','supproc_promo3p_ids',
                     'supproc_promo3f_ids', 'supproc_promo4_ids','supproc_promo5p_ids',
                     'supproc_promo5f_ids','supproc_promo6_ids', 'supproc_promo7_ids',
                     'supproc_promo8_ids']

    def name_get(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        res = []
        if not len(ids):
            return res

        dados = self.read(cr, uid, ids, ['supproc_id_promo', 'supproc_nome_promo',
                                         'supproc_scanntech'], context=context)
        for x in dados:
            descricao = x.get('supproc_nome_promo') or ''
            if not descricao:
                descricao = "Propria"
                if x.get('supproc_scanntech'):
                    descricao = "Scanntech"
            res.append((x['id'], str(x['supproc_id_promo']) + ' - ' + descricao.encode('utf-8')))

        return res

    def write(self, cr, uid, ids, vals, context=None):
        context = context and context.copy() or {}
        # pula os teste a baixo caso as modificacoes sejam apenas na capa e nao nas filhas
        exe_super = True
        for c in self._campos_filhas:
            if c in vals:
                exe_super = False
                break

        if exe_super:
            return super(supervisor_cad_promo_capa, self).write(cr,  uid, ids, vals, context=context)

        cont = 0
        campo_gravado = ''
        list_pai_filho = []
        pai = False
        filho = False
        if not isinstance(ids, list):
            ids = [ids]

        for promo in self.read(cr, uid, ids, self._campos_filhas, context=context):
            for campo in promo:
                if promo.get(campo) and campo != 'id':
                    if campo in ['supproc_promo3p_ids', 'supproc_promo5p_ids'] and not pai:
                        pai = True
                        if not (pai ^ filho):
                            list_pai_filho.append(campo)
                            continue
                    elif campo in ['supproc_promo3f_ids', 'supproc_promo5f_ids'] and not filho:
                        filho = True
                        if not (pai ^ filho):
                            list_pai_filho.append(campo)
                            continue
                    cont += 1
                    campo_gravado = campo

                if campo == 'supproc_promo8_ids' and vals.get(campo) and len(vals.get(campo)) > 1:
                    raise osv.except_osv(_('Atencao!'),
                                         _('A promocao 8 deve conter apenas 1 item por promocao!'))

        for campo in vals:
            if campo in self._campos_filhas \
                    and campo != campo_gravado:

                if (campo in ['supproc_promo3p_ids', 'supproc_promo5p_ids'] and not pai) or campo in list_pai_filho:
                    pai = True
                    if not (pai ^ filho):
                        continue
                elif (campo in ['supproc_promo3f_ids', 'supproc_promo5f_ids'] and not filho) or campo in list_pai_filho:
                    filho = True
                    if not (pai ^ filho):
                        continue
                cont += 1
                if cont > 1:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Apenas 1 tipo de promocao deve ser registrado!'))

        return super(supervisor_cad_promo_capa, self).write(cr,  uid, ids, vals, context=context)

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        # caso algum item da scanntech ja esteja em alguma promocao, esta irá ficar em esboço
        if not context.get('manutencao_promocoes'):
            res = super(supervisor_cad_promo_capa, self).create(cr,  uid, vals, context=context)
            if not res:
                return res

            brw_capa = self.browse(cr, uid, res, context=context)
            codigos_barras = self.supproc_get_codigos_barras(brw_capa)
            verifica = verifica_cb(cr, brw_capa.id, codigos_barras,
                                                    str(brw_capa.supproc_data_hora_inicio),
                                                    str(brw_capa.supproc_data_hora_fim),
                                                    context=context)
            if not verifica:
                return res

            ids_promocoes = []
            for i in verifica:
                ids_promocoes.append(i.keys()[0])

            promo_capa_ids = self.search(cr, uid, [('supproc_id_promo', 'in', ids_promocoes),
                                                   ('supproc_scanntech','=', False),
                                                   ('state','=', self._state_ativada)],
                                         context=context)
            if promo_capa_ids:
                self.write(cr, uid, promo_capa_ids, {'state': self._state_esboco}, context=context)

            return res

        cont = 0
        pai = False
        filho = False
        for campo in vals:
            if campo in self._campos_filhas and vals.get(campo):
                if campo in ['supproc_promo3p_ids', 'supproc_promo5p_ids'] and not pai:
                    pai = True
                    if not (pai ^ filho):
                        continue
                elif campo in ['supproc_promo3f_ids', 'supproc_promo5f_ids'] and not filho:
                    filho = True
                    if not (pai ^ filho):
                        continue

                if campo == 'supproc_promo8_ids' and vals.get(campo) and len(vals.get(campo)) > 1:
                    raise osv.except_osv(_('Atencao!'),
                                         _('A promocao 8 deve conter apenas 1 item por promocao!'))

                cont += 1
                if cont > 1:
                    raise osv.except_osv(_('Atencao!'),
                                         _('Apenas 1 tipo de promocao deve ser registrado!'))

        if cont == 0:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nenhum item registrado!'))

        res = super(supervisor_cad_promo_capa, self).create(cr,  uid, vals, context=context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('manutencao_promocoes'):
            for promo in self.read(cr, uid, ids, ['supproc_scanntech'], context=context):
                if promo.get('supproc_scanntech'):
                    raise osv.except_osv(_('Atencao!'),
                                         _('Promocoes SCANNTECH nao podem ser excluidas!'))

        return super(supervisor_cad_promo_capa, self).unlink(cr, uid, ids, context=context)

    def get_promo1(self, cr, uid, promo1, loja, path_file, context=None):
        context = context and context.copy() or {}
        if not promo1:
            return True, '', 0

        dict_promo_ids = {}
        for linha in promo1.readlines():
            data_inicio = datetime(int(linha[39:43]), int(linha[37:39]), int(linha[35:37]),
                                   int(linha[43:45]), int(linha[45:47]), int(linha[47:49]))
            data_fim = datetime(int(linha[53:57]), int(linha[51:53]), int(linha[49:51]),
                                int(linha[57:59]), int(linha[59:61]), int(linha[61:63]))
            try:
                maximo = int(linha[71:81])/1000.0
            except:
                maximo = 9999999

            id_promo = int(linha[63:69])
            data_inicio = self.get_date_timezone(cr, uid, data_inicio, tz_destino='UTC')
            data_fim = self.get_date_timezone(cr, uid, data_fim, tz_destino='UTC')
            if not id_promo in dict_promo_ids:
                dict_promo_ids[id_promo] = {'supproc_data_hora_inicio': data_inicio,
                                            'supproc_data_hora_fim': data_fim,
                                            'supproc_id_promo': id_promo,
                                            'supproc_tipo_promo': '1',
                                            'supproc_maximo': maximo,
                                            'supproc_scanntech': True,
                                            'supproc_promo1_ids': [],
                                            'state': self._state_ativada
                }
            op = 0
            dict_promo_ids[id_promo]['supproc_promo1_ids'].\
                append((op, False, {'suppro1_barras': linha[:14],
                                   'suppro1_quantidade': float(linha[14:24])/1000.0,
                                   'suppro1_desconto': float(linha[24:34])/100.0,
                                   'suppro1_tipo_desconto': linha[34:35],
                                   'suppro1_sequencial': int(linha[69:71])
                                  }
                      ))

        for promo_id in dict_promo_ids:
            args = [('supproc_id_promo', '=', promo_id),
                    ('supproc_scanntech', '=', True)]
            promo_ids = self.search(cr, uid, args, context=context)
            if promo_ids:
                self.unlink(cr, uid, promo_ids, context=context)

            self.create(cr, uid, dict_promo_ids.get(promo_id), context=context)

        promo1.close()
        os.remove(path_file + 'st_promo1.' + str(loja).zfill(3))
        return True, '', len(dict_promo_ids)

    def get_promo2(self, cr, uid, promo2, loja, path_file, context=None):
        if not promo2:
            return True, '', 0

        dict_promo_ids = {}
        for linha in promo2.readlines():
            data_inicio = datetime(int(linha[47:51]), int(linha[45:47]), int(linha[43:45]),
                                   int(linha[51:53]), int(linha[53:55]), int(linha[55:57]))
            data_fim = datetime(int(linha[61:65]), int(linha[59:61]), int(linha[57:59]),
                                int(linha[65:67]), int(linha[67:69]), int(linha[69:71]))
            try:
                maximo = int(linha[71:81])/1000.0
            except:
                maximo = 9999999

            id_promo = int(linha[0:6])
            data_inicio = self.get_date_timezone(cr, uid, data_inicio, tz_destino='UTC')
            data_fim = self.get_date_timezone(cr, uid, data_fim, tz_destino='UTC')
            if not id_promo in dict_promo_ids:
                dict_promo_ids[id_promo] = {'supproc_data_hora_inicio': data_inicio,
                                            'supproc_data_hora_fim': data_fim,
                                            'supproc_id_promo': id_promo,
                                            'supproc_tipo_promo': '2',
                                            'supproc_maximo': maximo,
                                            'supproc_scanntech': True,
                                            'supproc_promo2_ids': [],
                                            'state': self._state_ativada
                                             }
            op = 0
            dict_promo_ids[id_promo]['supproc_promo2_ids'].\
                        append((op, False, {'suppro2_barras': linha[8:22],
                                            'suppro2_quantidade': float(linha[22:32])/1000.0,
                                            'suppro2_desconto': float(linha[32:42])/100.0,
                                            'suppro2_tipo_desconto': linha[42:43],
                                            'suppro2_sequencial': int(linha[6:8])}
                                          ))
        for promo_id in dict_promo_ids:
            args = [('supproc_id_promo', '=', promo_id),
                    ('supproc_scanntech', '=', True)]
            promo_ids = self.search(cr, uid, args, context=context)
            if promo_ids:
                self.unlink(cr, uid, promo_ids, context=context)

            self.create(cr, uid, dict_promo_ids.get(promo_id), context=context)

        promo2.close()
        os.remove(path_file + 'st_promo2p.' + str(loja).zfill(3))
        return True, '', len(dict_promo_ids)

    def get_promo3(self, cr, uid, promo3p, promo3f, loja, path_file, context=None):
        if not promo3p or not promo3f:
            return True, '', 0
        linhas_promo3f = promo3f.readlines()
        dict_promo_ids = {}
        for linha in promo3p.readlines():
            data_inicio = datetime(int(linha[36:40]), int(linha[34:36]), int(linha[32:34]),
                                   int(linha[40:42]), int(linha[42:44]), int(linha[44:46]))
            data_fim = datetime(int(linha[50:54]), int(linha[48:50]), int(linha[46:48]),
                                int(linha[54:56]), int(linha[56:58]), int(linha[58:60]))
            try:
                maximo = int(linha[60:70])/1000.0
            except:
                maximo = 9999999

            id_promo = int(linha[0:6])
            data_inicio = self.get_date_timezone(cr, uid, data_inicio, tz_destino='UTC')
            data_fim = self.get_date_timezone(cr, uid, data_fim, tz_destino='UTC')
            if not id_promo in dict_promo_ids:
                dict_promo_ids[id_promo] = {'supproc_data_hora_inicio': data_inicio,
                                            'supproc_data_hora_fim': data_fim,
                                            'supproc_id_promo': id_promo,
                                            'supproc_tipo_promo': '3',
                                            'supproc_maximo': maximo,
                                            'supproc_scanntech': True,
                                            'supproc_promo3p_ids': [],
                                            'supproc_promo3f_ids': [],
                                            'state': self._state_ativada
                                           }
            op = 0
            dict_promo_ids[id_promo]['supproc_promo3p_ids'].\
                         append((op, False, {'suppro3p_barras': linha[8:22],
                                             'suppro3p_quantidade': float(linha[22:32])/1000.0,
                                             'suppro3p_sequencial': int(linha[6:8])}
                               ))
            if dict_promo_ids.get(id_promo).get('supproc_promo3f_ids'):
                continue

            for linhaf in linhas_promo3f:
                if id_promo != int(linhaf[0:6]):
                    continue

                op = 0
                supproc_promo3f_ids = [(op, False, {'suppro3f_barras': linhaf[6:20],
                                                    'suppro3f_quantidade': float(linhaf[20:30])/1000.0,
                                                    'suppro3f_desconto': float(linhaf[30:40])/100.0,
                                                    'suppro3f_tipo_desconto': linhaf[40:41]}
                                       )]
                dict_promo_ids[id_promo]['supproc_promo3f_ids'].extend(supproc_promo3f_ids)

        for promo_id in dict_promo_ids:
            args = [('supproc_id_promo', '=', promo_id),
                    ('supproc_scanntech', '=', True)]
            promo_ids = self.search(cr, uid, args, context=context)
            if promo_ids:
                self.unlink(cr, uid, promo_ids, context=context)

            self.create(cr, uid, dict_promo_ids.get(promo_id), context=context)

        promo3p.close()
        promo3f.close()
        os.remove(path_file + 'st_promo3p.' + str(loja).zfill(3))
        os.remove(path_file + 'st_promo3f.' + str(loja).zfill(3))
        return True, '', len(dict_promo_ids)

    def get_promo4(self, cr, uid, promo4, loja, path_file, context=None):
        if not promo4:
            return True, '', 0

        dict_promo_ids = {}
        for linha in promo4.readlines():
            data_inicio = datetime(int(linha[38:42]), int(linha[36:38]), int(linha[34:36]),
                                   int(linha[42:44]), int(linha[44:46]), int(linha[46:48]))
            data_fim = datetime(int(linha[52:56]), int(linha[50:52]), int(linha[48:50]),
                                int(linha[56:58]), int(linha[58:60]), int(linha[60:62]))
            try:
                maximo = int(linha[70:80])/1000.0
            except:
                maximo = 9999999

            id_promo = int(linha[62:68])
            data_inicio = self.get_date_timezone(cr, uid, data_inicio, tz_destino='UTC')
            data_fim = self.get_date_timezone(cr, uid, data_fim, tz_destino='UTC')
            if not id_promo in dict_promo_ids:
                dict_promo_ids[id_promo] = {'supproc_data_hora_inicio': data_inicio,
                                            'supproc_data_hora_fim': data_fim,
                                            'supproc_id_promo': id_promo,
                                            'supproc_tipo_promo': '4',
                                            'supproc_maximo': maximo,
                                            'supproc_scanntech': True,
                                            'supproc_promo4_ids': [],
                                            'state': self._state_ativada
                }
            op = 0
            try:
                sequencial = int(linha[68:70])
            except:
                sequencial = 1

            dict_promo_ids[id_promo]['supproc_promo4_ids'].\
                append((op, False, {'suppro4_barras': linha[0:14],
                                    'suppro4_quantidade': float(linha[14:24])/1000.0,
                                    'suppro4_preco': float(linha[24:34])/100.0,
                                    'suppro4_sequencial': sequencial}
                      ))

        new_promo_ids = []
        for promo_id in dict_promo_ids:
            args = [('supproc_id_promo', '=', promo_id),
                    ('supproc_scanntech', '=', True)]
            promo_ids = self.search(cr, uid, args, context=context)
            if promo_ids:
                self.unlink(cr, uid, promo_ids, context=context)

            new_promo_ids.append(self.create(cr, uid, dict_promo_ids.get(promo_id),
                                             context=context))

        promo4.close()
        os.remove(path_file + 'st_promo4.' + str(loja).zfill(3))
        if not new_promo_ids:
            return True, '', 0

        msg = []
        msg1 = []
        for i in self.browse(cr, uid, new_promo_ids, context):
            context['suppro_mostar_msg'] = True
            res = self.verificar_valor_itens(cr, uid, i.supproc_promo4_ids, context)
            if res:
                if isinstance(res, list):
                    msg1.extend(res)
                else:
                    msg.append(_('%s - Promocao Id: %s') % (i.supproc_nome_promo,
                                                            i.supproc_id_promo))

        txt = ''
        if msg or msg1:
            if msg1:
                txt = '\n'.join(msg1)

            if msg:
                txt += '\n' + _('As seguintes promocoes do tipo 4 estao com valor maior '
                                'que a soma dos itens: \n%s') % '\n'.join(msg)

        return True, txt, len(new_promo_ids)

    def get_promo5(self, cr, uid, promo5p, promo5f, loja, path_file, context=None):
        if not promo5p or not promo5f:
            return True, '', 0

        linhas_promo5f = promo5f.readlines()
        dict_promo_ids = {}
        for linha in promo5p.readlines():
            data_inicio = datetime(int(linha[36:40]), int(linha[34:36]), int(linha[32:34]),
                                   int(linha[40:42]), int(linha[42:44]), int(linha[44:46]))
            data_fim = datetime(int(linha[50:54]), int(linha[48:50]), int(linha[46:48]),
                                int(linha[54:56]), int(linha[56:58]), int(linha[58:60]))
            try:
                maximo = int(linha[60:70])/1000.0
            except:
                maximo = 9999999

            id_promo = int(linha[0:6])
            data_inicio = self.get_date_timezone(cr, uid, data_inicio, tz_destino='UTC')
            data_fim = self.get_date_timezone(cr, uid, data_fim, tz_destino='UTC')
            if not id_promo in dict_promo_ids:
                dict_promo_ids[id_promo] = {'supproc_data_hora_inicio': data_inicio,
                                            'supproc_data_hora_fim': data_fim,
                                            'supproc_id_promo': id_promo,
                                            'supproc_tipo_promo': '5',
                                            'supproc_maximo': maximo,
                                            'supproc_scanntech': True,
                                            'supproc_promo5p_ids': [],
                                            'supproc_promo5f_ids': [],
                                            'state': self._state_ativada
                                            }
            op = 0
            dict_promo_ids[id_promo]['supproc_promo5p_ids'].\
                append((op, False, {'suppro5p_barras': linha[8:22],
                                    'suppro5p_quantidade': float(linha[22:32])/1000.0,
                                    'suppro5p_sequencial': int(linha[6:8])}
                       ))
            if dict_promo_ids[id_promo]['supproc_promo5f_ids']:
                continue

            for linhaf in linhas_promo5f:
                if id_promo != int(linhaf[0:6]):
                    continue

                op = 0
                supproc_promo5f_ids = [(op, False, {'suppro5f_barras': linhaf[6:20],
                                                    'suppro5f_quantidade': float(linhaf[20:30])/1000.0}
                                               )]
                dict_promo_ids[id_promo]['supproc_promo5f_ids'].extend(supproc_promo5f_ids)

        for promo_id in dict_promo_ids:
            args = [('supproc_id_promo', '=', promo_id),
                    ('supproc_scanntech', '=', True)]
            promo_ids = self.search(cr, uid, args, context=context)
            if promo_ids:
                self.unlink(cr, uid, promo_ids, context=context)

            self.create(cr, uid, dict_promo_ids.get(id_promo), context=context)

        promo5p.close()
        promo5f.close()
        os.remove(path_file + 'st_promo5p.' + str(loja).zfill(3))
        os.remove(path_file + 'st_promo5f.' + str(loja).zfill(3))
        return True, '', len(dict_promo_ids)

    def get_promo6(self, cr, uid, promo6, loja, path_file, context=None):
        if not promo6:
            return True, '', 0

        dict_promo_ids = {}
        for linha in promo6.readlines():
            data_inicio = datetime(int(linha[44:48]), int(linha[42:44]), int(linha[40:42]),
                                   int(linha[48:50]), int(linha[50:52]), int(linha[52:54]))
            data_fim = datetime(int(linha[58:62]), int(linha[56:58]), int(linha[54:56]),
                                int(linha[62:64]), int(linha[64:66]), int(linha[66:68]))
            try:
                maximo = int(linha[70:80])/1000.0
            except:
                maximo = 9999999

            try:
                sequencial = int(linha[68:70])
            except:
                sequencial = 1

            id_promo = int(linha[0:6])
            data_inicio = self.get_date_timezone(cr, uid, data_inicio, tz_destino='UTC')
            data_fim = self.get_date_timezone(cr, uid, data_fim, tz_destino='UTC')
            if not id_promo in dict_promo_ids:
                dict_promo_ids[id_promo] = {'supproc_data_hora_inicio': data_inicio,
                                            'supproc_data_hora_fim': data_fim,
                                            'supproc_id_promo': id_promo,
                                            'supproc_tipo_promo': '6',
                                            'supproc_maximo': maximo,
                                            'supproc_scanntech': True,
                                            'supproc_promo6_ids': [],
                                            'state': self._state_ativada
                                            }
            op = 0
            dict_promo_ids[id_promo]['supproc_promo6_ids']\
                .append((op, False, {'suppro6_barras': linha[6:20],
                                     'suppro6_qtd_leva': float(linha[20:30])/1000.0,
                                     'suppro6_qtd_paga': float(linha[30:40])/1000.0,
                                     'suppro6_sequencial': sequencial}
                        ))

        for promo_id in dict_promo_ids:
            args = [('supproc_id_promo', '=', promo_id),
                    ('supproc_scanntech', '=', True)]
            promo_ids = self.search(cr, uid, args, context=context)
            if promo_ids:
                self.unlink(cr, uid, promo_ids, context=context)

            self.create(cr, uid, dict_promo_ids.get(promo_id), context=context)

        promo6.close()
        os.remove(path_file + 'st_promo6.' + str(loja).zfill(3))
        return True, '', len(dict_promo_ids)

    def importar_promocoes_api(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_sup_parametro = self.pool.get('supervisor.cfg.parametros')
        obj_sup_configuracao = self.pool.get('supervisor.cfg.configuracao')
        conf_ids = obj_sup_configuracao.search(cr, uid, [], context=context)
        loja = obj_sup_configuracao.read(cr, uid, conf_ids, ['supcfc_loja'],
                                               context=context)[0].get('supcfc_loja')
        if not loja:
            print_log('Sem loja configurada!', 'E')
            return True

        par_ids = obj_sup_parametro.search(cr, uid, [], context=context)
        parametros = obj_sup_parametro.read(cr, uid, par_ids, ['supcfp_arq_promocoes',
                                                               'supcfp_exp_automatico'],
                                            context=context)[0]
        path_file = parametros.get('supcfp_arq_promocoes')
        if not path_file:
            print_log('Parametro da Promocao Scanntech nao configurado!', 'E')
            return True

        print_log('Importando promocoes...')

        promo1 = False
        promo2 = False
        promo3f = False
        promo3p = False
        promo4 = False
        promo5f = False
        promo5p = False
        promo6 = False

        try:
            promo1 = open(path_file + 'st_promo1.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass
        try:
            promo2 = open(path_file + 'st_promo2p.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass
        try:
            promo3f = open(path_file + 'st_promo3f.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass
        try:
            promo3p = open(path_file + 'st_promo3p.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass
        try:
            promo4 = open(path_file + 'st_promo4.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass
        try:
            promo5f = open(path_file + 'st_promo5f.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass
        try:
            promo5p = open(path_file + 'st_promo5p.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass
        try:
            promo6 = open(path_file + 'st_promo6.' + str(loja).zfill(3), 'r')
        except Exception as e:
            pass

        teve_arquivo = False
        if promo1 or promo2 or promo3p or promo3f or promo4 or promo5p or promo5f or promo6:
            teve_arquivo = True

        qtd = 0
        txt = ''
        self.get_date_timezone = self.pool.get('supervisor.util').suputi_converte_tz
        res = self.get_promo1(cr, uid, promo1, loja, path_file, context=context)
        qtd += res[2]
        res = self.get_promo2(cr, uid, promo2, loja, path_file, context=context)
        qtd += res[2]
        res = self.get_promo3(cr, uid, promo3p, promo3f, loja, path_file, context=context)
        qtd += res[2]
        res = self.get_promo4(cr, uid, promo4, loja, path_file, context=context)
        qtd += res[2]
        txt += res[1]
        res = self.get_promo5(cr, uid, promo5p, promo5f, loja, path_file, context=context)
        qtd += res[2]
        res = self.get_promo6(cr, uid, promo6, loja, path_file, context=context)
        qtd += res[2]
        if qtd > 0 or txt:
            log_importacao = self.pool.get('supervisor.log.importacao').suplog_cria_log
            obj_sup_util = self.pool.get('supervisor.util')
            nome = '%s -  Promocao Scanntech' % obj_sup_util.suputi_datetime_to_timezone(cr, uid, datetime.now())
            log_importacao(cr, uid, nome, qtd, qtd, txt, True, context=context)

        self.finalizar_promocoes_vencidas(cr, uid, ids, context=context)
        # cria acao pendente para itens(a promocao vai junto)
        if teve_arquivo and parametros.get('supcfp_exp_automatico'):
            self.pool.get('supervisor.mov.acao.pendente').cria_acao_pendente(cr, uid, 'Promocoes',
                                                                             context=context)
        print_log('Fim da importacao de promocoes...')
        return True

    def supproc_button_esboco(self, cr, uid, ids, context=None):
        """
        Volta para esboco a promocao
        """
        context = context and context.copy() or {}
        return self.write(cr, uid, ids, {'state': self._state_esboco},context=context)

    def supproc_get_codigos_barras(self, brw_capa):
        '''
        retorna uma lista com os codigos de barras da promocao
        :return:
        '''
        res = []
        tipo_promo = brw_capa.supproc_tipo_promo
        if tipo_promo == '3':
            for p in brw_capa.supproc_promo3p_ids:
                res.append(p.suppro3p_barras)
            for p in brw_capa.supproc_promo3f_ids:
                res.append(p.suppro3f_barras)
        elif tipo_promo == '5':
            for p in brw_capa.supproc_promo5p_ids:
                res.append(p.suppro5p_barras)
            for p in brw_capa.supproc_promo5f_ids:
                res.append(p.suppro5f_barras)
        else:
            for p in eval('brw_capa.supproc_promo%s_ids' % tipo_promo):
                res.append(eval('p.suppro%s_barras' % tipo_promo))

        return res

    def verificar_cb_sem_item(self, cr, uid, cbs, context=None):
        context = context and context.copy() or {}
        obj_sup_item = self.pool.get('supervisor.cad.itens')
        itens_ids = obj_sup_item.search(cr, uid, [('supite_codigo_item', 'in', cbs)],
                                        context=context)
        if not itens_ids:
            raise osv.except_osv(_('Atencao!'),
                                 _('Itens sem cadastro no supervisor: %s') % ''.join(cbs))

        cbs_cadastrados = map(lambda x: x.get('supite_codigo_item'),
                              obj_sup_item.read(cr, uid, itens_ids, ['supite_codigo_item'],
                                                context))
        msg = list(set(cbs) - set(cbs_cadastrados))
        if msg:
            raise osv.except_osv(_('Atencao!'),
                                 _('Itens sem cadastro no supervisor: %s') % ''.join(msg))

        return True

    def supproc_button_ativar_promocao(self, cr, uid, ids, context=None):
        """
        Ativa a promocao caso nao haja nenhum item da mesma em uma outra promocao ativa
        """
        context = context and context.copy() or {}
        for brw_capa in self.browse(cr, uid, ids, context=context):
            codigos_barras = self.supproc_get_codigos_barras(brw_capa)
            self.verificar_cb_sem_item(cr, uid, codigos_barras, context)
            res =  verifica_cb(cr, brw_capa.id, codigos_barras,
                                                str(brw_capa.supproc_data_hora_inicio),
                                                str(brw_capa.supproc_data_hora_fim),
                                                context=context)
            if res and brw_capa.supproc_tipo_promo != '8':
                str_codigo = '\n'
                for i in res:
                    id_promo = i.keys()[0]
                    str_codigo += _('Promocao') + ' %s: %s' % (id_promo, i.get(id_promo)) + '\n'

                raise osv.except_osv(_("Atencao!"),
                                     _("Os codigos a seguir ja estao em outras promocoes:") +
                                     str_codigo)

            if brw_capa.supproc_tipo_promo == '4':
                if self.verifica_valor_promo(cr, uid, brw_capa.supproc_promo4_ids,
                                                context=context):
                    raise osv.except_osv(_("Atencao!"),
                                _("O preco do pacote deve ser o mesmo para todos os itens!"))

                if self.verificar_valor_itens(cr, uid, brw_capa.supproc_promo4_ids,
                                                 context=context):
                    raise osv.except_osv(_("Atencao!"),
                          _("A soma dos precos dos itens deve ser maior que o valor do pacote!"))

            res = self.restricao_promo_8(cr, uid, brw_capa, context)
            if res:
                raise osv.except_osv(_("Atencao!"), res)

            self.write(cr, uid, brw_capa.id, {'state': self._state_ativada}, context=context)

        # caso a promocao ativadas esteja com data vencidas, ja finaliza a promocao
        self.finalizar_promocoes_vencidas(cr, uid, ids, context=context)
        return True

    def restricao_promo_8(self, cr, uid, brw_capa, context=None):
        # Nao deixa ativar promocoes em que os itens da promo normal esteja com seguencial igual e
        # esteja cadastrado na promo 8 para nao aumentar a complexidade do calculo no frente
        # ex: Item A e Item B cadastrados em uma promo normal e os dois com sequencial 1, a
        # venda de um ou outro terah desconto, nao poderah ter uma promo 8 contendo nenhum dos
        # dois itens, ou se ja exitir, a promo normal nao pode ser ativada
        context = context and context.copy() or {}
        tipo = brw_capa.supproc_tipo_promo
        codigo_barras = []
        if tipo in ['3', '5']:
            for i in eval('brw_capa.supproc_promo%sp_ids' % tipo):
                codigo_barras.append(eval('i.suppro%sp_barras' % tipo))
            for i in eval('brw_capa.supproc_promo%sf_ids' % tipo):
                codigo_barras.append(eval('i.suppro%sf_barras' % tipo))
        else:
            for i in eval('brw_capa.supproc_promo%s_ids' % tipo):
                codigo_barras.append(eval('i.suppro%s_barras' % tipo))

        if tipo == '8':
            for i in PROMOCOES:
                n = i[-1]
                if n == 'p':
                    n = i[-2:]
                elif n == 'f':
                    continue

                obj_itens = self.pool.get(i)
                itens_ids = obj_itens.search(cr, uid, [('suppro%s_barras' % n, 'in', codigo_barras)],
                                             context=context)
                if not itens_ids:
                    continue

                cbs = []
                for i in obj_itens.browse(cr, uid, itens_ids, context=context):
                    brw_capa_consulta = eval('i.suppro%s_capa_id' % n)
                    if brw_capa_consulta.state != self._state_ativada:
                        continue

                    dict_sec = {}
                    for j in eval('brw_capa_consulta.supproc_promo%s_ids' % n):
                        sequencia = eval('j.suppro%s_sequencial' % n)
                        if not dict_sec.get(sequencia):
                            dict_sec[sequencia] = True
                        else:
                            cbs.append('%s - Promo ID: %s' % (eval('i.suppro%s_barras' % n),
                                                              eval('i.suppro%s_capa_id.supproc_id_promo' % n)))
                            break

                if not cbs:
                    continue

                return _('Nao pode haver itens com sequencial igual em outras '
                         'promocoes para ativar a promocao do tipo 8!\n'
                         'Itens:\n\n %s' % '\n'.join(cbs))

        else:

            obj_itens_8 = self.pool.get('supervisor.cad.promo8')
            itens_8_ids = obj_itens_8.search(cr, uid, [('suppro8_barras', 'in', codigo_barras)],
                                             context=context)
            if not itens_8_ids:
                return False

            cbs_8 = []
            for i in obj_itens_8.browse(cr, uid, itens_8_ids, context=context):
                if i.suppro8_capa_id.state != self._state_ativada:
                    continue

                cbs_8.append('%s - Promo ID: %s' % (i.suppro8_barras,
                                                    i.suppro8_capa_id.supproc_id_promo))

            if not cbs_8:
                return False

            cbs_8 = list(set(cbs_8))
            dict_sec = {}
            if tipo in ['3', '5']:
                for i in eval('brw_capa.supproc_promo%sp_ids' % tipo):
                    sequencia = eval('i.suppro%sp_sequencial' % tipo)
                    if not dict_sec.get(sequencia):
                        dict_sec[sequencia] = True
                    else:
                        return _('Nao pode haver itens com sequencial igual quando '
                                 'estiverem participando de promocao  tipo 8!\n'
                                 'Itens em promocao tipo 8:\n\n %s' % '\n'.join(cbs_8))


            else:
                for i in eval('brw_capa.supproc_promo%s_ids' % tipo):
                    sequencia = eval('i.suppro%s_sequencial' % tipo)
                    if not dict_sec.get(sequencia):
                        dict_sec[sequencia] = True
                    else:
                        return _('Nao pode haver itens com sequencial igual quando '
                                 'estiverem participando de promocao  tipo 8!\n'
                                 'Itens em promocao tipo 8:\n\n %s' % '\n'.join(cbs_8))

        return False

    def verifica_valor_promo(self, cr, uid, brw_itens, context=None):
        """
        Retorna True se tem itens com valor da promocao 4 diferentes. False se nao tiver.
        """
        context = context and context.copy() or {}
        preco = False
        for brw_item in brw_itens:
            if not preco:
                preco = brw_item.suppro4_preco
                continue

            if preco != brw_item.suppro4_preco:
                return True

        return False

    def verificar_valor_itens(self, cr, uid, brw_itens, context=None):
        """
        Retorna True se a soma dos itens é menor que o valor do pacote e False se nao for
        """
        context = context and context.copy() or {}
        brw_itens = sorted(brw_itens, key=lambda x: x.suppro4_sequencial)
        obj_sup_item = self.pool.get('supervisor.cad.itens')
        codigo_barras = map(lambda x: x.suppro4_barras, brw_itens)
        itens_ids = obj_sup_item.search(cr, uid,  [('supite_codigo_item', 'in', codigo_barras)],
                                        context=context)
        itens_valor = {}
        map(lambda x: itens_valor.update({x.get('supite_codigo_item'): x.get('supite_preco')}),
                            obj_sup_item.read(cr, uid, itens_ids, context=context))
        sequencia = False
        total_itens = 0
        item_menor_valor = False
        msgs = []
        for brw_item in brw_itens:
            if brw_item.suppro4_barras not in itens_valor:
                nome = ''
                if brw_item.suppro4_descricao:
                    nome = ' - %s' % brw_item.suppro4_descricao
                item = '"{cb}{nome}"'.format(cb=brw_item.suppro4_barras,
                                             nome=nome)
                brw_capa = brw_item.suppro4_capa_id
                nome_promo = ''
                if brw_capa.supproc_nome_promo:
                    nome_promo = ' - %s' % brw_capa.supproc_nome_promo
                promo = '"{cd}{nome}"'.format(cd=brw_capa.supproc_id_promo, nome=nome_promo)
                msg = _('Promocao %s invalida. Item %s nao esta cadastrado!' %
                                       (promo, item))
                if context.get('suppro_mostar_msg'):
                    msgs.append(msg)
                    continue

                raise osv.except_osv(_('Atencao!'), msg)

            if brw_item.suppro4_sequencial != sequencia:
                sequencia = brw_item.suppro4_sequencial
                if item_menor_valor:
                    total_itens += itens_valor.get(item_menor_valor.suppro4_barras) * \
                                   item_menor_valor.suppro4_quantidade

                item_menor_valor = brw_item

            elif itens_valor.get(brw_item.suppro4_barras) * brw_item.suppro4_quantidade < \
                 itens_valor.get(item_menor_valor.suppro4_barras) * \
                 item_menor_valor.suppro4_quantidade:
                item_menor_valor = brw_item

        if msgs:
            return msgs

        # soma o menor item da ultima sequencia
        total_itens += itens_valor.get(item_menor_valor.suppro4_barras) * \
                       item_menor_valor.suppro4_quantidade

        if item_menor_valor.suppro4_preco > total_itens:
            return True

        return False

    def finalizar_promocoes_vencidas(self, cr, uid, ids, context=None):
        '''
        Funcao ativada pelo cron 'cronjob_supervisor_deleta_promocoes'
        Deleta as promocoes vencidas
        '''
        context = context and context.copy() or {}
        data_atual = datetime.now()
        args = [('supproc_data_hora_fim','<', str(data_atual)),
                ('state','=',self._state_ativada)]
        if ids:
            if not isinstance(ids, list):
                args.append(('id', 'in', ids))

        promo_ids = self.search(cr, uid, args, context=context)
        if not promo_ids:
            return True

        return self.write(cr, uid, promo_ids, {'state': self._state_finalizada}, context=context)