#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _
from datetime import datetime


class supervisor_cad_sorteio(osv.Model):

    _inherit = 'supervisor.cad.sorteio'

    def create(self, cr, user, vals, context=None):
        self.supsor_verifica_valor(vals, context)
        return super(supervisor_cad_sorteio, self).create(cr, user, vals, context)

    def write(self, cr, user, ids, vals, context=None):
        if 'supsor_valor' in vals:
            self.supsor_verifica_valor(vals, context)

        return super(supervisor_cad_sorteio, self).write(cr, user, ids, vals, context)

    def button_supsor_esboco(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self.write(cr, uid, ids, {'state': '1'})
        return True

    def button_supsor_ativar(self, cr, uid, ids, context=None):
        """
        Ativa o sorteio 
        """
        context = context and context.copy() or {}
        self.supsor_verifica_cb_repetido(cr, uid, ids, context)
        for dados in self.read(cr, uid, ids, ['supsor_inicio', 'supsor_data_fim', 'supsor_valor'],
                               context=context):
            self.supsor_verifica_datas(cr, uid, dados, context)
            self.supsor_verifica_valor(dados, context)
            self.write(cr, uid, dados.get('id'), {'state': '2'}, context=context)

        return True

    def supsor_verifica_datas(self, cr, uid, dados, onchange=False, context=None):
        context = context and context.copy() or {}
        obj_sup_util = self.pool.get('supervisor.util')
        data_atual = obj_sup_util.suputi_converte_tz(cr, uid,
                                                      datetime.now(), 'UTC', context=context).date()
        msg = []
        if dados.get('supsor_data_fim') and dados.get('supsor_data_inicio') and \
                        dados.get('supsor_data_fim') < dados.get('supsor_data_inicio'):
            msg.append(_('Data final maior que a data inicial!'))

        if dados.get('supsor_data_fim') and dados.get('supsor_data_fim') < str(data_atual):
            msg.append(_('Data final menor que a data atual!'))

        if msg:
            if onchange:
                return {'warning': {'title': _('Atencao!'),
                                    'message': '\n'.join(msg)}}

            raise osv.except_osv(_('Atencao!'), '\n'.join(msg))

        if onchange:
            return {}

        return True

    def supsor_verifica_cb_repetido(self, cr, uid, ids, context):
        context = context and context.copy() or {}
        cbs = []
        for i in self.browse(cr, uid, ids, context):
            for j in i.supsor_item_ids:
                cbs.append(j.supsit_barras)

        args = [('supsit_barras', 'in', cbs)]
        if ids:
            args.append(('id', 'not in', ids))

        obj_itens = self.pool.get('supervisor.cad.sorteio.itens')
        item_ids = obj_itens.search(cr, uid, args, context=context)
        if not item_ids:
            return True

        dict_msg = {}
        for i in obj_itens.browse(cr, uid, item_ids, context):
            brw_capa = i.supsit_capa_id
            if brw_capa.state == '2':
                item = '%s - %s' % (i.supsit_barras, i.supsit_descricao)
                if not dict_msg.get(brw_capa.id):
                    name = brw_capa.name and ('%s ' % brw_capa.name) or ''
                    nome = 'Sorteio: %s%s <-> %s: R$ %s' % (name,
                                                            brw_capa.supsor_data_inicio,
                                                            brw_capa.supsor_data_fim,
                                                            str(brw_capa.supsor_valor).replace('.', ','))
                    dict_msg[brw_capa.id] = {'nome': nome,
                                             'itens': [item]}
                else:
                    dict_msg[brw_capa.id]['itens'].append(item)
                    
        if not dict_msg:
            return True
            
        msg = []
        for i in dict_msg.values():
            msg.append('%s:\n %s' % (i.get('nome') ,'\n'.join(i.get('itens'))))
            
        raise osv.except_osv(_('Atencao!'),
                             _('Os itens a seguir ja estao em outros sorteios:\n%s' % '\n\n'.join(msg)))

    def onchange_supsor_datas(self, cr, uid, ids, data_inicial, data_final, context=None):
        context = context and context.copy() or {}
        dados = {'supsor_data_inicio': data_inicial,
                 'supsor_data_fim': data_final}
        return self.supsor_verifica_datas(cr, uid, dados, context)

    def supsor_verifica_valor(self, dados, context=None):
        context = context and context.copy() or {}
        if dados.get('supsor_valor') <= 0:
            raise osv.except_osv(_('Atencao!'),
                                 _('O campo Valor deve ser maior que 0(zero)!'))

        return True


    def finalizar_promocoes_vencidas(self, cr, uid, ids, context=None):
        '''
        Funcao ativada pelo cron 'cronjob_supervisor_excluir_sorteio'
        Deleta os sorteios vencidps
        '''
        obj_sup_util = self.pool.get('supervisor.util')
        data_atual = obj_sup_util.suputi_converte_tz(cr, uid,
                                                      datetime.now(), 'UTC', context=context).date()
        ids = self.search(cr, uid, [('supsor_data_fim', '<', str(data_atual))], context=context)
        if not ids:
            return True

        self.unlink(cr, uid, ids, context=context)
        return True