
# -*- coding: utf-8 -*-
#
# OBS: para funcionar eh preciso que tenha a pasta 'static' dentro do modulo

import json
import os
import socket
import xmlrpclib

import openerp.addons.web.http as http
from openerp.tools import config as configERP
import ConfigParser

SEM_ERRO = '00'
ERRO_CONEXAO = '01'
ERRO_INTERNO = '99'


class Config(object):
    def __init__(self, modulo):
        my_file = configERP.get('addons_path') + '/' + modulo + '/app.config'
        if not os.path.exists(my_file):
            raise Exception('Arquivo de configuracao "app.config" nao existe '
                            'no modulo "%s"' % modulo)
        c = ConfigParser.ConfigParser()
        c.read(my_file)
        self.dados = c
        
        
class MySock(object):
    def __init__(self, params, modulo=''):
        super(MySock, self).__init__()
        if not (type(params) is dict):
            raise Exception('O Parametro informado para "MySock" deve ser um dicionario!')

        self.uid = int(params.get('uid') or 0)
        self.pwd = params.get('pwd') or ''
        if not modulo:
            self.db = params.get('db')
        else:
            cnpj = params.get('cnpj')
            config = Config(modulo)
            self.db = config.dados.get(cnpj, 'database')
            if not self.db:
                raise Exception('Nenhum banco configurado no arquivo "app.config" para o CNPJ: %s' % cnpj)

        porta = configERP.get('xmlrpc_port') or 8069
        #  se for passado o uid nao precisa logar novamente
        if not self.uid:
            sock_common = xmlrpclib.ServerProxy('http://localhost:%s' % porta + '/xmlrpc/common')
            socket.setdefaulttimeout(3)
            self.uid = sock_common.login(self.db, params.get('user'), self.pwd)

        self.sock = xmlrpclib.ServerProxy('http://localhost:%s/xmlrpc/object' % porta)
        socket.setdefaulttimeout(600)

    def execute(self, model, function, *args):
        try:
            return self.sock.execute(self.db, self.uid, self.pwd, 'valida.utils',
                                     'valuti_get_funcao', model, function, *args)
        except Exception as e:
            if type(e) == xmlrpclib.Fault:
                msg_erro = e.faultString
                if e.faultCode == 'AccessDenied':
                    msg_erro = 'Erro: %s' % "Acesso ao ERP negado para este Usuário e Senha!"
                elif '9999\n' in e.faultCode:
                    msg_erro = e.faultCode.split('9999\n')[1]
                raise Exception(msg_erro)

            raise e


from logging import getLogger

_logger = getLogger("SigErp")



class sc_valida(http.Controller):
    _cp_path = '/valida'

    @http.httprequest
    def teste(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            if req.params:
                sock = MySock(req.params)
                sock.execute('estoque.produto', 'search', [], 0, 1)

            ip = req.httprequest.environ.get('REMOTE_ADDR')
            print(ip)
            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': 'ok',
                               'retorno': [ip]})

        except Exception as e:
            msg = 'Erro: %s' % e
            if e.__dict__.get('faultCode'):
                msg = e.__dict__.get('faultCode')

            return json.dumps({'codigo': ERRO_CONEXAO,
                               'mensagem': msg,
                               'retorno': [],
                               })

    @http.httprequest
    def verificar(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        return 'Ok'

    @http.httprequest
    def controller_gerar_licensa(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            sock = MySock(req.params)
            dados = eval(req.params.get('dados'))
            res = sock.execute('financeiro.cliente', 'fincli_gerar_licensa', dados.get('cnpj'), dados.get('ip'))
            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': 'ok',
                               'retorno': [res]})

        except Exception as e:
            return json.dumps({'codigo': ERRO_CONEXAO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': [],
                               })

    @http.httprequest
    def controller_verificar_rede_sig(self, req, s_action=None, **kw):
        _logger.info('==++> SigController.%s Params: %s. Data:%s' % (req.httprequest.path, req.params, req.httprequest.data))
        try:
            ip = req.httprequest.environ.get('REMOTE_ADDR')
            return json.dumps({'codigo': SEM_ERRO,
                               'mensagem': 'ok',
                               'retorno': [ip]})

        except Exception as e:
            return json.dumps({'codigo': ERRO_CONEXAO,
                               'mensagem': 'Erro: %s' % e,
                               'retorno': [],
                               })